#include "config.h"
#include "hebrcal.h"
#include "anni.h"

#define IDMONTH_NISAN            1
#define IDMONTH_IYYAR            2
#define IDMONTH_SIVAN            3
#define IDMONTH_TAMMUZ           4
#define IDMONTH_AV               5
#define IDMONTH_ELUL             6
#define IDMONTH_TISHRI           7
#define IDMONTH_HESHVAN          8
#define IDMONTH_KISLEV           9
#define IDMONTH_TEVETH          10
#define IDMONTH_SHEVAT          11
#define IDMONTH_ADAR            12
#define IDMONTH_ADARI           12
#define IDMONTH_ADARII          13

void AnniBirth(AnniStruct* anni)
{
  CalDate dateHebrew;
  long absDate;
  int iLastMonth;

  iLastMonth = HebrcalGetLastMonthOfYear(anni->anniDate.year);
  if (anni->anniDate.month == iLastMonth)
  {
    dateHebrew.day   = anni->anniDate.day;
    dateHebrew.month = HebrcalGetLastMonthOfYear
                          (anni->inHebrewYear);
    dateHebrew.year  = anni->inHebrewYear;
    absDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    HebrcalForeignFromAbsolute(absDate, &dateHebrew);
    anni->anniDate2.day   = dateHebrew.day;
    anni->anniDate2.month = dateHebrew.month;
    anni->anniDate2.year  = dateHebrew.year;
  }
  else
  {
    dateHebrew.day   = anni->anniDate.day;
    dateHebrew.month = anni->anniDate.month;
    dateHebrew.year  = anni->inHebrewYear;
    absDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    HebrcalForeignFromAbsolute(absDate, &dateHebrew);
    anni->anniDate2.day   = dateHebrew.day;
    anni->anniDate2.month = dateHebrew.month;
    anni->anniDate2.year  = dateHebrew.year;
  }
}

void AnniDeath(AnniStruct* anni)
{
  CalDate dateHebrew;
  long absDate, lStartDate, lEndDate;

  if ((anni->anniDate.day == 30) &&
      (anni->anniDate.month == IDMONTH_HESHVAN))
  {
    dateHebrew.day   = 1;
    dateHebrew.month = IDMONTH_HESHVAN;
    dateHebrew.year  = anni->anniDate.year+1;
    lStartDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    dateHebrew.day   = 1;
    dateHebrew.month = IDMONTH_HESHVAN+1;
    dateHebrew.year  = anni->anniDate.year+1;
    lEndDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    if ((lEndDate-lStartDate) < 30)
    {
      dateHebrew.day   = 1;
      dateHebrew.month = IDMONTH_KISLEV;
      dateHebrew.year  = anni->inHebrewYear;
      absDate = HebrcalAbsoluteFromForeign(&dateHebrew);
      absDate--;
      HebrcalForeignFromAbsolute(absDate, &dateHebrew);
      anni->anniDate2.day   = dateHebrew.day;
      anni->anniDate2.month = dateHebrew.month;
      anni->anniDate2.year  = dateHebrew.year;
    }
    else
    {
      dateHebrew.day   = anni->anniDate.day;
      dateHebrew.month = anni->anniDate.month;
      dateHebrew.year  = anni->inHebrewYear;
      absDate = HebrcalAbsoluteFromForeign(&dateHebrew);
      HebrcalForeignFromAbsolute(absDate, &dateHebrew);
      anni->anniDate2.day   = dateHebrew.day;
      anni->anniDate2.month = dateHebrew.month;
      anni->anniDate2.year  = dateHebrew.year;
    }
    return;
  }

  if ((anni->anniDate.day == 30) &&
      (anni->anniDate.month == IDMONTH_KISLEV))
  {
    dateHebrew.day   = 1;
    dateHebrew.month = IDMONTH_KISLEV;
    dateHebrew.year  = anni->anniDate.year+1;
    lStartDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    dateHebrew.day   = 1;
    dateHebrew.month = IDMONTH_KISLEV+1;
    dateHebrew.year  = anni->anniDate.year+1;
    lEndDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    if ((lEndDate-lStartDate) < 30)
    {
      dateHebrew.day   = 1;
      dateHebrew.month = IDMONTH_TEVETH;
      dateHebrew.year  = anni->inHebrewYear;
      absDate = HebrcalAbsoluteFromForeign(&dateHebrew);
      absDate--;
      HebrcalForeignFromAbsolute(absDate, &dateHebrew);
      anni->anniDate2.day   = dateHebrew.day;
      anni->anniDate2.month = dateHebrew.month;
      anni->anniDate2.year  = dateHebrew.year;
    }
    else
    {
      dateHebrew.day   = anni->anniDate.day;
      dateHebrew.month = anni->anniDate.month;
      dateHebrew.year  = anni->inHebrewYear;
      absDate = HebrcalAbsoluteFromForeign(&dateHebrew);
      HebrcalForeignFromAbsolute(absDate, &dateHebrew);
      anni->anniDate2.day   = dateHebrew.day;
      anni->anniDate2.month = dateHebrew.month;
      anni->anniDate2.year  = dateHebrew.year;
    }
    return;
  }

  if (anni->anniDate.month == 13)
  {
    anni->anniDate2.day   = anni->anniDate.day;
    anni->anniDate2.month = HebrcalGetLastMonthOfYear
                                 (anni->inHebrewYear);
    anni->anniDate2.year  = anni->inHebrewYear;

    return;
  }

  if ((anni->anniDate.day == 30) &&
      (anni->anniDate.month == IDMONTH_ADARI))
  {
    dateHebrew.day   = 1;
    dateHebrew.month = IDMONTH_TISHRI;
    dateHebrew.year  = anni->inHebrewYear;
    lStartDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    dateHebrew.day   = 1;
    dateHebrew.month = IDMONTH_TISHRI;
    dateHebrew.year  = anni->inHebrewYear+1;
    lEndDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    if ((lEndDate-lStartDate) < 380)
    {
      anni->anniDate2.day   = 30;
      anni->anniDate2.month = IDMONTH_SHEVAT;
      anni->anniDate2.year  = anni->inHebrewYear;
    }
    else
    {
      anni->anniDate2.day   = anni->anniDate.day;
      anni->anniDate2.month = anni->anniDate.month;
      anni->anniDate2.year  = anni->inHebrewYear;
    }
    return;
  }

  dateHebrew.day   = anni->anniDate.day;
  dateHebrew.month = anni->anniDate.month;
  dateHebrew.year  = anni->inHebrewYear;
  absDate = HebrcalAbsoluteFromForeign(&dateHebrew);
  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  anni->anniDate2.day   = dateHebrew.day;
  anni->anniDate2.month = dateHebrew.month;
  anni->anniDate2.year  = dateHebrew.year;
}

void AnniGetAnniversary(AnniStruct* anni)
{
  switch (anni->chAnniType)
  {
    case ANNITYPE_BIRTH:
    case ANNITYPE_ANNIVERSARY:
      AnniBirth(anni);
      break;

    case ANNITYPE_DEATH:
      AnniDeath(anni);
      break;
  }
}

