#include <gtk/gtk.h>
#include "config.h"
#include "dialogs.h"
#include "names.h"
#include "gregor.h"
#include "hebrcal.h"
#include "anni.h"
#include "annifile.h"
#include "annidlg.h"

#define HYEAR_BEGIN         5360
#define HYEAR_END           5860

#define ANNI_TITLE          "Anniversaries"
#define ANNI_LABEL          "Annivers./Birthd./" YAHRZEIT_ABBREV

#define BUTTON_ADD          "Add..."
#define BUTTON_MODIFY       "Modify..."
#define BUTTON_REMOVE       "Remove..."

#define LABEL_CALCULATEFROM "From"
#define LABEL_TO            "to"

#define BUTTON_CALCULATE    "Calc"

#define BUTTON_CLOSE        "Close"

#define STRING_TYPEANNI         "Anniv."
#define STRING_TYPEBIRTHDAY     "Birthd."
#define STRING_TYPEYAHRZEIT     YAHRZEIT_ABBREV

int NumberIsInteger(char *text); /* defined in shabconf.c */

int idAnniClickedButton;

void AnniDialogButton(GtkWidget* widget, gpointer data)
{
  idAnniClickedButton = (int) data;
}

void AnniFillTheList(GtkList* glist, GList** list, AnniFile** af)
{
  struct tm *currentTM;
  time_t currentTime;
  CalDate date, dateCurrentHebrew, dateAnni, dateGregorian;
  long currentDate, absDate;
  int iCount, i, isHebrewDate, afterSunset;
  char name[256], type, monthGregorian[256], monthHebrew[256], text[256];
  AnniStruct anni;
  GtkListItem *listItem;

  currentTime = time(NULL);
  currentTM = localtime(&currentTime);

  date.day = currentTM->tm_mday;
  date.month = currentTM->tm_mon + 1;
  date.year = currentTM->tm_year + 1900;
  currentDate = GregorAbsoluteFromForeign(&date);
  HebrcalForeignFromAbsolute(currentDate, &dateCurrentHebrew);

  if (*list)
  {
    gtk_list_remove_items(glist, *list);
    *list = NULL;
  }

  iCount = AnniFileCount(af);
  for (i = 0; i < iCount; i++)
  {
    AnniFileGet(af, i, name, &type, &isHebrewDate, &date, &afterSunset);
    if (isHebrewDate)
      absDate = HebrcalAbsoluteFromForeign(&date);
    else
    {
      absDate = GregorAbsoluteFromForeign(&date);
      if (afterSunset) absDate++;
    }
    HebrcalForeignFromAbsolute(absDate, &dateAnni);
    if (absDate < currentDate)
    {
      memcpy(&anni.anniDate, &dateAnni, sizeof(CalDate));
      anni.chAnniType = type;
      anni.inHebrewYear = dateCurrentHebrew.year;
      AnniGetAnniversary(&anni);
      absDate = HebrcalAbsoluteFromForeign(&anni.anniDate2);
      if (absDate < currentDate)
      {
        memcpy(&anni.anniDate, &dateAnni, sizeof(CalDate));
        anni.chAnniType = type;
        anni.inHebrewYear = dateCurrentHebrew.year+1;
        AnniGetAnniversary(&anni);
      }
      GregorForeignFromAbsolute(absDate, &dateGregorian);
      GetGregorianMonthName(dateGregorian.month, monthGregorian);
      GetHebrewMonthName(anni.anniDate2.month, anni.anniDate2.year,
                         monthHebrew);
      sprintf(text, "%s (%c): %d %s %d (%d %s %d)", 
              name, type,
              dateGregorian.day, monthGregorian, dateGregorian.year,
              anni.anniDate2.day, monthHebrew, anni.anniDate2.year);
    }
    else
      strcpy(text, name);
    listItem = GTK_LIST_ITEM(gtk_list_item_new_with_label(text));
    gtk_widget_show(GTK_WIDGET(listItem));
    *list = g_list_append(*list, listItem);
  }
  gtk_list_append_items(glist, *list);
}

void AnniDoCalculate(GtkList* glist, GList** list, AnniFile** af, 
                     int index, int yearFrom, int yearTo)
{
  char name[256], type, monthGregorian[256], monthHebrew[256], text[256];
  CalDate date, dateAnni, dateStart, dateEnd, dateStartHebrew, dateEndHebrew;
  CalDate dateGregorian;
  int isHebrewDate, afterSunset;
  long absDate, startDate, endDate, hebrewYear;
  AnniStruct anni;
  GtkListItem *listItem;

  if (*list)
  {
    gtk_list_remove_items(glist, *list);
    *list = NULL;
  }

  if (index == (-1))
    return;

  AnniFileGet(af, index, name, &type, &isHebrewDate, &date, &afterSunset);
  if (isHebrewDate)
    absDate = HebrcalAbsoluteFromForeign(&date);
  else
  {  
    absDate = GregorAbsoluteFromForeign(&date);
    if (afterSunset) absDate++;
  }
  HebrcalForeignFromAbsolute(absDate, &dateAnni);

  dateStart.day = 1;
  dateStart.month = 1;
  dateStart.year = yearFrom;
  startDate = GregorAbsoluteFromForeign(&dateStart);
  HebrcalForeignFromAbsolute(startDate, &dateStartHebrew);
  dateEnd.day = 31;
  dateEnd.month = 12;
  dateEnd.year = yearTo;
  endDate = GregorAbsoluteFromForeign(&dateEnd);
  HebrcalForeignFromAbsolute(endDate, &dateEndHebrew);

  for (hebrewYear = dateStartHebrew.year;
       hebrewYear <= dateEndHebrew.year; 
       hebrewYear++)
  {
    if (hebrewYear > dateAnni.year)
    {
      memcpy(&anni.anniDate, &dateAnni, sizeof(CalDate));
      anni.chAnniType = type;
      anni.inHebrewYear = hebrewYear;
      AnniGetAnniversary(&anni);
      absDate = HebrcalAbsoluteFromForeign(&anni.anniDate2);
      GregorForeignFromAbsolute(absDate, &dateGregorian);
      if ((dateGregorian.year >= yearFrom) &&
          (dateGregorian.year <= yearTo))
      {
        GetGregorianMonthName(dateGregorian.month, monthGregorian);
        GetHebrewMonthName(anni.anniDate2.month, anni.anniDate2.year, monthHebrew);
        sprintf(text, "%d %s %d (%d %s %d)",
                dateGregorian.day, monthGregorian, dateGregorian.year,
                anni.anniDate2.day, monthHebrew, anni.anniDate2.year);
        listItem = GTK_LIST_ITEM(gtk_list_item_new_with_label(text));
        gtk_widget_show(GTK_WIDGET(listItem));
        *list = g_list_append(*list, listItem);
      }
    }
  }

  gtk_list_append_items(glist, *list);
}

int AnniGetCurrentItem(GtkList *list)
{
  GList* selectedItemList;
  gpointer selectedItemData;

  selectedItemList = g_list_nth(list->selection, 0);
  if (selectedItemList)
  {
    selectedItemData = selectedItemList->data;
    if (selectedItemData)
      return g_list_index(list->children, selectedItemData);
    else
      return (-1);
  }
  else
    return (-1);
}

AnniFile *af;
GtkList *listCalculated, *listAnni;
GList *glistCalculate;
GtkEntry *entryFrom, *entryTo;

void Recalculate(int doMessage)
{
  int iSelected, fromYear, toYear, h, ok;
  char *text2, *endptr;

  AnniDoCalculate(listCalculated, &glistCalculate, &af,
                  (-1), 0, 0);
  iSelected = AnniGetCurrentItem(listAnni);
  if (iSelected != (-1))
  {
    text2 = gtk_entry_get_text(entryFrom);
    fromYear = strtol(text2, &endptr, 10);
    if (endptr)
    {
      if (*endptr == 0)
        ok = 1;
      else
        ok = 0;
    }
    else
      ok = 1;
    if (ok)
    {
      if ((fromYear >= YEAR_BEGIN) && (fromYear <= YEAR_END))
      {
        text2 = gtk_entry_get_text(entryTo);
        toYear = strtol(text2, &endptr, 10);
        if (endptr)
        {
          if (*endptr == 0)
            ok = 1;
          else
            ok = 0;
        }
        else
          ok = 1;
        if (ok)
        {
          if ((toYear >= YEAR_BEGIN) && (toYear <= YEAR_END))
          {
            if (fromYear > toYear)
            {
              h = fromYear;
              fromYear = toYear;
              toYear = h;
            }
            AnniDoCalculate(listCalculated, &glistCalculate, &af,
                            iSelected, fromYear, toYear);
          }
          else
          {
            if (doMessage) DialogsDoMessage("'To' year is not in the correct range");
          }
        }
        else
        {
          if (doMessage) DialogsDoMessage("'To' year is not a number");
        }
      }
      else
      {
        if (doMessage) DialogsDoMessage("'From' year is not in the correct range");
      } 
    }
    else
    {
      if (doMessage) DialogsDoMessage("'From' year is not a number");
    } 
  }
  else
  {
    if (doMessage) DialogsDoMessage("An entry must be selected");
  }
}

void AnniDlgSelectionChanged(GtkWidget *widget, gpointer data)
{
  Recalculate(0);
}

#define ID_ANNI_ADD       1
#define ID_ANNI_MODIFY    2
#define ID_ANNI_REMOVE    3
#define ID_ANNI_CALCULATE 4
#define ID_ANNI_CLOSE     5

void AnniDoDialog(void)
{
  struct tm *currentTM;
  time_t currentTime;
  GList *glistAnni;
  GtkDialog *dialog;
  GtkLabel *label;
  GtkHBox *hbox;
  GtkScrolledWindow *scrolledWindow;
  GtkButton *buttonAdd, *buttonModify, *buttonRemove, *buttonCalculate;
  GtkButton *buttonClose;
  char name[256], type, text[256], msg[256];
  CalDate date;
  int iSelected, isHebrewDate, afterSunset;

  currentTime = time(NULL);
  currentTM = localtime(&currentTime);

  glistAnni = NULL;
  glistCalculate = NULL;

  AnniFileLoad(&af);
  
  dialog = GTK_DIALOG(gtk_dialog_new());
  gtk_window_set_title(GTK_WINDOW(dialog), ANNI_TITLE);

  label = GTK_LABEL(gtk_label_new(ANNI_LABEL));
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));

  listAnni = GTK_LIST(gtk_list_new());
  gtk_signal_connect(GTK_OBJECT(listAnni), "selection-changed",
                     GTK_SIGNAL_FUNC(AnniDlgSelectionChanged), NULL);
  scrolledWindow = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
  gtk_scrolled_window_add_with_viewport(scrolledWindow, GTK_WIDGET(listAnni));
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(scrolledWindow));
  AnniFillTheList(listAnni, &glistAnni, &af);
  gtk_widget_show(GTK_WIDGET(listAnni));
  gtk_widget_show(GTK_WIDGET(scrolledWindow));

  buttonAdd = GTK_BUTTON(gtk_button_new_with_label(BUTTON_ADD));
  gtk_signal_connect(GTK_OBJECT(buttonAdd), "clicked",
                     GTK_SIGNAL_FUNC(AnniDialogButton), (gpointer) ID_ANNI_ADD);
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(buttonAdd));
  gtk_widget_show(GTK_WIDGET(buttonAdd));

  buttonModify = GTK_BUTTON(gtk_button_new_with_label(BUTTON_MODIFY));
  gtk_signal_connect(GTK_OBJECT(buttonModify), "clicked",
                     GTK_SIGNAL_FUNC(AnniDialogButton), (gpointer) ID_ANNI_MODIFY);
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(buttonModify));
  gtk_widget_show(GTK_WIDGET(buttonModify));

  buttonRemove = GTK_BUTTON(gtk_button_new_with_label(BUTTON_REMOVE));
  gtk_signal_connect(GTK_OBJECT(buttonRemove), "clicked",
                     GTK_SIGNAL_FUNC(AnniDialogButton), (gpointer) ID_ANNI_REMOVE);
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(buttonRemove));
  gtk_widget_show(GTK_WIDGET(buttonRemove));

  hbox = GTK_HBOX(gtk_hbox_new(FALSE, 1));

  label = GTK_LABEL(gtk_label_new(LABEL_CALCULATEFROM));
  gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));
  entryFrom = GTK_ENTRY(gtk_entry_new());
  sprintf(text, "%d", (currentTM->tm_year+1900));
  gtk_entry_set_text(entryFrom, text);
  gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(entryFrom));
  gtk_widget_show(GTK_WIDGET(entryFrom));
  
  gtk_widget_show(GTK_WIDGET(hbox));
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(hbox));

  hbox = GTK_HBOX(gtk_hbox_new(FALSE, 1));

  label = GTK_LABEL(gtk_label_new(LABEL_TO));
  gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));
  entryTo = GTK_ENTRY(gtk_entry_new());
  sprintf(text, "%d", (currentTM->tm_year+1900+5));
  gtk_entry_set_text(entryTo, text);
  gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(entryTo));
  gtk_widget_show(GTK_WIDGET(entryTo));

  buttonCalculate = GTK_BUTTON(gtk_button_new_with_label(BUTTON_CALCULATE));
  gtk_signal_connect(GTK_OBJECT(buttonCalculate), "clicked",
                     GTK_SIGNAL_FUNC(AnniDialogButton), (gpointer) ID_ANNI_CALCULATE);
  gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(buttonCalculate));
  gtk_widget_show(GTK_WIDGET(buttonCalculate));

  gtk_widget_show(GTK_WIDGET(hbox));
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(hbox));

  listCalculated = GTK_LIST(gtk_list_new());
  scrolledWindow = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
  gtk_scrolled_window_add_with_viewport(scrolledWindow, GTK_WIDGET(listCalculated));
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(scrolledWindow));
  gtk_widget_show(GTK_WIDGET(listCalculated));
  gtk_widget_show(GTK_WIDGET(scrolledWindow));

  buttonClose = GTK_BUTTON(gtk_button_new_with_label(BUTTON_CLOSE));
  gtk_signal_connect(GTK_OBJECT(buttonClose), "clicked",
                     GTK_SIGNAL_FUNC(AnniDialogButton), (gpointer) ID_ANNI_CLOSE);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                     GTK_WIDGET(buttonClose), TRUE, TRUE, 0);
  gtk_widget_show(GTK_WIDGET(buttonClose));
  gtk_widget_show(GTK_WIDGET(dialog));

  gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
                     GTK_SIGNAL_FUNC(AnniDialogButton), (gpointer) ID_ANNI_CLOSE);

  for (;;)
  {
    idAnniClickedButton = (-1);
    while (idAnniClickedButton == (-1))
    {
/*    if (gtk_events_pending())*/
        gtk_main_iteration();
    }
    if (idAnniClickedButton == ID_ANNI_CLOSE)
      break;

    switch (idAnniClickedButton)
    {
      case ID_ANNI_ADD:
        {
          strcpy(name, "");
          type = 'A';
          isHebrewDate = 0;
          date.day = currentTM->tm_mday;
          date.month = currentTM->tm_mon + 1;
          date.year = currentTM->tm_year + 1900;
          afterSunset = FALSE;
          if (AnniDoEdit(name, &type, &isHebrewDate, &date, &afterSunset))
          {
            AnniFileInsertSorted(&af, name, type, isHebrewDate,
                                 &date, afterSunset);
            AnniFillTheList(listAnni, &glistAnni, &af);
            AnniDoCalculate(listCalculated, &glistCalculate, &af,
                            (-1), 0, 0);
          }
        }
        break;

      case ID_ANNI_MODIFY:
        {
          iSelected = AnniGetCurrentItem(listAnni);
          if (iSelected == (-1))
          {
            DialogsDoMessage("An entry must be selected");
            break;
          }

          AnniFileGet(&af, iSelected, name, &type, &isHebrewDate,
                      &date, &afterSunset);
          if (AnniDoEdit(name, &type, &isHebrewDate, &date, &afterSunset))
          {
            AnniFileModify(&af, iSelected, name, type, isHebrewDate,
                           &date, afterSunset);
            AnniFillTheList(listAnni, &glistAnni, &af);
            AnniDoCalculate(listCalculated, &glistCalculate, &af,
                            (-1), 0, 0);
          }
        }
        break;

      case ID_ANNI_REMOVE:
        {
          iSelected = AnniGetCurrentItem(listAnni);
          if (iSelected == (-1))
          {
            DialogsDoMessage("An entry must be selected");
            break;
          }

          AnniFileGet(&af, iSelected, name, &type, &isHebrewDate,
                      &date, &afterSunset);
          sprintf(msg,
                  "Are you really sure to\n" 
                  "delete entry for %s?",
                  name);
          if (DialogsDoYesNo(msg) == IDYES)
          {
            AnniFileRemove(&af, iSelected);
            AnniFillTheList(listAnni, &glistAnni, &af);
            AnniDoCalculate(listCalculated, &glistCalculate, &af,
                            (-1), 0, 0);
          }
        }
        break;

      case ID_ANNI_CALCULATE:
        {
          Recalculate(1);
        }
        break;
    }
  }
  if (GTK_IS_DIALOG(dialog)) gtk_widget_destroy(GTK_WIDGET(dialog));
  AnniFileStore(&af);
  AnniFileDone(&af);
}

char* getComboMonth(int month)
{
  switch (month)
  {
    case 1: return "Nisan";
    case 2: return "Iyar";
    case 3: return "Sivan";
    case 4: return "Tamuz";
    case 5: return "Av";
    case 6: return "Elul";
    case 7: return "Tishri";
    case 8: return "Heshvan";
    case 9: return "Kislev";
    case 10: return "Tevet";
    case 11: return "Shevat";
    case 12: return "Adar/Adar I";
    case 13: return "Adar II";
  }
  return "";
}

int AnniDoEdit(char *name, char *type, int *isHebrewDate,
               CalDate* date, int *afterSunset)
{
  GtkDialog *dialog;
  GtkLabel *label;
  GtkEntry *entryName, *entryDate;
  GtkRadioButton *radioA, *radioB, *radioY;
  GtkCheckButton *checkAfterSunset;
  GtkButton *button;
  char text[256], *dateString, *lineLauf;
  int i, iReturn;

  /* Entering Hebrew dates */
  GtkBox *thebox; GtkHBox* hbox;
  GtkRadioButton *radioGregorian, *radioHebrew;
  GtkEntry *entryHebrewDay, *entryHebrewYear;
  GtkCombo *comboHebrewMonth;
  GList *monthList;
  char hebrewDayString[256], hebrewMonthString[256], hebrewYearString[256];
  int hebrewDay, hebrewMonth, hebrewYear;

  dialog = GTK_DIALOG(gtk_dialog_new());
  gtk_window_set_title(GTK_WINDOW(dialog), 
                       "Edit Anniversary/Birthday/" YAHRZEIT_FULL);

  label = GTK_LABEL(gtk_label_new("Name:"));
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));
  entryName = GTK_ENTRY(gtk_entry_new());
  gtk_entry_set_text(entryName, name);
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(entryName));
  gtk_widget_show(GTK_WIDGET(entryName));

  thebox = GTK_BOX(gtk_hbox_new(FALSE, 1));

  radioA = GTK_RADIO_BUTTON(gtk_radio_button_new_with_label
              (NULL, STRING_TYPEANNI));
  if (*type == 'A') 
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radioA), TRUE);
  gtk_container_add(GTK_CONTAINER(thebox), GTK_WIDGET(radioA));
  gtk_widget_show(GTK_WIDGET(radioA));

  radioB = GTK_RADIO_BUTTON(gtk_radio_button_new_with_label_from_widget
              (radioA, STRING_TYPEBIRTHDAY));
  if (*type == 'B') 
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radioB), TRUE);
  gtk_container_add(GTK_CONTAINER(thebox), GTK_WIDGET(radioB));
  gtk_widget_show(GTK_WIDGET(radioB));

  radioY = GTK_RADIO_BUTTON(gtk_radio_button_new_with_label_from_widget
              (radioB, STRING_TYPEYAHRZEIT));
  if (*type == 'Y') 
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radioY), TRUE);
  gtk_container_add(GTK_CONTAINER(thebox), GTK_WIDGET(radioY));
  gtk_widget_show(GTK_WIDGET(radioY));

  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(thebox));
  gtk_widget_show(GTK_WIDGET(thebox));

  radioGregorian = GTK_RADIO_BUTTON(gtk_radio_button_new_with_label
                        (NULL, "Gregorian date of event:"));
  if (!*isHebrewDate)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radioGregorian), TRUE);
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(radioGregorian));
  gtk_widget_show(GTK_WIDGET(radioGregorian));

  entryDate = GTK_ENTRY(gtk_entry_new());
  if (!*isHebrewDate)
  {
    sprintf(text, "%d.%d.%d", date->day, date->month, date->year);
    gtk_entry_set_text(entryDate, text);
  }
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(entryDate));
  gtk_widget_show(GTK_WIDGET(entryDate));

  checkAfterSunset = GTK_CHECK_BUTTON(gtk_check_button_new_with_label
                           ("After Sunset"));
  if (!*isHebrewDate)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkAfterSunset), 
                                 *afterSunset);
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(checkAfterSunset));
  gtk_widget_show(GTK_WIDGET(checkAfterSunset));

  radioHebrew = GTK_RADIO_BUTTON(gtk_radio_button_new_with_label_from_widget
                           (radioGregorian, "Hebrew date of event:"));
  if (*isHebrewDate)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radioHebrew), TRUE);
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(radioHebrew));
  gtk_widget_show(GTK_WIDGET(radioHebrew));

  thebox = GTK_BOX(gtk_vbox_new(FALSE, 1));
    hbox = GTK_HBOX(gtk_hbox_new(FALSE, 1));
      label = GTK_LABEL(gtk_label_new("Day:"));
      gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(label));
      gtk_widget_show(GTK_WIDGET(label));
      entryHebrewDay = GTK_ENTRY(gtk_entry_new());
      if (*isHebrewDate)
      {
        sprintf(text, "%d", date->day);
        gtk_entry_set_text(GTK_ENTRY(entryHebrewDay), text);
      }
      gtk_container_add(GTK_CONTAINER(hbox),
                        GTK_WIDGET(entryHebrewDay));
      gtk_widget_show(GTK_WIDGET(entryHebrewDay));
    gtk_container_add(GTK_CONTAINER(thebox),
                      GTK_WIDGET(hbox));
    gtk_widget_show(GTK_WIDGET(hbox));

    hbox = GTK_HBOX(gtk_hbox_new(FALSE, 1));
      label = GTK_LABEL(gtk_label_new("Month:"));
      gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(label));
      gtk_widget_show(GTK_WIDGET(label));
      comboHebrewMonth = GTK_COMBO(gtk_combo_new());
        monthList = g_list_alloc();
        for (i = 1; i <= 13; i++)
          g_list_append(monthList, getComboMonth(i));
        gtk_combo_set_popdown_strings(comboHebrewMonth, monthList);
        gtk_combo_set_value_in_list(comboHebrewMonth, TRUE, FALSE);
        if (*isHebrewDate)
          gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboHebrewMonth)->entry),
                             getComboMonth(date->month));
      gtk_container_add(GTK_CONTAINER(hbox),
                        GTK_WIDGET(comboHebrewMonth));
      gtk_widget_show(GTK_WIDGET(comboHebrewMonth));
    gtk_container_add(GTK_CONTAINER(thebox),
                      GTK_WIDGET(hbox));
    gtk_widget_show(GTK_WIDGET(hbox));

    hbox = GTK_HBOX(gtk_hbox_new(FALSE, 1));
      label = GTK_LABEL(gtk_label_new("Year:"));
      gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(label));
      gtk_widget_show(GTK_WIDGET(label));
      entryHebrewYear = GTK_ENTRY(gtk_entry_new());
      if (*isHebrewDate)
      {
        sprintf(text, "%d", date->year);
        gtk_entry_set_text(GTK_ENTRY(entryHebrewYear), text);
      }
      gtk_container_add(GTK_CONTAINER(hbox),
                        GTK_WIDGET(entryHebrewYear));
      gtk_widget_show(GTK_WIDGET(entryHebrewYear));
    gtk_container_add(GTK_CONTAINER(thebox),
                      GTK_WIDGET(hbox));
    gtk_widget_show(GTK_WIDGET(hbox));

  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(thebox));
  gtk_widget_show(GTK_WIDGET(thebox));

  button = GTK_BUTTON(gtk_button_new_with_label("OK"));
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(AnniDialogButton), (gpointer) 1);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                     GTK_WIDGET(button),
                     TRUE, TRUE, 0);
  gtk_widget_show(GTK_WIDGET(button));

  button = GTK_BUTTON(gtk_button_new_with_label("Cancel"));
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(AnniDialogButton), (gpointer) 2);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
                     GTK_WIDGET(button),
                     TRUE, TRUE, 0);
  gtk_widget_show(GTK_WIDGET(button));

  gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
                     GTK_SIGNAL_FUNC(AnniDialogButton), (gpointer) 2);

  gtk_widget_show(GTK_WIDGET(dialog));

  for (;;)
  {  
    idAnniClickedButton = (-1);
    while (idAnniClickedButton == (-1))
    {
/*    if (gtk_events_pending())*/
        gtk_main_iteration();
    }

    if (idAnniClickedButton == 1)
    {
      strcpy(name, gtk_entry_get_text(entryName));
      if (strcmp(name, "") == 0)
      {
        DialogsDoMessage("Name must not be empty");
        continue;
      }

      *type = ' ';
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radioA))) *type = 'A';
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radioB))) *type = 'B';
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radioY))) *type = 'Y';

      *isHebrewDate = gtk_toggle_button_get_active
                             (GTK_TOGGLE_BUTTON(radioHebrew));
      if (*isHebrewDate)
      {
        strcpy(hebrewDayString, gtk_entry_get_text(entryHebrewDay));
        if (!NumberIsInteger(hebrewDayString))
        {
          DialogsDoMessage("Hebrew Day must be a number");
          continue;
        }
        hebrewDay = atoi(hebrewDayString);

        hebrewMonth = (-1);
        strcpy(hebrewMonthString,
           gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(comboHebrewMonth)->entry)));
        for (i = 1; i <= 13; i++)
        {
          if (strcmp(hebrewMonthString, getComboMonth(i)) == 0)
            hebrewMonth = i;
        }
        if (hebrewMonth == (-1))
        {
          DialogsDoMessage("Hebrew month must be selected");
          continue;
        }

        strcpy(hebrewYearString, gtk_entry_get_text(entryHebrewYear));
        if (!NumberIsInteger(hebrewYearString))
        {
          DialogsDoMessage("Hebrew Year must be a number");
          continue;
        }
        hebrewYear = atoi(hebrewYearString);

        if ((hebrewYear < HYEAR_BEGIN) || (hebrewYear > HYEAR_END))
        {
          sprintf(text, "Hebrew Year must be in range %d-%d",
                  HYEAR_BEGIN, HYEAR_END);
          DialogsDoMessage(text);
          continue;
        }
        if ((hebrewMonth < 1) || 
            (hebrewMonth > HebrcalGetLastMonthOfYear(hebrewYear)))
        {
          DialogsDoMessage("Hebrew month is invalid");
          continue;
        }
        if ((hebrewDay < 1) ||
            (hebrewDay > HebrcalGetLastDayOfHebrewMonth
                              (hebrewMonth, hebrewYear)))
        {
          DialogsDoMessage("Hebrew day is invalid");
          continue;
        }
        date->day = hebrewDay;
        date->month = hebrewMonth;
        date->year = hebrewYear;
      }
      else
      {
        dateString = gtk_entry_get_text(entryDate);
        lineLauf = dateString;
        while (*lineLauf == ' ') lineLauf++;
        date->day = 0;
        while ((*lineLauf >= '0') && (*lineLauf <= '9'))
        {
          date->day = date->day*10 + (*lineLauf-'0');
          lineLauf++;
        }
        if (*lineLauf == '.')
        {
          lineLauf++;
          date->month = 0;
          while ((*lineLauf >= '0') && (*lineLauf <= '9'))
          {
            date->month = date->month*10 + (*lineLauf-'0');
            lineLauf++;
          }
          if (*lineLauf == '.')
          {
            lineLauf++;
            date->year = 0;
            while ((*lineLauf >= '0') && (*lineLauf <= '9'))
            {
              date->year = date->year*10 + (*lineLauf-'0');
              lineLauf++;
            }

            if ((date->year < YEAR_BEGIN) || (date->year > YEAR_END))
            {
              DialogsDoMessage("Date: Year is not in the correct range");
              continue;
            }
            if ((date->month < 1) || (date->month > 12))
            {
              DialogsDoMessage("Date: Month is not in the correct range");
              continue;
            }
            if ((date->day < 1) || (date->day > 
                   GregorGetLastDayOfMonth(date->month, date->year)))
            {
              DialogsDoMessage("Date: Day is not in the correct range");
              continue;
            }
          }
          else
          {
            DialogsDoMessage("Date: . after month number expected");
            continue;
          }
        }
        else
        {
          DialogsDoMessage("Date: . after day number expected");
          continue;
        }

        *afterSunset = gtk_toggle_button_get_active
                         (GTK_TOGGLE_BUTTON(checkAfterSunset));
      }

      iReturn = 1;
      break;
    }
    else
    {
      iReturn = 0;
      break;
    }
  }

  if (GTK_IS_DIALOG(dialog)) gtk_widget_destroy(GTK_WIDGET(dialog));

  return (iReturn);
}

