#include "config.h"
#include "gregor.h"
#include "hebrcal.h"
#include "anni.h"
#include "annifile.h"

void AnniFileInit(AnniFile** af)
{
  *af = NULL;
}

void AnniFileDone(AnniFile** af)
{
  AnniFile *afLauf, *afDel;

  afLauf = *af;
  while (afLauf)
  {
    afDel = afLauf;
    afLauf = afLauf->next;
    free(afDel);
  }

  *af = NULL;
}

void AnniFileInsertSorted(AnniFile** af, char *name, char type,
                          int isHebrewDate, CalDate* dateAnni, int afterSunset)
{
  AnniFile *afTemp, *afPrev, *afLauf;

  afTemp = (AnniFile *) malloc(sizeof(AnniFile));
  strcpy(afTemp->name, name);
  afTemp->type = type;
  afTemp->isHebrewDate = isHebrewDate;
  memcpy(&afTemp->dateAnni, dateAnni, sizeof(CalDate));
  afTemp->afterSunset = afterSunset;
  afTemp->next = NULL;

  afPrev = NULL;
  afLauf = *af;
  while (afLauf)
  {
    if (strcmp(afLauf->name, name) > 0)
    {
      if (afPrev)
      {
        afTemp->next = afPrev->next;
        afPrev->next = afTemp;
      }
      else
      {
        afTemp->next = *af;
        *af = afTemp;
      }
      return;
    }
    afPrev = afLauf;
    afLauf = afLauf->next;
  }
  if (afPrev)
    afPrev->next = afTemp;
  else
    *af = afTemp;
}

int AnniFileCount(AnniFile** af)
{
  AnniFile *afLauf;
  int i;

  i = 0;
  afLauf = *af;
  while (afLauf)
  {
    i++;
    afLauf = afLauf->next;
  }
  return (i);
}

void AnniFileGet(AnniFile** af, int index, char *name, char *type,
                 int *isHebrewDate, CalDate* dateAnni, int* afterSunset)
{
  AnniFile *afLauf;
  int i;

  i = 0;
  afLauf = *af;
  while (afLauf)
  {
    if (i == index)
    {
      strcpy(name, afLauf->name);
      *type = afLauf->type;
      *isHebrewDate = afLauf->isHebrewDate;
      memcpy(dateAnni, &afLauf->dateAnni, sizeof(CalDate));
      *afterSunset = afLauf->afterSunset;
      return;
    }
    i++;
    afLauf = afLauf->next;
  }
}

void AnniFileModify(AnniFile** af, int index, char *name, char type,
                    int isHebrewDate, CalDate* dateAnni, int afterSunset)
{
  AnniFile *afLauf;
  int i;

  i = 0;
  afLauf = *af;
  while (afLauf)
  {
    if (i == index)
    {
      strcpy(afLauf->name, name);
      afLauf->type = type;
      afLauf->isHebrewDate = isHebrewDate;
      memcpy(&afLauf->dateAnni, dateAnni, sizeof(CalDate));
      afLauf->afterSunset = afterSunset;
      return;
    }
    i++;
    afLauf = afLauf->next;
  }
}

void AnniFileRemove(AnniFile** af, int index)
{
  AnniFile *afPrev, *afLauf;
  int i;

  i = 0;
  afPrev = NULL;
  afLauf = *af;
  while (afLauf)
  {
    if (i == index)
    {
      if (afPrev)
        afPrev->next = afLauf->next;
      else
        *af = afLauf->next;
      free(afLauf);
      return;
    }
    i++;
    afPrev = afLauf;
    afLauf = afLauf->next;
  }
}

void AnniFileGetFilename(char *filename)
{
  char *homePath;

  homePath = getenv("HOME");
  if (homePath)
    strcpy(filename, homePath);
  else
    strcpy(filename, ".");
  if (filename[strlen(filename)-1] != '/')
    strcat(filename, "/");
  strcat(filename, ".jewcalanni");
}

void AnniFileStore(AnniFile** af)
{
  char filename[256], line[256], name[256], *nameLauf;
  char name2[256], *name2Lauf, type;
  CalDate date;
  FILE* stream;
  int i, iCount, isHebrewDate, afterSunset;

  AnniFileGetFilename(filename);
  stream = fopen(filename, "wb");
  if (stream)
  {
    iCount = AnniFileCount(af);
    for (i = 0; i < iCount; i++)
    {
      AnniFileGet(af, i, name, &type, &isHebrewDate, &date, &afterSunset);
      nameLauf = name;
      name2Lauf = name2;
      while (*nameLauf)
      {
        if ((*nameLauf == '"') || (*nameLauf == '\\'))
        {
          *name2Lauf++ = '\\';
          *name2Lauf++ = *nameLauf;
        }
        else
          *name2Lauf++ = *nameLauf;
        nameLauf++;
      }
      *name2Lauf = 0;
      sprintf(line, 
              "\"%s\" %c%s%d.%d.%d %d\n",
              name2, type,
              isHebrewDate ? " H " : " ", 
              date.day, date.month, date.year, afterSunset);
      fwrite(line, strlen(line), 1, stream);
    }

    fclose(stream);
  }
}

void AnniFileLoad(AnniFile** af)
{
  char filename[256], *buffer, *bufferLauf;
  char line[256], *lineLauf, *sharp;
  char name[256], *nameLauf, type;
  CalDate date;
  long fileSize;
  FILE* stream;
  int isHebrewDate, afterSunset;

  AnniFileInit(af);

  AnniFileGetFilename(filename);
  stream = fopen(filename, "rb");
  if (stream)
  {
    fseek(stream, 0, SEEK_END);
    fileSize = ftell(stream);
    fseek(stream, 0, SEEK_SET);
    buffer = malloc(fileSize+1);
    fread(buffer, fileSize, 1, stream);
    *(buffer + fileSize) = 0;
    fclose(stream);

    bufferLauf = buffer;
    while (*bufferLauf)
    {
      lineLauf = line;
      while ((*bufferLauf != 0) && (*bufferLauf != 10))
      {
        *lineLauf++ = *bufferLauf;
        bufferLauf++;
      }
      *lineLauf = 0;
      while (*bufferLauf == 10) bufferLauf++;

      sharp = strchr(line, '#');
      if (sharp) *sharp = 0;

      lineLauf = line;
      while (*lineLauf == ' ') lineLauf++;
      if (*lineLauf == '"')
      {
        lineLauf++;
        nameLauf = name;
        while ((*lineLauf != 0) && (*lineLauf != '"'))
        {
          if (*lineLauf == '\\') lineLauf++;
          *nameLauf++ = *lineLauf;
          lineLauf++;
        }
        *nameLauf = 0;
        if (*lineLauf == '"')
        {
          lineLauf++;
          while (*lineLauf == ' ') lineLauf++;
          type = *lineLauf++;
          if ((type == 'A') || (type == 'B') || (type == 'Y'))
          {
            while (*lineLauf == ' ') lineLauf++;

            if (*lineLauf == 'H')
            {
              isHebrewDate = 1;
              lineLauf++;
              while (*lineLauf == ' ') lineLauf++;
            }
            else
              isHebrewDate = 0;

            date.day = 0;
            while ((*lineLauf >= '0') && (*lineLauf <= '9'))
            {
              date.day = date.day*10 + (*lineLauf-'0');
              lineLauf++;
            }
            if (*lineLauf == '.')
            {
              lineLauf++;
              date.month = 0;
              while ((*lineLauf >= '0') && (*lineLauf <= '9'))
              {
                date.month = date.month*10 + (*lineLauf-'0');
                lineLauf++;
              }
              if (*lineLauf == '.')
              {
                lineLauf++;
                date.year = 0;
                while ((*lineLauf >= '0') && (*lineLauf <= '9'))
                {
                  date.year = date.year*10 + (*lineLauf-'0');
                  lineLauf++;
                }
                while (*lineLauf == ' ') lineLauf++;
                afterSunset = 0;
                while ((*lineLauf >= '0') && (*lineLauf <= '9'))
                {
                  afterSunset = afterSunset*10 + (*lineLauf-'0');
                  lineLauf++;
                }
                if (*lineLauf == 0)
                {
                  AnniFileInsertSorted(af, name, type, isHebrewDate,
                                       &date, afterSunset);
                }
              }
            }
          }
        }
      }
    }

    free(buffer);
  }
}

int AnnifileGetForDay(long currentDate, char *caption, char *text)
{
  int result, iCount, i, isHebrewDate, afterSunset;
  calDate dateCurrentHebrew, dateAnni;
  AnniFile* af;
  char name[256], type, ach[256], type_str[256];
  long absDate;
  AnniStruct anni;
 
  result = 0;

  if (caption) strcpy(caption, "");
  if (text) strcpy(text, "");

  HebrcalForeignFromAbsolute(currentDate, &dateCurrentHebrew);

  AnniFileLoad(&af);
  iCount = AnniFileCount(&af);
  for (i = 0; i < iCount; i++)
  {
    AnniFileGet(&af, i, name, &type, &isHebrewDate, &dateAnni, &afterSunset);
    if (!isHebrewDate)
    {
      absDate = GregorAbsoluteFromForeign(&dateAnni);
      if (afterSunset) absDate++;
      HebrcalForeignFromAbsolute(absDate, &dateAnni);
    }
    memcpy(&anni.anniDate, &dateAnni, sizeof(calDate));
    anni.chAnniType = type;
    anni.inHebrewYear = dateCurrentHebrew.year;
    AnniGetAnniversary(&anni);
    absDate = HebrcalAbsoluteFromForeign(&anni.anniDate2);
    if (absDate == currentDate)
    {
      result = 1;
      if (caption)
      {
        ach[0] = type; ach[1] = 0;
        strcat(caption, ach);
      }
      if (text)
      {
        switch (type)
        {
          case 'A': strcpy(type_str, "Anniversary"); break;
          case 'B': strcpy(type_str, "Birthday"); break;
          case 'Y': strcpy(type_str, YAHRZEIT_FULL); break;
        }
        sprintf(ach, "%s: %s\n", type_str, name);  
        strcat(text, ach);
      }
    }  
  }

  AnniFileDone(&af);

  return (result);
}

