#include "config.h"
#include "gregor.h"
#include "misc.h"
#include "suntime.h"
#include "calctime.h"

int lstrcmpi(char *str1, char *str2)
{
  char *str1Lauf, *str2Lauf, ch1, ch2;

  str1Lauf = str1;
  str2Lauf = str2;
  for (;;)
  {
    if ((*str1Lauf == 0) && (*str2Lauf == 0)) 
      return 0;
    if ((*str1Lauf != 0) && (*str2Lauf != 0))
    {
      ch1 = toupper(*str1Lauf);
      ch2 = toupper(*str2Lauf);
      if (ch1 != ch2)
        return 1;
      else
      {
        str1Lauf++;
        str2Lauf++;
      }
    }
    else
      return 1;
  }
}

#define TOKEN_UNKNOWN   0
#define TOKEN_EOF       1
#define TOKEN_ADDOP     2
#define TOKEN_SUBOP     3
#define TOKEN_MULOP     4
#define TOKEN_DIVOP     5
#define TOKEN_LPAREN    6
#define TOKEN_RPAREN    7

#define TOKEN_ID        8
#define TOKEN_NUM       9
#define TOKEN_NUMREAL   10
                       
typedef struct tagCALC
{
  int iDay;
  int iMonth;
  int iYear;
  
  int iSunrise;
  int iSunset;
  
  int iLatitude; /* e.g. 3532 - 35 hours 32 minutes */
  int iLongitude;
  int iTimeZone;

  /************ Internal variables (DO NOT TOUCH) *******/
  
  char *lpLauf;
  int fEOFReached;
  int iCurrentToken;
  
  char szTokenString[256];
  int iTokenNum;
  double fTokenNumReal;
} CALC;
typedef CALC * LPCALC;

void CalcInitScanner(LPCALC lpCalc, char *lpBuffer)
{
  lpCalc->lpLauf = lpBuffer;
  lpCalc->fEOFReached = 0;
}  

char CalcGetChar(LPCALC lpCalc)
{
  char ch;
  
  if (lpCalc->fEOFReached)
    return 0;
    
  if (*(lpCalc->lpLauf) == 0)
  {
    lpCalc->fEOFReached = 1;
    lpCalc->lpLauf++;
    return 0;
  }  
  else
  {
    ch = *(lpCalc->lpLauf);
    lpCalc->lpLauf++;
    return (ch);    
  }  
}

void CalcUngetChar(LPCALC lpCalc)
{
  lpCalc->lpLauf--;
}

int CalcGetToken(LPCALC lpCalc)
{
  char ch, szText[256], szText2[256], *szLauf;
  int iValue, i;
  
token_again: ;
  ch = CalcGetChar(lpCalc);
  switch (ch)
  {
    case 0: return (TOKEN_EOF);
    case ' ': goto token_again;
    case 9: goto token_again;
    case 13: goto token_again;
    case 10: goto token_again;
    case '+': return (TOKEN_ADDOP);
    case '-': return (TOKEN_SUBOP);
    case '*': return (TOKEN_MULOP);
    case '/': return (TOKEN_DIVOP);
    case '(': return (TOKEN_LPAREN);
    case ')': return (TOKEN_RPAREN);
    
    default:
      {
        if ((ch >= '0') && (ch <= '9'))
        {
          iValue = 0;
          while ((ch >= '0') && (ch <= '9'))
          {
            iValue = iValue*10 + (ch-'0');
            ch = CalcGetChar(lpCalc);
          }
          if (ch != '.')
          {
            CalcUngetChar(lpCalc);
            lpCalc->iTokenNum = iValue;
            return (TOKEN_NUM);
          }  
          else
          {
            ch = CalcGetChar(lpCalc);
            szLauf = szText;
            while ((ch >= '0') && (ch <= '9'))
            {
              *szLauf++ = ch;
              ch = CalcGetChar(lpCalc);
            }
            *szLauf = 0;
            CalcUngetChar(lpCalc);
            
            sprintf(szText2, "%d.%s", iValue, (char *) szText);
            lpCalc->fTokenNumReal = atof(szText2);
            return (TOKEN_NUMREAL);
          }
        }
        else
        {
          if ( ((ch >= 'A') && (ch <= 'Z')) ||
               ((ch >= 'a') && (ch <= 'z')) ||
               (ch == '_'))
          {
            szLauf = lpCalc->szTokenString;
            while ( ((ch >= 'A') && (ch <= 'Z')) ||
                    ((ch >= 'a') && (ch <= 'z')) ||
                    ((ch >= '0') && (ch <= '9')) ||
                    (ch == '_'))
            {
              *szLauf++ = ch;
              ch = CalcGetChar(lpCalc);
            }        
            *szLauf = 0;
            CalcUngetChar(lpCalc);
            
            for (i = 0; i < strlen(lpCalc->szTokenString); i++)
              lpCalc->szTokenString[i] = toupper(lpCalc->szTokenString[i]);
            return (TOKEN_ID);
          }     
          else
            return (TOKEN_UNKNOWN);
        }
      }
      break;
  }
}

void CalcParseError(LPCALC lpCalc, char *message)
{
  printf("Parsing error %s\n", message);
}

int CalcParseExpr(LPCALC lpCalc);

int CalcParseFactor(LPCALC lpCalc)
{
  switch (lpCalc->iCurrentToken)
  {
    case TOKEN_LPAREN:
      {
        int iValue;
       
        lpCalc->iCurrentToken = CalcGetToken(lpCalc);
        iValue = CalcParseExpr(lpCalc);
        if (lpCalc->iCurrentToken != TOKEN_RPAREN)
        {
          CalcParseError(lpCalc, ") expected");
          return (-1);
        }
        lpCalc->iCurrentToken = CalcGetToken(lpCalc);
        return (iValue);
      }
      
    case TOKEN_NUM: 
      {
        int iValue;
        
        iValue = lpCalc->iTokenNum;
        lpCalc->iCurrentToken = CalcGetToken(lpCalc);
        return (iValue);
      }  
      
    case TOKEN_ID:
      {
        double phr, result, db;
        double sunset, sunrise, day_length_in_mins, phr_length;
        CalDate date;
        long absDate;
        int iTemp, iValue, iCHour, iCMin;
        int hour, min, fResult;
        
        if (lstrcmpi(lpCalc->szTokenString, "SUNRISE") == 0)
        {
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          return (lpCalc->iSunrise);
        }  
        if (lstrcmpi(lpCalc->szTokenString, "SUNSET") == 0)
        {
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          return (lpCalc->iSunset);
        }  
          
        if (lstrcmpi(lpCalc->szTokenString, "PROPORTIONAL_HOUR") == 0)
        {
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          if (lpCalc->iCurrentToken != TOKEN_LPAREN)
          {
            CalcParseError(lpCalc, "( expected");
            return (-1);
          }
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          if ((lpCalc->iCurrentToken != TOKEN_NUM) &&
              (lpCalc->iCurrentToken != TOKEN_NUMREAL))
          {
            CalcParseError(lpCalc, "Integer-number or real-number expected");
            return (-1);
          }    
          
          if (lpCalc->iCurrentToken == TOKEN_NUMREAL)
            phr = lpCalc->fTokenNumReal;            
          else
            phr = (double) lpCalc->iTokenNum;
            
          sunrise = (double) lpCalc->iSunrise;
          sunset = (double) lpCalc->iSunset;
          day_length_in_mins = (double) sunset - (double) sunrise;
          phr_length = (double) day_length_in_mins / (double) 12.0;
          result = (double) phr * (double) phr_length;

          hour = (int) ((double) result / (double) 60.0);
          min = (int) (fmodl((double) result, (double) 60.0));
          iValue = hour*60 + min;
          
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          if (lpCalc->iCurrentToken != TOKEN_RPAREN)
          {
            CalcParseError(lpCalc, ") expected");
            return (-1);
          }
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);

          return (iValue);          
        }  

        if (lstrcmpi(lpCalc->szTokenString, "DEGREES_BELOW_HORIZON") == 0)
        {
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          if (lpCalc->iCurrentToken != TOKEN_LPAREN)
          {
            CalcParseError(lpCalc, "( expected");
            return (-1);
          }
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          if ((lpCalc->iCurrentToken != TOKEN_NUM) &&
              (lpCalc->iCurrentToken != TOKEN_NUMREAL))
          {
            CalcParseError(lpCalc, "Integer-number or real-number expected");
            return (-1);
          }    
          
          if (lpCalc->iCurrentToken == TOKEN_NUMREAL)
            db = lpCalc->fTokenNumReal;            
          else
            db = (double) lpCalc->iTokenNum;

          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          if (lpCalc->iCurrentToken != TOKEN_RPAREN)
          {
            CalcParseError(lpCalc, ") expected");
            return (-1);
          }
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);

          fResult = GetDegreesBelowHorizon
               (lpCalc->iDay, lpCalc->iMonth, lpCalc->iYear, db,
                lpCalc->iLatitude, lpCalc->iLongitude,
                lpCalc->iTimeZone, 
                &iCHour, &iCMin);
          if (!fResult || ((iCHour < 0) || (iCMin < 0)))
            return (-1);
          else  
            return (iCHour * 60 + iCMin);
        }  

        if (lstrcmpi(lpCalc->szTokenString, "FRIDAY") == 0)
        {
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          if (lpCalc->iCurrentToken != TOKEN_LPAREN)
          {
            CalcParseError(lpCalc, "( expected");
            return (-1);
          }
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          
          iTemp = CalcParseExpr(lpCalc);

          if (lpCalc->iCurrentToken != TOKEN_RPAREN)
          {
            CalcParseError(lpCalc, ") expected");
            return (-1);
          }
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);

          date.day = lpCalc->iDay;
          date.month = lpCalc->iMonth;
          date.year = lpCalc->iYear;
          absDate = GregorAbsoluteFromForeign(&date);
          if ((absDate % 7) == 5)
            return (iTemp);
          else
            return (-1);  
        }

        if (lstrcmpi(lpCalc->szTokenString, "SATURDAY") == 0)
        {
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          if (lpCalc->iCurrentToken != TOKEN_LPAREN)
          {
            CalcParseError(lpCalc, "( expected");
            return (-1);
          }
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);
          
          iTemp = CalcParseExpr(lpCalc);

          if (lpCalc->iCurrentToken != TOKEN_RPAREN)
          {
            CalcParseError(lpCalc, ") expected");
            return (-1);
          }
          lpCalc->iCurrentToken = CalcGetToken(lpCalc);

          date.day = lpCalc->iDay;
          date.month = lpCalc->iMonth;
          date.year = lpCalc->iYear;
          absDate = GregorAbsoluteFromForeign(&date);
          if ((absDate % 7) == 6)
            return (iTemp);
          else
            return (-1);  
        }
      }
      break;
  }
  CalcParseError(lpCalc, 
                 "Number or ID "
                 "(SUNRISE, SUNSET, "
                 "PROPORTIONAL_HOUR, "
                 "DEGREES_BELOW_HORIZON, "
                 "FRIDAY or SATURDAY expected)");
  return (-1);
}

int CalcParseTerm(LPCALC lpCalc)
{
  int iValue, iValue2, iOp;
  
  iValue = CalcParseFactor(lpCalc);
  if (iValue == (-1))
    return (-1);
    
  while ((lpCalc->iCurrentToken == TOKEN_MULOP) ||
         (lpCalc->iCurrentToken == TOKEN_DIVOP))
  {
    iOp = lpCalc->iCurrentToken;
    lpCalc->iCurrentToken = CalcGetToken(lpCalc);
    iValue2 = CalcParseFactor(lpCalc);
    if (iValue2 == (-1))
      return (-1);
    
    switch (iOp)
    {
      case TOKEN_MULOP: iValue = iValue * iValue2; break;
      case TOKEN_DIVOP: iValue = iValue / iValue2; break;
    }    
  } 
  return (iValue);         
}
  
int CalcParseExpr(LPCALC lpCalc)
{
  int iValue, iValue2, iOp;
  
  iValue = CalcParseTerm(lpCalc);
  if (iValue == (-1))
    return (-1);
    
  while ((lpCalc->iCurrentToken == TOKEN_ADDOP) ||
         (lpCalc->iCurrentToken == TOKEN_SUBOP))
  {
    iOp = lpCalc->iCurrentToken;
    lpCalc->iCurrentToken = CalcGetToken(lpCalc);
    iValue2 = CalcParseTerm(lpCalc);
    if (iValue2 == (-1))
      return (-1);
    
    switch (iOp)
    {
      case TOKEN_ADDOP: iValue = iValue + iValue2; break;
      case TOKEN_SUBOP: iValue = iValue - iValue2; break;
    }    
  } 
  return (iValue);         
}
  
int CalcEvaluate(LPCALC lpCalc, 
                 char *expression)
{
  int iResult;
  
  CalcInitScanner(lpCalc, expression);
  lpCalc->iCurrentToken = CalcGetToken(lpCalc);
  iResult = CalcParseExpr(lpCalc);
  if (iResult == (-1))
    return (-1);
  if (lpCalc->iCurrentToken != TOKEN_EOF)
  {
    CalcParseError(lpCalc, "Unexpected end of text");
    return (-1);
  }  
  return (iResult);    
}
                  
int CalculateZmanim(int latitude, int longitude, int timeZone,
                    int day, int month, int year,
                    char *calculation)
{
  CALC calc;
  int iHour, iMin;
  
  memset(&calc, 0, sizeof(CALC));
  
  calc.iDay = day;
  calc.iMonth = month;
  calc.iYear = year;

  GetSunsetSunrise(day, month, year,
                   OCCURRENCE_SUNRISE, 
                   latitude, longitude, timeZone,
                   &iHour, &iMin);
  calc.iSunrise = iHour*60 + iMin;
  GetSunsetSunrise(day, month, year,
                   OCCURRENCE_SUNSET, 
                   latitude, longitude, timeZone,
                   &iHour, &iMin);
  calc.iSunset = iHour*60 + iMin;
    
  calc.iLatitude = latitude;
  calc.iLongitude = longitude;
  calc.iTimeZone = timeZone;
        
  return CalcEvaluate(&calc, calculation);
}

void EliminateSpecialChars(char *str)
{
  char *strLauf, newStr[256], *newStrLauf;

  strLauf = str;
  newStrLauf = newStr;
  while (*strLauf)
  {
    switch ((unsigned char) *strLauf)
    {
      case '': *newStrLauf++ = 'a'; *newStrLauf++ = 'e'; break;
      case '': *newStrLauf++ = 'o'; *newStrLauf++ = 'e'; break;
      case '': *newStrLauf++ = 'u'; *newStrLauf++ = 'e'; break;
      case '': *newStrLauf++ = 'A'; *newStrLauf++ = 'e'; break;
      case '': *newStrLauf++ = 'O'; *newStrLauf++ = 'e'; break;
      case '': *newStrLauf++ = 'U'; *newStrLauf++ = 'e'; break;
      case '': *newStrLauf++ = 's'; *newStrLauf++ = 's'; break;

      case '': *newStrLauf++ = 'a'; break;
      case '': *newStrLauf++ = 'a'; break;
      case '': *newStrLauf++ = 'a'; break;
      case '': *newStrLauf++ = 'e'; break;
      case '': *newStrLauf++ = 'e'; break;
      case '': *newStrLauf++ = 'e'; break;
      case '': *newStrLauf++ = 'i'; break;
      case '': *newStrLauf++ = 'i'; break;
      case '': *newStrLauf++ = 'i'; break;
      case '': *newStrLauf++ = 'o'; break;
      case '': *newStrLauf++ = 'o'; break;
      case '': *newStrLauf++ = 'o'; break;
      case '': *newStrLauf++ = 'u'; break;
      case '': *newStrLauf++ = 'u'; break;
      case '': *newStrLauf++ = 'u'; break;

      case '`': *newStrLauf++ = 39; break;
      case 0xB4: *newStrLauf++ = 39; break;

      default:
        *newStrLauf++ = *strLauf;
        break;
    }
    strLauf++;
  }
  *newStrLauf = 0;
  strcpy(str, newStr);
}

int CityProcessLine(char *lpLine, char *lpCity,
                    int *lpiLatitude, int *lpiLongitude,
                    int *lpiTimeZone, char *lpErrorMessage)
{
  char *lpLineLauf, *lpCityLauf;
  int iLatitudeHour, iLatitudeMin, iLongitudeHour, iLongitudeMin, iGMT;
  int fLatitudeNeg, fLongitudeNeg, fGMTNeg;
  char szGMT[256], *szGMTLauf;

  lpLineLauf = lpLine;
  while (*lpLineLauf == ' ') lpLineLauf++;
  if (*lpLineLauf != '"')
  {
    strcpy(lpErrorMessage, "Begin of city name (\") expected");
    return 0;
  }
  lpLineLauf++;

  lpCityLauf = lpCity;
  while ((*lpLineLauf != 0) && (*lpLineLauf != '"'))
  {
    *lpCityLauf++ = *lpLineLauf;
    lpLineLauf++;
  }
  *lpCityLauf = 0;

  EliminateSpecialChars(lpCity);

  if (*lpLineLauf != '"')
  {
    strcpy(lpErrorMessage, "End of city name (\") expected");
    return 0;
  }
  lpLineLauf++;
  while (*lpLineLauf == ' ') lpLineLauf++;
  if (*lpLineLauf != ',')
  {
    strcpy(lpErrorMessage, "Comma (,) after city name expected");
    return 0;
  }
  lpLineLauf++;
  while (*lpLineLauf == ' ') lpLineLauf++;

  iLatitudeHour = 0;
  while ((*lpLineLauf >= '0') && (*lpLineLauf <= '9'))
  {
    iLatitudeHour = iLatitudeHour * 10 + (*lpLineLauf-'0');
    lpLineLauf++;
  }
  if (*lpLineLauf != ',')
  {
    strcpy(lpErrorMessage, "Comma (,) after latitude hour expected");
    return 0;
  }
  lpLineLauf++;
  while (*lpLineLauf == ' ') lpLineLauf++;
  iLatitudeMin = 0;
  while ((*lpLineLauf >= '0') && (*lpLineLauf <= '9'))
  {
    iLatitudeMin = iLatitudeMin * 10 + (*lpLineLauf-'0');
    lpLineLauf++;
  }
  while (*lpLineLauf == ' ') lpLineLauf++;
  if ((*lpLineLauf == 'N') || (*lpLineLauf == 'n'))
    fLatitudeNeg = 0;
  else
  {
    if ((*lpLineLauf == 'S') || (*lpLineLauf == 's'))
      fLatitudeNeg = 1;
    else
    {
      strcpy(lpErrorMessage, "N or S in latitude expected");
      return 0;
    }
  }
  lpLineLauf++;

  *lpiLatitude = iLatitudeHour * 100 + iLatitudeMin;
  if (fLatitudeNeg) *lpiLatitude = -(*lpiLatitude);

  while (*lpLineLauf == ' ') lpLineLauf++;
  if (*lpLineLauf != ',')
  {
    strcpy(lpErrorMessage, "Comma (,) after N/S expected");
    return 0;
  }
  lpLineLauf++;
  while (*lpLineLauf == ' ') lpLineLauf++;

  iLongitudeHour = 0;
  while ((*lpLineLauf >= '0') && (*lpLineLauf <= '9'))
  {
    iLongitudeHour = iLongitudeHour * 10 + (*lpLineLauf-'0');
    lpLineLauf++;
  }
  if (*lpLineLauf != ',')
  {
    strcpy(lpErrorMessage, "Comma (,) after longitude hour expected");
    return 0;
  }
  lpLineLauf++;
  while (*lpLineLauf == ' ') lpLineLauf++;
  iLongitudeMin = 0;
  while ((*lpLineLauf >= '0') && (*lpLineLauf <= '9'))
  {
    iLongitudeMin = iLongitudeMin * 10 + (*lpLineLauf-'0');
    lpLineLauf++;
  }
  while (*lpLineLauf == ' ') lpLineLauf++;
  if ((*lpLineLauf == 'E') || (*lpLineLauf == 'e'))
    fLongitudeNeg = 0;
  else
  {
    if ((*lpLineLauf == 'W') || (*lpLineLauf == 'w'))
      fLongitudeNeg = 1;
    else
    {
      strcpy(lpErrorMessage, "W or E in latitude expected");
      return 0;
    }
  }
  lpLineLauf++;

  *lpiLongitude = iLongitudeHour * 100 + iLongitudeMin;
  if (fLongitudeNeg) *lpiLongitude = -(*lpiLongitude);

  while (*lpLineLauf == ' ') lpLineLauf++;
  if (*lpLineLauf != ',')
  {
    strcpy(lpErrorMessage, "Comma (,) after W/E expected");
    return 0;
  }
  lpLineLauf++;
  while (*lpLineLauf == ' ') lpLineLauf++;

  szGMTLauf = szGMT;
  while (((*lpLineLauf >= 'A') && (*lpLineLauf <= 'Z')) ||
         ((*lpLineLauf >= 'a') && (*lpLineLauf <= 'z')))
  {
    *szGMTLauf++ = *lpLineLauf;
    lpLineLauf++;
  }
  *szGMTLauf = 0;
  if (lstrcmpi(szGMT, "GMT") != 0)
  {
    strcpy(lpErrorMessage, "GMT expected");
    return 0;
  }
  if ((*lpLineLauf == 0) || (*lpLineLauf == ' '))
    iGMT = 0;
  else
  {
    if (*lpLineLauf == '+')
      fGMTNeg = 0;
    else
    {
      if (*lpLineLauf == '-')
        fGMTNeg = 1;
      else
      {
        strcpy(lpErrorMessage, "+ or - in GMT zone expected");
        return 0;
      }
    }
    lpLineLauf++;
    iGMT = 0;
    while ((*lpLineLauf >= '0') && (*lpLineLauf <= '9'))
    {
      iGMT = iGMT * 10 + (*lpLineLauf-'0');
      lpLineLauf++;
    }
    if (fGMTNeg)
      iGMT = -iGMT;
  }
  *lpiTimeZone = iGMT;

  return 1;
}
                               
int CityGet(char *fileName, char *city,
            int *latitude, int *longitude, int *timeZone)
{
  FILE *stream;
  long fileSize;
  char *lpBuffer, *lpBufferLauf;
  char szLine[256], *szLineLauf, *szComment, szCity[256];
  int result, iLatitude, iLongitude, iTimeZone, iLineNo;
  char errorMessage[256], errorMessage2[256];

  result = 0;
  stream = fopen(fileName, "rb");
  if (stream)
  {
    fseek(stream, 0, SEEK_END);
    fileSize = ftell(stream);
    fseek(stream, 0, SEEK_SET);
    lpBuffer = malloc(fileSize+1);
    fread(lpBuffer, fileSize, 1, stream);
    *(lpBuffer + fileSize) = 0;
    fclose(stream);

    iLineNo = 1;
    lpBufferLauf = lpBuffer;
    while (*lpBufferLauf)
    {
      szLineLauf = szLine;
      while ((*lpBufferLauf != 0) &&
             (*lpBufferLauf != 13) &&
             (*lpBufferLauf != 10))
      {
        *szLineLauf++ = *lpBufferLauf;
        lpBufferLauf++;
      }
      *szLineLauf = 0;
      while ((*lpBufferLauf == 13) ||
             (*lpBufferLauf == 10))
      {
        if (*lpBufferLauf == 10)
          iLineNo++;
        lpBufferLauf++;
      }

      szComment = strstr(szLine, "#");
      if (szComment) *szComment = 0;
      if (!IsBlank(szLine))
      {
        if (CityProcessLine(szLine, szCity, &iLatitude, &iLongitude,
                            &iTimeZone, errorMessage))
        {
          if (lstrcmpi(szCity, city) == 0)
          {
            *latitude = iLatitude;
            *longitude = iLongitude;
            *timeZone = iTimeZone;
            result = 1;
            break;
          }
        }
        else
        {
          sprintf(errorMessage2, "File %s, line %d: %s",
                   (char *) fileName, iLineNo, errorMessage);
          printf("%s\n", errorMessage2);
        }                       
      }
    }

    free(lpBuffer);
  }
  else
  {
    sprintf(errorMessage, "File %s could not be opened", fileName);
    printf("%s\n", errorMessage);
  }
  return (result);
}

int Daylight_Correction(int day, int month, int year,
                        int *hour1, int* hour2,
                        dstStruct *dst)
{
  int iStartWeekNo, iStartWeekDay, iStartMonth;
  int iEndWeekNo, iEndWeekDay, iEndMonth;
  CalDate dateStart, dateEnd, dateCurrent;
  long lStartDate, lEndDate, lCurrentDate;
  int i;
  
  if (dst)
  {
    iStartWeekNo = dst->startWeekNo;
    iStartWeekDay = dst->startWeekday;
    iStartMonth = dst->startMonth;

    iEndWeekNo = dst->endWeekNo;
    iEndWeekDay = dst->endWeekday;
    iEndMonth = dst->endMonth;

    if ((iStartWeekNo < 1) || (iStartWeekNo > DST_WEEK_LAST))
      return 0;
    if ((iStartWeekDay < 0) || (iStartWeekNo > 6))
      return 0;
    if ((iStartMonth < 1) || (iStartMonth > 12))
      return 0;

    if ((iEndWeekNo < 1) || (iEndWeekNo > DST_WEEK_LAST))
      return 0;
    if ((iEndWeekDay < 0) || (iEndWeekNo > 6))
      return 0;
    if ((iEndMonth < 1) || (iEndMonth > 12))
      return 0;
                                     
    if (iStartWeekNo != DST_WEEK_LAST)
    {
      dateStart.day   = 1;
      dateStart.month = iStartMonth;
      dateStart.year  = year;
      lStartDate = GregorAbsoluteFromForeign(&dateStart);
      while (GetWeekday(lStartDate) != iStartWeekDay)
        lStartDate++;
      for (i = 1; i < iStartWeekNo; i++)
        lStartDate += 7;
    }                   
    else
    {
      dateStart.day   = GregorGetLastDayOfMonth(iStartMonth, year);
      dateStart.month = iStartMonth;
      dateStart.year  = year;
      lStartDate = GregorAbsoluteFromForeign(&dateStart);
      while (GetWeekday(lStartDate) != iStartWeekDay)
        lStartDate--;
    }                      

    if (iEndWeekNo != DST_WEEK_LAST)
    {
      dateEnd.day   = 1;
      dateEnd.month = iEndMonth;
      dateEnd.year  = year;
      lEndDate = GregorAbsoluteFromForeign(&dateEnd);
      while (GetWeekday(lEndDate) != iEndWeekDay)
        lEndDate++;
      for (i = 1; i < iEndWeekNo; i++)
        lEndDate += 7;
    }                   
    else
    {
      dateEnd.day   = GregorGetLastDayOfMonth(iEndMonth, year);
      dateEnd.month = iEndMonth;
      dateEnd.year  = year;
      lEndDate = GregorAbsoluteFromForeign(&dateEnd);
      while (GetWeekday(lEndDate) != iEndWeekDay)
        lEndDate--;
    }                      
    
    dateCurrent.day   = day;
    dateCurrent.month = month;
    dateCurrent.year  = year;
    lCurrentDate = GregorAbsoluteFromForeign(&dateCurrent);
    
    if ((lCurrentDate >= lStartDate) && (lCurrentDate < lEndDate))
    {
      *hour1 = *hour1 + 1;
      if (*hour1 == 24)
        *hour1 = 0;
      *hour2 = *hour2 + 1;
      if (*hour2 == 24)
        *hour2 = 0;
    }    
  }
  return 1;
}
