#include <gtk/gtk.h>
#include "config.h"
#include "dialogs.h"
#include "misc.h"
#include "names.h"
#include "settings.h"
#include "shabconf.h"

#define DIALOG_TITLE       "Configure Daylight Savings Time"

#define LABEL_DST         "Consider DST"

void GetWeekNoString(int i, char *text)
{
  switch (i)
  {
    case 1: strcpy(text, "1st"); break;
    case 2: strcpy(text, "2nd"); break;
    case 3: strcpy(text, "3rd"); break;
    case 4: strcpy(text, "4th"); break;
    case 5: strcpy(text, "Last"); break;
  }
}
    
GList* GetWeekNoList(void)
{
  GList* list;
  int i;
  char text[256];

  list = g_list_alloc();
  for (i = 1; i <= 5; i++)
  {
    GetWeekNoString(i, text);
    list = g_list_append(list, strdup(text));
  }
  return (list);
}

int GetWeekNoFromString(char *str)
{
  int i;
  char text[256];

  for (i = 1; i <= 5; i++)
  {
    GetWeekNoString(i, text);
    if (strcmp(text, str) == 0)
      return (i);
  }
  return (-1);
}
    
GList* GetWeekdayList(void)
{
  GList* list;
  int i;
  char text[256];

  list = g_list_alloc();
  for (i = 0; i < 7; i++)
  {
    GetWeekdayName(i, text);
    list = g_list_append(list, strdup(text));
  }
  return (list);
}

int GetWeekdayFromString(char *str)
{
  int i;
  char text[256];

  for (i = 0; i < 7; i++)
  {
    GetWeekdayName(i, text);
    if (strcmp(text, str) == 0)
      return (i);
  }
  return (-1);
}

GList* GetMonthList(void)
{
  GList* list;
  int i;
  char text[256];

  list = g_list_alloc();
  for (i = 1; i <= 12; i++)
  {
    GetGregorianMonthName(i, text);
    list = g_list_append(list, strdup(text));
  }
  return (list);
}
              
int GetMonthFromString(char *str)
{
  int i;
  char text[256];

  for (i = 1; i <= 12; i++)
  {
    GetGregorianMonthName(i, text);
    if (strcmp(text, str) == 0)
      return (i);
  }
  return (-1);
}
         
int idDSTConfigClicked;

void DSTConfigClick(GtkWidget *widget, gpointer data)
{
  idDSTConfigClicked = (int) data;
}

int DoConfigureDST(void)
{
  GtkDialog *dialog;
  GtkVBox *vbox;
  GtkLabel *label;
  GtkButton *button;
  GtkCombo *comboStartWeekNo, *comboStartWeekday, *comboStartMonth;
  GtkCombo *comboEndWeekNo, *comboEndWeekday, *comboEndMonth;
  GtkCheckButton *checkDST;
  dstStruct dst;
  char text[256], errMsg[256];
  char *textCombo;
  int isDST;

  dialog = GTK_DIALOG(gtk_dialog_new());
  gtk_window_set_title(GTK_WINDOW(dialog), DIALOG_TITLE);

  checkDST = GTK_CHECK_BUTTON(gtk_check_button_new_with_label
                    (LABEL_DST));
  if (getIsDST())
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkDST), TRUE);
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(checkDST));
  gtk_widget_show(GTK_WIDGET(checkDST));

  getDSTstruct(&dst);

  vbox = GTK_VBOX(gtk_vbox_new(FALSE, 1));

  label = GTK_LABEL(gtk_label_new("Start:"));
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));

  comboStartWeekNo = GTK_COMBO(gtk_combo_new());
  gtk_combo_set_popdown_strings(GTK_COMBO(comboStartWeekNo),
                                GetWeekNoList());
  gtk_combo_set_value_in_list(GTK_COMBO(comboStartWeekNo), TRUE, FALSE);
  GetWeekNoString(dst.startWeekNo, text);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboStartWeekNo)->entry), text);
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(comboStartWeekNo));
  gtk_widget_show(GTK_WIDGET(comboStartWeekNo));

  comboStartWeekday = GTK_COMBO(gtk_combo_new());
  gtk_combo_set_popdown_strings(GTK_COMBO(comboStartWeekday),
                                GetWeekdayList());
  gtk_combo_set_value_in_list(GTK_COMBO(comboStartWeekday), TRUE, FALSE);
  GetWeekdayName(dst.startWeekday, text);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboStartWeekday)->entry), text);
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(comboStartWeekday));
  gtk_widget_show(GTK_WIDGET(comboStartWeekday));

  comboStartMonth = GTK_COMBO(gtk_combo_new());
  gtk_combo_set_popdown_strings(GTK_COMBO(comboStartMonth),
                                GetMonthList());
  gtk_combo_set_value_in_list(GTK_COMBO(comboStartMonth), TRUE, FALSE);
  GetGregorianMonthName(dst.startMonth, text);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboStartMonth)->entry), text);
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(comboStartMonth));
  gtk_widget_show(GTK_WIDGET(comboStartMonth));

  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(vbox));
  gtk_widget_show(GTK_WIDGET(vbox));

  vbox = GTK_VBOX(gtk_vbox_new(FALSE, 1));

  label = GTK_LABEL(gtk_label_new("End:"));
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));

  comboEndWeekNo = GTK_COMBO(gtk_combo_new());
  gtk_combo_set_popdown_strings(GTK_COMBO(comboEndWeekNo),
                                GetWeekNoList());
  gtk_combo_set_value_in_list(GTK_COMBO(comboEndWeekNo), TRUE, FALSE);
  GetWeekNoString(dst.endWeekNo, text);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboEndWeekNo)->entry), text);
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(comboEndWeekNo));
  gtk_widget_show(GTK_WIDGET(comboEndWeekNo));

  comboEndWeekday = GTK_COMBO(gtk_combo_new());
  gtk_combo_set_popdown_strings(GTK_COMBO(comboEndWeekday),
                                GetWeekdayList());
  gtk_combo_set_value_in_list(GTK_COMBO(comboEndWeekday), TRUE, FALSE);
  GetWeekdayName(dst.endWeekday, text);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboEndWeekday)->entry), text);
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(comboEndWeekday));
  gtk_widget_show(GTK_WIDGET(comboEndWeekday));

  comboEndMonth = GTK_COMBO(gtk_combo_new());
  gtk_combo_set_popdown_strings(GTK_COMBO(comboEndMonth),
                                GetMonthList());
  gtk_combo_set_value_in_list(GTK_COMBO(comboEndMonth), TRUE, FALSE);
  GetGregorianMonthName(dst.endMonth, text);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboEndMonth)->entry), text);
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(comboEndMonth));
  gtk_widget_show(GTK_WIDGET(comboEndMonth));

  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(vbox));
  gtk_widget_show(GTK_WIDGET(vbox));

  button = GTK_BUTTON(gtk_button_new_with_label("OK"));
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(DSTConfigClick), (gpointer) 1);
  gtk_container_add(GTK_CONTAINER(dialog->action_area), GTK_WIDGET(button));
  gtk_widget_show(GTK_WIDGET(button));

  button = GTK_BUTTON(gtk_button_new_with_label("Cancel"));
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(DSTConfigClick), (gpointer) 2);
  gtk_container_add(GTK_CONTAINER(dialog->action_area), GTK_WIDGET(button));
  gtk_widget_show(GTK_WIDGET(button));

  gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
                     GTK_SIGNAL_FUNC(DSTConfigClick), (gpointer) 2);

  gtk_widget_show(GTK_WIDGET(dialog));
 
  for (;;)
  {
    idDSTConfigClicked = (-1);
    while (idDSTConfigClicked == (-1))
    {
/*    if (gtk_events_pending())*/
        gtk_main_iteration();
    }
    if (idDSTConfigClicked == 1)
    {
          {
            isDST = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkDST));

            if (isDST)
            {
              textCombo = (char *) gtk_entry_get_text
                             (GTK_ENTRY(GTK_COMBO(comboStartWeekNo)->entry));
              dst.startWeekNo = GetWeekNoFromString(textCombo);
              textCombo = (char *) gtk_entry_get_text
                             (GTK_ENTRY(GTK_COMBO(comboStartWeekday)->entry));
              dst.startWeekday = GetWeekdayFromString(textCombo);
              textCombo = (char *) gtk_entry_get_text
                             (GTK_ENTRY(GTK_COMBO(comboStartMonth)->entry));
              dst.startMonth = GetMonthFromString(textCombo);
              
              textCombo = (char *) gtk_entry_get_text
                             (GTK_ENTRY(GTK_COMBO(comboEndWeekNo)->entry));
              dst.endWeekNo = GetWeekNoFromString(textCombo);
              textCombo = (char *) gtk_entry_get_text
                             (GTK_ENTRY(GTK_COMBO(comboEndWeekday)->entry));
              dst.endWeekday = GetWeekdayFromString(textCombo);
              textCombo = (char *) gtk_entry_get_text
                             (GTK_ENTRY(GTK_COMBO(comboEndMonth)->entry));
              dst.endMonth = GetMonthFromString(textCombo);
            }

            if (!isDST ||
                ((dst.startWeekNo != (-1)) && (dst.startWeekday != (-1)) &&
                 (dst.startMonth != (-1)) && (dst.endWeekNo != (-1)) &&
                 (dst.endWeekday != (-1)) && (dst.endMonth != (-1))))
            {
              setIsDST(isDST);
              if (isDST)
                setDSTstruct(&dst);

              break;
            }
            else
            {
              strcpy(errMsg, "");
              if (dst.startWeekNo == (-1)) 
                strcpy(errMsg, "Start Week No must be selected");
              if (dst.startWeekday == (-1))
                strcpy(errMsg, "Start Weekday must be selected");
              if (dst.startMonth == (-1))
                strcpy(errMsg, "Start Month must be selected");
              if (dst.endWeekNo == (-1))
                strcpy(errMsg, "End Week No must be selected");
              if (dst.endWeekday == (-1))
                strcpy(errMsg, "End Weekday must be selected");
              if (dst.endMonth == (-1))
                strcpy(errMsg, "End Month must be selected");
              if (strcmp(errMsg, "") != 0)
                DialogsDoMessage(errMsg);
            }           
          }
    }
    else
      break;
  }

  if (GTK_IS_DIALOG(dialog)) gtk_widget_destroy(GTK_WIDGET(dialog));

  if (idDSTConfigClicked == 1)
    return 1;
  else
    return 0;
}

