#include "config.h"

#include "date.h"
#include "gregor.h"

int month_list[12] =
  {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

int LastDayOfGregorianMonth(int month, int year)
{
  if ((month == 2) &&
      ((year % 4) == 0) &&
      ((year % 400) != 100) &&
      ((year % 400) != 200) &&
      ((year % 400) != 300))
    return 29;
  else
    return month_list[month-1];
}

long AbsoluteFromGregorian(calDate* date)
{
  long lValue, lReturnValue;
  int m;

  /* Days so far this month */
  lValue = date->day;
  lReturnValue = lValue;

  /* Days in prior months this year */
  for (m = 1; m < date->month; m++)
  {
    lValue = LastDayOfGregorianMonth(m, date->year);
    lReturnValue += lValue;
  }

  /* Days in prior years */
  lValue = (365 * (long) (date->year-1));
  lReturnValue += lValue;

  /* Julian leap days in prior years ... */
  lValue = ((date->year-1) / 4);
  lReturnValue += lValue;

  /* ... minus prior century years ... */
  lValue = ((date->year-1) / 100);
  lReturnValue -= lValue;

  /* ... plus prior years divisible by 400 */
  lValue = ((date->year-1) / 400);
  lReturnValue += lValue;

  return (lReturnValue);
}

void GregorianFromAbsolute(long lDate, calDate* date)
{
  int approx, y, m, day, month, year;
  long temp;
  calDate mydate;

  /* Approximation */
  approx = lDate/366;

  /* Search forward from the approximation */
  y = approx;
  for (;;)
  {
    mydate.day   = 1;
    mydate.month = 1;
    mydate.year  = y+1;
    temp = AbsoluteFromGregorian(&mydate);
    if (lDate < temp) break;
    y++;
  }
  year = y;

  /* Search forward from January */
  m = 1;
  for (;;)
  {
    mydate.day   = LastDayOfGregorianMonth(m, year);
    mydate.month = m;
    mydate.year  = year;
    temp = AbsoluteFromGregorian(&mydate);
    if (lDate <= temp) break;
    m++;
  }
  month = m;

  /* Calculate the day by subtraction */
  mydate.day   = 1;
  mydate.month = month;
  mydate.year  = year;
  temp = AbsoluteFromGregorian(&mydate);
  day = lDate-temp+1;

  date->day   = day;
  date->month = month;
  date->year  = year;
}

