/* Parameters: Sefardic, Ashkenazi */

#define HALACHA2_INTERNAL
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "gregor.h"
#include "hebrcal.h"
#include "settings.h"
#include "torah.h"
#include "torahids.h"
#include "halacha2.h"

int halachaDiaspora;

HALACHA2_EXTERN int isDay(long date, char *holiday)
{
  long dateHoliday;

  dateHoliday = h_holiday(date, holiday);
  if (dateHoliday == date)
    return 1;
  else
    return 0;
}
 
HALACHA2_EXTERN int day_in_month(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  return (dateHebrew.day);
}

/*
#define end_teyn_brokhe(day)
	(civil_year(day) % 4 = 3 ? December_5 : December_4)
*/
HALACHA2_EXTERN long end_teyn_brokhe(long absDate)
{
  calDate dateGregorian;
  long desiredDate;

  GregorForeignFromAbsolute(absDate, &dateGregorian);
  dateGregorian.day = ((dateGregorian.year % 4) == 3) ? 5 : 4;
  dateGregorian.month = 12;
  desiredDate = GregorAbsoluteFromForeign(&dateGregorian);
  return (desiredDate);
}

HALACHA2_EXTERN int month(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  return (dateHebrew.month);
}

HALACHA2_EXTERN int civil_year(long absDate)
{
  calDate date;

  GregorForeignFromAbsolute(absDate, &date);
  return (date.year);
}
  
HALACHA2_EXTERN long gregDate(int month, int day, long date)
{
  calDate date1, date2;

  GregorForeignFromAbsolute(date, &date1);
  date2.day = day;
  date2.month = month;
  date2.year = date1.year;
  return AbsoluteFromGregorian(&date2);
}

HALACHA2_EXTERN long hebrDate(int month, int day, long date)
{
  calDate date1, date2;

  HebrcalForeignFromAbsolute(date, &date1);
  date2.day = day;
  date2.month = month;
  date2.year = date1.year;
  return AbsoluteFromHebrew(&date2);
}

int h_holiday2(long absDate, char *nameHoliday, long *returnDate);

HALACHA2_EXTERN long h_holiday(long absDate, char *name)
{
  long returnDate;

  if (h_holiday2(absDate, name, &returnDate))
    return (returnDate);
  else
    return 0;
}

HALACHA2_EXTERN long h_holiday_prev(long absDate, char *name)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  if (dateHebrew.month >= 7) dateHebrew.year--;
  return h_holiday(HebrcalAbsoluteFromForeign(&dateHebrew), name);
}

int h_holiday2(long absDate, char *nameHoliday, long *returnDate)
{
  calDate dateHebrew;
  int hebrewYear, i;
  char name[256];

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  hebrewYear = dateHebrew.year;

  for (i = 0; i < strlen(nameHoliday); i++)
    name[i] = toupper(*(nameHoliday + i));
  name[strlen(nameHoliday)] = 0;

  if (strcmp(name, "ROSH_HASHANA_I") == 0) {
    dateHebrew.day = 1;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1; 
  }
  if (strcmp(name, "ROSH_HASHANA_II") == 0) {
    dateHebrew.day = 2;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if (strcmp(name, "TZOM_GEDALIA") == 0) {
    dateHebrew.day = 3;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    if (GetWeekday(*returnDate) == DAY_SATURDAY)
      *returnDate = *returnDate + 1;
    return 1;
  }
  if (strcmp(name, "YOM_KIPPUR") == 0) {
    dateHebrew.day = 10;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;  
  }
  if (strcmp(name, "SUKKOT_I") == 0) {
    dateHebrew.day = 15;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if (strcmp(name, "SUKKOT_II") == 0) {
    dateHebrew.day = 16;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if (strcmp(name, "SUKKOT_VI") == 0) {
    dateHebrew.day = 20;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if (strcmp(name, "SUKKOT_VII") == 0) {
    dateHebrew.day = 21;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if ((strcmp(name, "SHEMINI_ATSERES_I") == 0) ||
      (strcmp(name, "SHMINI_ATSERES_I") == 0)) {
    dateHebrew.day = 22;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if ((strcmp(name, "SHEMINI_ATSERES_II") == 0) ||
      (strcmp(name, "SHMINI_ATSERES_II") == 0)) {
    dateHebrew.day = 23;
    dateHebrew.month = 7;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }

  if (strcmp(name, "KHANNUKA_I") == 0) {
    dateHebrew.day = 25;
    dateHebrew.month = 9;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if (strcmp(name, "KHANNUKA_VII") == 0) {
    dateHebrew.day = 25;
    dateHebrew.month = 9;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew) + 7-1;
    return 1;
  }

  if (strcmp(name, "TZOM_TEVET") == 0) {
    dateHebrew.day = 10;
    dateHebrew.month = 10;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    if (GetWeekday(*returnDate) == DAY_SATURDAY)
      *returnDate = *returnDate + 1;
    return 1;
  }

  if (strcmp(name, "TAANIT_ESTHER") == 0) {
    dateHebrew.day = 14;
    if (!Hebrew_Leap_Year(hebrewYear))
      dateHebrew.month = 12;
    else
      dateHebrew.month = 13;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    if (GetWeekday(*returnDate) == DAY_SUNDAY)
      *returnDate = *returnDate - 3;
    else
      *returnDate = *returnDate - 1;
    return 1;
  }
  if (strcmp(name, "PURIM") == 0) {
    dateHebrew.day = 14;
    if (!Hebrew_Leap_Year(hebrewYear))
      dateHebrew.month = 12;
    else
      dateHebrew.month = 13;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }

  if (strcmp(name, "PESAKH_I") == 0) {
    dateHebrew.day = 15;
    dateHebrew.month = 1;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);  
    return 1;
  }
  if (strcmp(name, "PESAKH_II") == 0) {
    dateHebrew.day = 16;
    dateHebrew.month = 1;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if (strcmp(name, "PESAKH_VII") == 0) {
    dateHebrew.day = 21;
    dateHebrew.month = 1;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if (strcmp(name, "PESAKH_VIII") == 0) {
    dateHebrew.day = 22;
    dateHebrew.month = 1;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }

  if (strcmp(name, "SHAVUOT_I") == 0) {
    dateHebrew.day = 6;
    dateHebrew.month = 3;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }
  if (strcmp(name, "SHAVUOT_II") == 0) {
    dateHebrew.day = 7;
    dateHebrew.month = 3;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    return 1;
  }

  if (strcmp(name, "TZOM_TAMMUZ") == 0) {
    dateHebrew.day = 17;
    dateHebrew.month = 4;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    if (GetWeekday(*returnDate) == DAY_SATURDAY)
      *returnDate = *returnDate + 1;
    return 1;
  }

  if (strcmp(name, "TISHA_BEAV") == 0) {
    dateHebrew.day = 9;
    dateHebrew.month = 5;
    dateHebrew.year = hebrewYear;
    *returnDate = HebrcalAbsoluteFromForeign(&dateHebrew);
    if (GetWeekday(*returnDate) == DAY_SATURDAY)
      *returnDate = *returnDate + 1;
    return 1;
  }

  printf("Error: Day %s not found\n", name);

  return 0;
}

/* Rosh HaShanah I/II, Yom Kippur */
HALACHA2_EXTERN int high_holiday(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  if (dateHebrew.month == 7)
  {
    if ((dateHebrew.day == 1) || (dateHebrew.day == 2) ||
        (dateHebrew.day == 10))
      return 1;
  }

  return 0;
}

HALACHA2_EXTERN int rosh_khodesh(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  if ((dateHebrew.day == 1) && (dateHebrew.month == 7))
    return 0;
  else
  {
    if ((dateHebrew.day == 1) || (dateHebrew.day == 30))
      return 1;
    else
      return 0;
  }
}

HALACHA2_EXTERN int khol_haMoed(long absDate)
{
  calDate dateHebrew;
  int iBegin, iEnd;

  if (khol_haMoed_Pesakh(absDate))
    return 1;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  iBegin = (halachaDiaspora ? 17 : 16);
  iEnd = 21;
  if ((dateHebrew.month == 7) &&
      (dateHebrew.day >= iBegin) && (dateHebrew.day <= iEnd))
    return 1;
  else
    return 0;
}

HALACHA2_EXTERN int khol_haMoed_Pesakh(long absDate)
{
  calDate dateHebrew;
  int iBegin, iEnd;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  iBegin = (halachaDiaspora ? 17 : 16);
  iEnd = 20;
  if ((dateHebrew.month == 1) &&
      (dateHebrew.day >= iBegin) && (dateHebrew.day <= iEnd))
    return 1;
  else
    return 0;
}

HALACHA2_EXTERN int day_in_omer(long absDate)
{
  calDate dateHebrew, dateOmerStart, dateOmerEnd;
  long omerStartDate, omerEndDate;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);

  dateOmerStart.day = 16;
  dateOmerStart.month = 1;
  dateOmerStart.year = dateHebrew.year;
  omerStartDate = HebrcalAbsoluteFromForeign(&dateOmerStart);
  dateOmerEnd.day = 5;
  dateOmerEnd.month = 3;
  dateOmerEnd.year = dateHebrew.year;
  omerEndDate = HebrcalAbsoluteFromForeign(&dateOmerEnd);

  if ((absDate >= omerStartDate) && (absDate <= omerEndDate))
    return (absDate-omerStartDate+1);
  else
    return 0;
}

HALACHA2_EXTERN int tisha_beAv(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  if (dateHebrew.month == 5)
  {
    if ((GetWeekday(absDate) == DAY_SUNDAY) &&
        (dateHebrew.day == 10))
      return 1;
    if (dateHebrew.day == 9)
      return 1;
  }
  return 0;
}

HALACHA2_EXTERN int Av_15(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  if ((dateHebrew.day == 15) && (dateHebrew.month == 5))
    return 1;
  else
    return 0;
}

HALACHA2_EXTERN int khanukah(long absDate)
{
  calDate dateHebrew, dateStart;
  long startDate;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  dateStart.day = 25;
  dateStart.month = 9;
  dateStart.year = dateHebrew.year;
  startDate = HebrcalAbsoluteFromForeign(&dateStart);
  if ((absDate >= startDate) && (absDate <= startDate+7))
    return 1;
  else
    return 0;
}

HALACHA2_EXTERN int tu_beshvat(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  if ((dateHebrew.day == 15) && (dateHebrew.month == 11))
    return 1;
  else
    return 0;
}

HALACHA2_EXTERN int purim(long absDate)
{
  calDate dateHebrew, dateHoliday;
  long holidayDate;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);

  dateHoliday.day = 14;
  if (!Hebrew_Leap_Year(dateHebrew.year))
    dateHoliday.month = 12;
  else
    dateHoliday.month = 13;
  dateHoliday.year = dateHebrew.year;
  holidayDate = HebrcalAbsoluteFromForeign(&dateHoliday);
  if (holidayDate == absDate)
    return 1;
  else
    return 0;
}

HALACHA2_EXTERN int shushan_purim(long absDate)
{
  calDate dateHebrew, dateHoliday;
  long holidayDate;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);

  dateHoliday.day = 15;
  if (!Hebrew_Leap_Year(dateHebrew.year))
    dateHoliday.month = 12;
  else
    dateHoliday.month = 13;
  dateHoliday.year = dateHebrew.year;
  holidayDate = HebrcalAbsoluteFromForeign(&dateHoliday);

  if (postponeShushanPurimGetFlag())
  {
    if (GetWeekday(holidayDate) == DAY_SATURDAY)
      holidayDate++;
  }

  if (holidayDate == absDate)
    return 1;
  else
    return 0;
}

HALACHA2_EXTERN int adar_14(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  if (Hebrew_Leap_Year(dateHebrew.year) &&
      (dateHebrew.day == 14) && (dateHebrew.month == 12))
    return 1;
  else
    return 0;
}

HALACHA2_EXTERN int adar_15(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  if (Hebrew_Leap_Year(dateHebrew.year) &&
      (dateHebrew.day == 15) && (dateHebrew.month == 12))
    return 1;
  else
    return 0;
}

int Halacha2CheckForSpecialShabbat(long absDate, int id)
{
  int id1, id2, id3;

  if (GetWeekday(absDate) == DAY_SATURDAY)
  {
    if (GetTorahSectionShabbat(absDate, &id1, &id2, &id3))
    {
      if ((id1 == id) || (id2 == id) || (id3 == id))
        return 1;
    }
  }
  return 0;
}

HALACHA2_EXTERN int shabbat_zakhor(long absDate)
{
  return Halacha2CheckForSpecialShabbat(absDate, ID_ZAHOR);
}

HALACHA2_EXTERN int shabbat_parah(long absDate)
{
  return Halacha2CheckForSpecialShabbat(absDate, ID_PARAH);
}

HALACHA2_EXTERN int shabbat_haKhodesh(long absDate)
{
  return Halacha2CheckForSpecialShabbat(absDate, ID_HAHODESH);
}

HALACHA2_EXTERN int shabbat_shekalim(long absDate)
{
  return Halacha2CheckForSpecialShabbat(absDate, ID_SHEKALIM);
}

HALACHA2_EXTERN int rosh_haShana(long absDate)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);
  if ((dateHebrew.month == 7) &&
      ((dateHebrew.day == 1) || (dateHebrew.day == 2)))
    return 1;
  else
    return 0;
}

