#include "config.h"

#include "date.h"
#include "gregor.h"
#include "hebrcal.h"
#include "settings.h"
#include "holidays.h"

int GetJewishHolidayCount(void)
{
  return HOLIDAY_LAST+1;
}

void GetJewishHolidayName(int holiday, char *name)
{
  switch (holiday)
  {
    case 0: strcpy(name, "Rosh HaShana I"); break;
    case 1: strcpy(name, "Yom Kippur"); break;
    case 2: if (diasporaGetFlag())
              strcpy(name, "Sukkot I");
            else
              strcpy(name, "Sukkot");
            break;
    case 3: if (diasporaGetFlag())
              strcpy(name, "Sukkot II");
            else
              strcpy(name, "Khol HaMoed Sukkot");
            break;
    case 4: strcpy(name, "Khol HaMoed Sukkot"); break;
    case 5: strcpy(name, "Khol HaMoed Sukkot"); break;
    case 6: strcpy(name, "Khol HaMoed Sukkot"); break;
    case 7: strcpy(name, "Khol HaMoed Sukkot"); break;
    case 8: strcpy(name, "Hoshana Rabba"); break;
    case 9: if (diasporaGetFlag())
              strcpy(name, "Shemini Atseret");
            else
              strcpy(name, "Shemini Atseret/Simkhat Tora");
            break;
    case 10: strcpy(name, "Simkhat Tora"); break; /* (Diaspora) */
    case 11: strcpy(name, "Pesakh I"); break;
    case 12: if (diasporaGetFlag())
              strcpy(name, "Pesakh II");
            else
              strcpy(name, "Khol HaMoed Pesakh");
            break;
    case 13: strcpy(name, "Khol HaMoed Pesakh"); break;
    case 14: strcpy(name, "Khol HaMoed Pesakh"); break;
    case 15: strcpy(name, "Khol HaMoed Pesakh"); break;
    case 16: strcpy(name, "Khol HaMoed Pesakh"); break;
    case 17: strcpy(name, "Pesakh VII"); break;
    case 18: strcpy(name, "Khanukka I"); break;
    case 19: strcpy(name, "Khanukka II"); break;
    case 20: strcpy(name, "Khanukka III"); break;
    case 21: strcpy(name, "Khanukka IV"); break;
    case 22: strcpy(name, "Khanukka V"); break;
    case 23: strcpy(name, "Khanukka VI"); break;
    case 24: strcpy(name, "Khanukka VII"); break;
    case 25: strcpy(name, "Khanukka VIII"); break;
    case 26: if (diasporaGetFlag())
               strcpy(name, "Shavuot I");
             else
               strcpy(name, "Shavuot");
             break;
    case 27: strcpy(name, "Tu beShvat"); break;
    case 28: strcpy(name, "Purim"); break;
    case 29: strcpy(name, "Taanit Ester"); break;
    case 30: strcpy(name, "Tisha BeAv"); break;
    case 31: strcpy(name, "Tsom Gedalya"); break;
    case 32: strcpy(name, "Tsom Tevet"); break;
    case 33: strcpy(name, "Tsom Tammuz"); break;
    case 34: strcpy(name, "Shushan Purim"); break;
    case 35: strcpy(name, "Lag BeOmer"); break;
    case 36: strcpy(name, "Yom HaShoa"); break;
    case 37: strcpy(name, "Yom HaAtsmaut"); break;
    case 38: strcpy(name, "Yom Yerushalayim"); break;
    case 39: strcpy(name, "Shavuot II"); break; /* (Diaspora) */
    case 40: strcpy(name, "Pesakh VIII"); break; /* (Diaspora) */
    case 41: strcpy(name, "Shabbat HaGadol"); break;
    case 42: strcpy(name, "Rosh HaShana II"); break;
    case 43: strcpy(name, "Yom HaZikaron"); break;
    case 44: strcpy(name, "Purim Katan"); break;
    case 45: strcpy(name, "Shushan Purim Katan"); break;

    case 46: strcpy(name, "Erev Rosh HaShana"); break;
    case 47: strcpy(name, "Erev Yom Kippur"); break;
    case 48: strcpy(name, "Erev Sukkot"); break;
    case 49: strcpy(name, "Isru Khag"); break;
    case 50: strcpy(name, "Erev Pesakh"); break;
    case 51: strcpy(name, "Isru Khag"); break;
    case 52: strcpy(name, "Erev Shavuot"); break;
    case 53: strcpy(name, "Isru Khag"); break;
  }
}

void GetJewishHolidayDate(int holiday, int year, calDate* date)
{
  switch (holiday)
  {
    case 0: /* Rosh HaShanah */
      date->day   = 1;
      date->month = 7; /* Tishri */
      date->year  = year;
      break;

    case 1: /* Yom Kippur */
      date->day   = 10;
      date->month = 7; /* Tishri */
      date->year  = year;
      break;

    case 2: /* Sukkot */
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
      date->day   = 15 + holiday-2;
      date->month = 7; /* Tishri */
      date->year  = year;
      break;

    case 9: /* Shemini Azereth */
      date->day   = 22;
      date->month = 7; /* Tishri */
      date->year  = year;
      break;

    case 10: /* Simchat Torah (outside Israel) */
      if (diasporaGetFlag())
      {
        date->day   = 23;
        date->month = 7; /* Tishri */
        date->year  = year;
      }
      else
      {
        date->day   = 0;
        date->month = 0;
        date->year  = 0;
      }
      break;

    case 11: /* Passover */
    case 12:
    case 13:
    case 14:
    case 15:
    case 16:
    case 17:
      date->day   = 15 + holiday-11;
      date->month = 1; /* Nisan */
      date->year  = year;
      break;

    case 18: /* Chanukkah */
    case 19:
    case 20:
    case 21:
    case 22:
    case 23:
    case 24:
    case 25:
      date->day   = 25 + holiday-18;
      date->month = 9; /* Kislev */
      if (date->day > Last_Day_Of_Hebrew_Month(9, year))
      {
        date->day -= Last_Day_Of_Hebrew_Month(9, year);
        date->month++;
      }
      date->year  = year;
      break;

    case 26: /* Shavuot */
      date->day   = 6;
      date->month = 3; /* Sivan */
      date->year  = year;
      break;

    case 27: /* Tu-B'Shevat */
      date->day   = 15;
      date->month = 11; /* Shevat */
      date->year  = year;
      break;

    case 28: /* Purim */
      date->day = 14;
      if (!Hebrew_Leap_Year(year))
        date->month = 12; /* Adar */
      else
        date->month = 13; /* Adar II */
      date->year = year;
      break;

    case 29: /* Ta'anith Ester */
      {
        long lAbs;

        date->day = 14;
        if (!Hebrew_Leap_Year(year))
          date->month = 12; /* Adar */
        else
          date->month = 13; /* Adar II */
        date->year = year;
        lAbs = AbsoluteFromHebrew(date);
        if (GetWeekday(lAbs) == 0)
          lAbs -= 3;
        else
          lAbs--;
        HebrewFromAbsolute(lAbs, date);
      }
      break;

    case 30: /* Tisha B'Av */
      {
        long lAbs;

        date->day   = 9;
        date->month = 5; /* Av */
        date->year  = year;
        lAbs = AbsoluteFromHebrew(date);
        if (GetWeekday(lAbs) == 6)
        {
          lAbs++;
          HebrewFromAbsolute(lAbs, date);
        }
      }
      break;

    case 31: /* Tzom Gedaliah */
      {
        long lAbs;

        date->day   = 3;
        date->month = 7; /* Tishri */
        date->year  = year;
        lAbs = AbsoluteFromHebrew(date);
        if (GetWeekday(lAbs) == 6)
        {
          lAbs++;
          HebrewFromAbsolute(lAbs, date);
        }
      }
      break;

    case 32: /* Tzom Teveth */
      {
        long lAbs;

        date->day   = 10;
        date->month = 10; /* Teveth */
        date->year  = year;
        lAbs = AbsoluteFromHebrew(date);
        if (GetWeekday(lAbs) == 6)
        {
          lAbs++;
          HebrewFromAbsolute(lAbs, date);
        }
      }
      break;

    case 33: /* Tzom Tammuz */
      {
        long lAbs;

        date->day   = 17;
        date->month = 4; /* Tammuz */
        date->year  = year;
        lAbs = AbsoluteFromHebrew(date);
        if (GetWeekday(lAbs) == 6)
        {
          lAbs++;
          HebrewFromAbsolute(lAbs, date);
        }
      }
      break;

    case 34: /* Shushan Purim */
      {
        long lAbs;

        date->day = 15;
        if (!Hebrew_Leap_Year(year))
          date->month = 12; /* Adar */
        else
          date->month = 13; /* Adar II */
        date->year = year;
        lAbs = AbsoluteFromHebrew(date);

        /* Postponement is chooseable */
        if (postponeShushanPurimGetFlag())
        {
          if (GetWeekday(lAbs) == 6)
          {
            lAbs++;
            HebrewFromAbsolute(lAbs, date);
          }
        }
      }
      break;

    case 35: /* Lag B'Omer */
      date->day   = 18;
      date->month = 2; /* Iyyar */
      date->year  = year;
      break;

    case 36: /* Yom Hashoah (Kohn) */
      {
        long lAbs;

        date->day   = 27;
        date->month = 1; /* Nisan */
        date->year  = year;
        lAbs = AbsoluteFromHebrew(date);
        if (GetWeekday(lAbs) == 6) /* if it is Saturday... */
          lAbs -= 2;              
        else
          if (GetWeekday(lAbs) == 5) /* ...or if it is Friday... */
            lAbs--; /* ...then choose Thursday */
        if (date->year >= 5757) /* Reingold, Errata (since year 1997) */
        {
          if (GetWeekday(lAbs) == 0) /* if it is Sunday */
            lAbs++; /* postpone it to the following day (Monday) */
        }
        HebrewFromAbsolute(lAbs, date);
      }
      break;

    case 37: /* Yom Haatzmaut (Kohn) */
      {
        long lAbs;

        date->day   = 5;
        date->month = 2; /* Iyyar */
        date->year  = year;
        lAbs = AbsoluteFromHebrew(date);
        if (GetWeekday(lAbs) == 6) /* if it is Saturday... */
          lAbs -= 2;              
        else
          if (GetWeekday(lAbs) == 5) /* ...or if it is Friday... */
            lAbs--; /* ...then choose Thursday */

        if (year >= 5764) /* Since 2004 ... */
        {
          if (GetWeekday(lAbs) == 1) /* ... when 4 Iyar falls on Sunday */
                                     /* (5 Iyar on Monday) ... */
            lAbs++; /* ... postpone Yom Haatzmaut by one day */
        }

        HebrewFromAbsolute(lAbs, date);
      }
      break;

    case 38: /* Yom Yerushalayim */
      date->day   = 28;
      date->month = 2; /* Iyyar */
      date->year  = year;
      break;

    case 39: /* Shavuot #2 (outside Israel) */
      if (diasporaGetFlag())
      {
        date->day   = 7;
        date->month = 3; /* Sivan */
        date->year  = year;
      }
      else
      {
        date->day   = 0;
        date->month = 0;
        date->year  = 0;
      }
      break;

    case 40: /* Passover #8 (outside Israel) */
      if (diasporaGetFlag())
      {
        date->day   = 15 + 8-1;
        date->month = 1; /* Nisan */
        date->year  = year;
      }
      else
      {
        date->day   = 0;
        date->month = 0;
        date->year  = 0;
      }
      break;

    case 41: /* The Great Shabbat */
    /* NOTE: It is the shabbat before Pessach. If Pessach
             falls on Saturday, The Great Shabbat is on
             the previous Saturday. */ 
      {
        long lAbs;

        date->day   = 15;
        date->month = 1; /* Nisan */
        date->year  = year;
        lAbs = AbsoluteFromHebrew(date);
        lAbs--;
        while (GetWeekday(lAbs) != 6)
          lAbs--;
        HebrewFromAbsolute(lAbs, date);
      }
      break;

    case 42: /* Rosh HaShanah (2) */
      date->day   = 2;
      date->month = 7; /* Tishri */
      date->year  = year;
      break;

    case 43: /* Yom Hazicaron (Reingold) */
      {
        long lAbs;

        date->day   = 4;
        date->month = 2; /* Iyyar */
        date->year  = year;
        lAbs = AbsoluteFromHebrew(date);
        if (GetWeekday(lAbs) == 5) /* if it is Friday... */
          lAbs -= 2;              
        else
          if (GetWeekday(lAbs) == 4) /* ...or if it is Thursday... */
            lAbs--; /* ...then choose Wednesday */

        if (year >= 5764) /* Since 2004 ... */
        {
          if (GetWeekday(lAbs) == 0) /* If 4 Iyar falls on Sunday ... */
            lAbs++; /* ... postpone Yom Hazicaron by one day */
        }

        HebrewFromAbsolute(lAbs, date);
      }
      break;

    case HOLIDAY_PURIM_KATAN_1:
      {
        date->day = 14;
        if (Hebrew_Leap_Year(year))
          date->month = 12;
        else
          date->month = 13;
        date->year = year;
      }
      break;

    case HOLIDAY_PURIM_KATAN_2:
      {
        date->day = 15;
        if (Hebrew_Leap_Year(year))
          date->month = 12;
        else
          date->month = 13;
        date->year = year;
      }
      break;

    case HOLIDAY_EREV_ROSH_HASHANAH:
      date->day = 29;
      date->month = 6;
      date->year = year;
      break;

    case HOLIDAY_EREV_YOM_KIPPUR:
      date->day = 9;
      date->month = 7;
      date->year = year;
      break;

    case HOLIDAY_EREV_SUKKOT:
      date->day = 14;
      date->month = 7;
      date->year = year;
      break;

    case HOLIDAY_ISRU_CHAG_SUKKOT:
      if (diasporaGetFlag())
        date->day = 24;
      else
        date->day = 23;
      date->month = 7;
      date->year = year;
      break;

    case HOLIDAY_EREV_PESACH:
      date->day = 14;
      date->month = 1;
      date->year = year;
      break;

    case HOLIDAY_ISRU_CHAG_PESACH:
      if (diasporaGetFlag())
        date->day = 23;
      else
        date->day = 22;
      date->month = 1;
      date->year = year;
      break;

    case HOLIDAY_EREV_SHAVUOT:
      date->day = 5;
      date->month = 3; 
      date->year = year;
      break;

    case HOLIDAY_ISRU_CHAG_SHAVUOT:
      if (diasporaGetFlag())
        date->day = 8;
      else
        date->day = 7;
      date->month = 3;
      date->year = year;
      break;
  }
}

int GetJewishHolidayFirstYear(int holiday)
{
  switch (holiday)
  {
    case 36: /* Yom Hashoah */
      return 1951;

    case 37: /* Yom Haatzmaut */
      return 1949;

    case 38: /* Yom Yerushalayim */
      return 1969;

    case 43: /* Yom Hazicaron */
      return 1949;

    default:
      return 0;
  }
}

void GetSpecialDay(long absDate, char *name)
{
  calDate dateCurrentGregorian, dateCurrentHebrew, dateHoliday;
  int countHolidays, holidayCounter, firstYear, permitHoliday;

  GregorForeignFromAbsolute(absDate, &dateCurrentGregorian);
  HebrcalForeignFromAbsolute(absDate, &dateCurrentHebrew);
  strcpy(name, "");

  countHolidays = GetJewishHolidayCount();
  for (holidayCounter = 0; holidayCounter < countHolidays; holidayCounter++)
  {
    firstYear = GetJewishHolidayFirstYear(holidayCounter);
    if (firstYear == 0)
      permitHoliday = 1;
    else
    {
      if (dateCurrentGregorian.year >= firstYear)
        permitHoliday = 1;
      else
        permitHoliday = 0;
    }
    if (permitHoliday)
    {
      GetJewishHolidayDate(holidayCounter, dateCurrentHebrew.year,
                           &dateHoliday);
      if (dateHoliday.day != 0)
      {
        if ((dateHoliday.day == dateCurrentHebrew.day) &&
            (dateHoliday.month == dateCurrentHebrew.month) &&
            (dateHoliday.year == dateCurrentHebrew.year))
        {
          GetJewishHolidayName(holidayCounter, name);
        }
      }
    }
  }

  if ((dateCurrentHebrew.day == 1) && (dateCurrentHebrew.month == 7))
    ;
  else
  {
    if ((dateCurrentHebrew.day == 30) || (dateCurrentHebrew.day == 1))
    {
      if (strcmp(name, "") != 0)
        strcat(name, ", ");
      strcat(name, "Rosh Khodesh");
    }
  }
}

