#define INI_INTERNAL
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "config.h"
#include "ini.h"

SETTINGS_EXTERN void SettingsGetHomePath(char *homePath)
{
  char* home;

  home = (char *) getenv("HOME");
  if (home)
    strcpy(homePath, home);
  else
    strcat(homePath, ".");
}

int SettingsReadLine(FILE* stream, char *line)
{
  char *lineLauf, ch;

  if (fread(&ch, 1, 1, stream) < 1)
    return 0;

  lineLauf = line;
  for (;;)
  {
    if (ch == 10)
      break;
    *lineLauf++ = ch;
    if (fread(&ch, 1, 1, stream) < 1)
      break;
  }
  *lineLauf = 0;
  return 1;
}

void SettingsParseLine(char *line, char *key, char *value)
{
  char *lineLauf, *keyLauf, *valueLauf;

  strcpy(key, "");
  strcpy(value, "");

  lineLauf = line;
  while (*lineLauf == ' ') lineLauf++;
  if (*lineLauf != 0)
  {
    keyLauf = key;
    while (((*lineLauf >= 'A') && (*lineLauf <= 'Z')) ||
           ((*lineLauf >= 'a') && (*lineLauf <= 'z')))
    {
      *keyLauf++ = *lineLauf;
      lineLauf++;
    }
    *keyLauf = 0;
    while (*lineLauf == ' ') lineLauf++;
    if (*lineLauf == ':')
    {
      lineLauf++;
      while (*lineLauf == ' ') lineLauf++;
      if (*lineLauf == '"')
      {
        lineLauf++;
        valueLauf = value;
        while ((*lineLauf != '"') && (*lineLauf != 0))
        {
          *valueLauf++ = *lineLauf;
          lineLauf++;
        }
        *valueLauf = 0;
      }
    }
  }
}

SETTINGS_EXTERN void SettingsGet(char *key, char *value, char *defaultValue)
{
  char filename[256], line[256], lineKey[256], lineValue[256];
  FILE* stream;

  SettingsGetHomePath(filename);
  if (strlen(filename) > 0)
  {
    if (filename[strlen(filename)-1] != '/')
      strcat(filename, "/");
  }
  strcat(filename, SETTINGS_FILENAME);

  strcpy(value, defaultValue);

  stream = fopen(filename, "rb");
  if (stream)
  {
    while (SettingsReadLine(stream, line))
    {
      SettingsParseLine(line, lineKey, lineValue);
      if (strcmp(lineKey, key) == 0)
      {
        strcpy(value, lineValue);
        break;
      }
    }
    fclose(stream);
  }
}

void SettingsGetTempFileName(char *tempFileName)
{
  if (!tmpnam(tempFileName))
    strcpy(tempFileName, "/tmp/settings.temp");
}

void SettingsWriteLine(FILE* stream, char *key, char *value)
{
  char line[256];

  sprintf(line, "%s: \"%s\"\n", key, value);
  fwrite(line, strlen(line), 1, stream);
}

SETTINGS_EXTERN void SettingsSet(char *key, char *value)
{
  char filename[256], tempFilename[256], line[256];
  char lineKey[256], lineValue[256];
  FILE *stream1, *stream2;
  int written;

  SettingsGetHomePath(filename);
  if (strlen(filename) > 0)
  {
    if (filename[strlen(filename)-1] != '/')
      strcat(filename, "/");
  }
  strcat(filename, SETTINGS_FILENAME);

  SettingsGetTempFileName(tempFilename);
  stream1 = fopen(tempFilename, "wb");
  if (stream1)
  {
    stream2 = fopen(filename, "rb");
    if (stream2)
    {
      written = 0;
      while (SettingsReadLine(stream2, line))
      {
        SettingsParseLine(line, lineKey, lineValue);
        if (strcmp(lineKey, key) == 0)
        {
          SettingsWriteLine(stream1, key, value);
          written = 1;
        }
        else
          SettingsWriteLine(stream1, lineKey, lineValue);
      }
      if (!written)
        SettingsWriteLine(stream1, key, value);
      fclose(stream2);
    }
    else
      SettingsWriteLine(stream1, key, value);
    fclose(stream1);
    unlink(filename);
    rename(tempFilename, filename);
  }
  else
    fprintf(stderr, "Fatal error: Could not create temporary "
            "file %s\n", tempFilename);
}

SETTINGS_EXTERN void SettingsSetInt(char *key, int value)
{
  char valueString[256];

  sprintf(valueString, "%d", value);
  SettingsSet(key, valueString);
}

SETTINGS_EXTERN int SettingsGetInt(char *key, int defaultValue)
{
  char defaultValueString[256], value[256];

  sprintf(defaultValueString, "%d", defaultValue);
  SettingsGet(key, value, defaultValueString);
  return atoi(value);
}

