#include <gtk/gtk.h>
#include "config.h"
#include "version.h"
#include "gregor.h"
#include "names.h"
#include "settings.h"
#include "paint.h"
#include "dialogs.h"
#include "shabconf.h"
#include "annidlg.h"
#include "annifile.h"
#include "dstconf.h"
#include "zmanim.h"
#include "zmanimdlg.h"
#include "halacha.h"

GtkWindow *gMainWindow;

#define MENUSTRING_ANNI             "Annivers./Birthd./" YAHRZEIT_ABBREV
#define MENUSTRING_ISRAELDIASPORASTYLE   "Set Israel/Diaspora & Style..."
#define MENUSTRING_SHABBATTIMES     "Configure Shabbat Times..."
#define MENUSTRING_CONFIGZMANIM     "Configure Zmanim..."
#define MENUSTRING_DST              "Daylight Savings Time..."
#define MENUSTRING_EXIT             "Exit"
#define MENUSTRING_ABOUT            "About..."

void MainToday(void)
{
  time_t currentTime;
  struct tm *currentTM;

  currentTime = time(NULL);
  currentTM = localtime(&currentTime);
  gCurrentDay = currentTM->tm_mday;
  gCurrentMonth = currentTM->tm_mon + 1;
  gCurrentYear = currentTM->tm_year + 1900;

  if (gCurrentYear < YEAR_BEGIN) gCurrentYear = YEAR_BEGIN;
  if (gCurrentYear > YEAR_END) gCurrentYear = YEAR_END; 
}

int gMenuActivate;

void MainMenuDestroy(GtkWidget* widget, gpointer data)
{
  if (gMenuActivate == (-1))
    gMenuActivate = 0;
}

void MainMenuActivate(GtkWidget* widget, gpointer data)
{
  gMenuActivate = (int) data;
}

#define IDM_EXIT                       1
#define IDM_ABOUT                      2
#define IDM_CONFIGURE_ISRAELDIASPORA_STYLE   3
#define IDM_ANNI                       4
#define IDM_CONFIGURE_SHABBATTIMES     5
#define IDM_CONFIGURE_DST              6
#define IDM_CONFIGURE_ZMANIM           7

char *textHalacha;

void HalachaOutput(char *msg)
{
  strcat(textHalacha, msg);
  strcat(textHalacha, "\n");
}

void showYom(void)
{
  char *textZmanim;
  calDate date;
  long absDate;

  textZmanim = malloc(20480);
  textHalacha = malloc(20480);
  date.day = gCurrentDay;
  date.month = gCurrentMonth;
  date.year = gCurrentYear;
  absDate = GregorAbsoluteFromForeign(&date);
  GetZmanimForDay(absDate, textZmanim);
  sprintf(textHalacha, "Halakha for %d.%d.%d:\n",
          gCurrentDay, gCurrentMonth, gCurrentYear);
  HalachaDo(absDate, diasporaGetFlag(), getStyle());  
  DialogsDoLongMessage(gMainWindow, textZmanim, textHalacha);
  free(textHalacha);
  free(textZmanim);
}

void MainWindowDestroy(GtkWidget* widget, GdkEvent *event, gpointer data);

typedef struct tagMenuStringRec
{
  char str[256];
  int id;
} MenuStringRec;

MenuStringRec MenuStrings[] = {
  {MENUSTRING_ANNI,            IDM_ANNI},
  {MENUSTRING_ISRAELDIASPORASTYLE,  IDM_CONFIGURE_ISRAELDIASPORA_STYLE},
  {MENUSTRING_SHABBATTIMES,    IDM_CONFIGURE_SHABBATTIMES},
  {MENUSTRING_CONFIGZMANIM,    IDM_CONFIGURE_ZMANIM},
  {MENUSTRING_DST,             IDM_CONFIGURE_DST},
  {MENUSTRING_EXIT,            IDM_EXIT},
  {MENUSTRING_ABOUT,           IDM_ABOUT},
  {"", 0}};

void MainWindowButtonPress(GtkWidget* widget,
                           GdkEventButton* event,
                           gpointer data)
{
  PaintItTest pit;
  PaintItPart pip;
  GtkMenu* menu;
  GtkMenuItem* menuItem;
  int xMenu, yMenu, cxMenu, cyMenu;
  int xToday, yToday, cxToday, cyToday;
  int xYom, yYom, cxYom, cyYom;
  int cxWindow, cyWindow;
  int i, iLastDay, startYear, endYear, year;
  char text[256], menuString[256], yomString[256];
  char todayString[256], fontName[256];
  GdkFont *font;
  calDate dateCurrent;
  long currentDate;
  
  gdk_window_get_geometry(event->window, NULL, NULL,
                          &cxWindow, &cyWindow,
                          NULL);

  generateFontName("helvetica", 10, "medium", fontName);
  font = gdk_font_load(fontName);
  GetMenuButton(font, menuString, &xMenu, &yMenu, &cxMenu, &cyMenu);

  if ((event->x >= xMenu) && (event->x <= xMenu + cxMenu))
  {
    if ((event->y >= yMenu) && (event->y <= yMenu + cyMenu))
    {
      gMenuActivate = (-1);
      menu = GTK_MENU(gtk_menu_new());
      gtk_signal_connect(GTK_OBJECT(menu), "hide",
                        GTK_SIGNAL_FUNC(MainMenuDestroy), NULL);

      i = 0;
      for (;;)
      {
        if (MenuStrings[i].id == 0)
          break;

        menuItem = GTK_MENU_ITEM(gtk_menu_item_new_with_label
                       (MenuStrings[i].str));
        gtk_signal_connect(GTK_OBJECT(menuItem), "activate",
                           GTK_SIGNAL_FUNC(MainMenuActivate),
                           (gpointer) MenuStrings[i].id);
        gtk_widget_show(GTK_WIDGET(menuItem));
        gtk_menu_append(menu, GTK_WIDGET(menuItem));

        i++;
      }
      gtk_widget_show(GTK_WIDGET(menu));
      gtk_menu_popup(menu, NULL, NULL, NULL, NULL, event->button, event->time);
      while (gMenuActivate == (-1))
      {
/*      if (gtk_events_pending())*/
          gtk_main_iteration();
      }
      if (gMenuActivate)
      {
        switch (gMenuActivate)
        {
          case IDM_EXIT:
            MainWindowDestroy(NULL, NULL, NULL);
            break;
           case IDM_ABOUT:
             DialogsDoAbout();
             break;
          case IDM_CONFIGURE_ISRAELDIASPORA_STYLE:
            if (DialogsConfigureIsraelDiasporaStyle())
              gtk_widget_queue_draw(widget);
            break;
          case IDM_ANNI:
            AnniDoDialog();
            gtk_widget_queue_draw(widget);
            break;
          case IDM_CONFIGURE_SHABBATTIMES:
            if (DoConfigureShabbatTimes())
              gtk_widget_queue_draw(widget);
            break;
          case IDM_CONFIGURE_DST:
            if (DoConfigureDST())
              gtk_widget_queue_draw(widget);
            break;
          case IDM_CONFIGURE_ZMANIM:
            ZmanimdlgDoDialog();
            break;
        }
      }    
      return;
    }
  }

  generateFontName("helvetica", 10, "medium", fontName);
  font = gdk_font_load(fontName);
  GetTodayButton(font, todayString, &xToday, &yToday, &cxToday, &cyToday,
                 cxWindow);

  if ((event->x >= xToday) && (event->x <= xToday + cxToday))
  {
    if ((event->y >= yToday) && (event->y <= yToday + cyToday))
    {
      MainToday();
      gtk_widget_queue_draw(widget);
      return;
    }
  }

  generateFontName("helvetica", 10, "medium", fontName);
  font = gdk_font_load(fontName);
  GetYomButton(font, yomString, &xYom, &yYom, &cxYom, &cyYom);

  if ((event->x >= xYom) && (event->x <= xYom + cxYom))
  {
    if ((event->y >= yYom) && (event->y <= yYom + cyYom))
    {
      showYom();
      return;
    }
  }

  pit.x = event->x;
  pit.y = event->y;
  PaintIt(event->window, FALSE, &pit, NULL);

  if (pit.returnDay != 0)
  {
    pip.iDay1 = gCurrentDay;
    gCurrentDay = pit.returnDay;
    pip.iDay2 = gCurrentDay;
    PaintIt(event->window, TRUE, NULL, &pip);

    if (pit.returnDayAnni)
    {
      dateCurrent.day = gCurrentDay;
      dateCurrent.month = gCurrentMonth;
      dateCurrent.year = gCurrentYear;
      currentDate = GregorAbsoluteFromForeign(&dateCurrent);
      if (AnnifileGetForDay(currentDate, NULL, text))
        DialogsDoMessage(text);
    }

    return;
  }

  if (pit.returnIsMonth)
  {
    gMenuActivate = (-1);
    menu = GTK_MENU(gtk_menu_new());
    gtk_signal_connect(GTK_OBJECT(menu), "hide",
                       GTK_SIGNAL_FUNC(MainMenuDestroy), NULL);
    for (i = 1; i <= 12; i++)
    {
      GetGregorianMonthName(i, text);
      menuItem = GTK_MENU_ITEM(gtk_menu_item_new_with_label(text));
      gtk_signal_connect(GTK_OBJECT(menuItem), "activate",
                         GTK_SIGNAL_FUNC(MainMenuActivate), (gpointer) i);
      gtk_widget_show(GTK_WIDGET(menuItem));
      gtk_menu_append(menu, GTK_WIDGET(menuItem));
    }
    gtk_widget_show(GTK_WIDGET(menu));
    gtk_menu_popup(menu, NULL, NULL, NULL, NULL, event->button, event->time);
    while (gMenuActivate == (-1))
    {
/*    if (gtk_events_pending())*/
        gtk_main_iteration();
    }
    if (gMenuActivate)
    {
      gCurrentMonth = gMenuActivate;
      iLastDay = GregorGetLastDayOfMonth(gCurrentMonth, gCurrentYear);
      if (gCurrentDay > iLastDay)
        gCurrentDay = iLastDay;
      gtk_widget_queue_draw(widget);
    }
    return;
  }

  if (pit.returnIsYear)
  {
    gMenuActivate = (-1);
    menu = GTK_MENU(gtk_menu_new());
    startYear = gCurrentYear - 2;
    if (startYear < YEAR_BEGIN) startYear = YEAR_BEGIN;
    endYear = gCurrentYear + 2;
    if (endYear > YEAR_END) endYear = YEAR_END;
    gtk_signal_connect(GTK_OBJECT(menu), "hide",
                       GTK_SIGNAL_FUNC(MainMenuDestroy), NULL);
    for (year = startYear; year <= endYear; year++)
    {
      sprintf(text, "%d", year);
      menuItem = GTK_MENU_ITEM(gtk_menu_item_new_with_label(text));
      gtk_signal_connect(GTK_OBJECT(menuItem), "activate",
                         GTK_SIGNAL_FUNC(MainMenuActivate), (gpointer) year);
      gtk_widget_show(GTK_WIDGET(menuItem));
      gtk_menu_append(menu, GTK_WIDGET(menuItem));
    }
    menuItem = GTK_MENU_ITEM(gtk_menu_item_new_with_label("Other..."));
    gtk_signal_connect(GTK_OBJECT(menuItem), "activate",
                       GTK_SIGNAL_FUNC(MainMenuActivate), (gpointer) 1);
    gtk_widget_show(GTK_WIDGET(menuItem));
    gtk_menu_append(menu, GTK_WIDGET(menuItem));
    gtk_widget_show(GTK_WIDGET(menu));
    gtk_menu_popup(menu, NULL, NULL, NULL, NULL, event->button, event->time);
    while (gMenuActivate == (-1))
    {
/*    if (gtk_events_pending())*/
        gtk_main_iteration();
    }
    if (gMenuActivate)
    {
      if (gMenuActivate != 1)
      {
        gCurrentYear = gMenuActivate;
        iLastDay = GregorGetLastDayOfMonth(gCurrentMonth, gCurrentYear);
        if (gCurrentDay > iLastDay)
          gCurrentDay = iLastDay;
        gtk_widget_queue_draw(widget);
      }
      else
      {
        year = DialogsDoYear(gCurrentYear);
        if (year)
        {
          gCurrentYear = year;
          iLastDay = GregorGetLastDayOfMonth(gCurrentMonth, gCurrentYear);
          if (gCurrentDay > iLastDay)
            gCurrentDay = iLastDay;
          gtk_widget_queue_draw(widget);
        }
      }
    }
    return;
  }

  if (pit.returnIsPrev || pit.returnIsNext)
  {
    if (pit.returnIsPrev)
    {
      if (gCurrentMonth == 1)
      {
        if (gCurrentYear > YEAR_BEGIN)
        {
          gCurrentMonth = 12;
          gCurrentYear--;
        }
      }
      else
        gCurrentMonth--;
    }
    if (pit.returnIsNext)
    {
      if (gCurrentMonth == 12)
      {
        if (gCurrentYear < YEAR_END)
        {
          gCurrentMonth = 1;
          gCurrentYear++;
        }
      }
      else
        gCurrentMonth++;
    }
    iLastDay = GregorGetLastDayOfMonth(gCurrentMonth, gCurrentYear);
    if (gCurrentDay > iLastDay)
      gCurrentDay = iLastDay;
    gtk_widget_queue_draw(widget);
  }
}

void MainWindowDestroy(GtkWidget* widget, GdkEvent *event, gpointer data)
{
  settingsDone();

  gtk_main_quit();
}

int main(int argc, char *argv[])
{
  GtkWindow *window;
  GtkWidget *vbox, *drawingArea;

  gtk_init(&argc, &argv);

  gdk_rgb_init();

  settingsInit();
  MainToday();
  
  window = GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
  gMainWindow = window;
  gtk_widget_set_usize(GTK_WIDGET(window), INITIAL_WIDTH, INITIAL_HEIGHT);
  gtk_window_set_title(GTK_WINDOW(window), APPTITLE);

  vbox = gtk_vbox_new(FALSE, 1);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 1);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  gtk_widget_show(vbox);

  drawingArea = gtk_drawing_area_new();
  gtk_signal_connect(GTK_OBJECT(drawingArea), "expose-event",
                     GTK_SIGNAL_FUNC(MainWindowPaint), NULL);
  gtk_signal_connect(GTK_OBJECT(drawingArea), "button-press-event",
                     GTK_SIGNAL_FUNC(MainWindowButtonPress), NULL);
  gtk_widget_add_events(GTK_WIDGET(drawingArea), GDK_BUTTON_PRESS_MASK);
  gtk_box_pack_start(GTK_BOX(vbox), drawingArea, TRUE, TRUE, 0);
  gtk_widget_show(drawingArea);

  gtk_signal_connect(GTK_OBJECT(window), "destroy",
                     GTK_SIGNAL_FUNC(MainWindowDestroy), NULL);

  gtk_widget_show(GTK_WIDGET(window));

  gtk_main();

  return 0;
}

