#define MOLAD_INTERNAL
#include "config.h"
#include "hebrcal.h"
#include "names.h"
#include "molad.h"

void MoladMultiplyLarge(int n, int days, int hours, int parts,
                        int* lpiReturnDays,  
                        int* lpiReturnHours, 
                        int* lpiReturnMin, 
                        int* lpiReturnParts)
{
  LargeInteger llLarge, llLarge2;

  llLarge = (2 * (24*1080) + 5 * 1080 + 204);
  llLarge2 = (days * (24*1080) + hours * 1080 + parts);
  llLarge += (llLarge2 * n);
  *lpiReturnParts = llLarge % 18;
  llLarge /= 18;
  *lpiReturnMin = llLarge % 60;
  llLarge /= 60;
  *lpiReturnHours = llLarge % 24;
  llLarge /= 24;
  *lpiReturnDays = llLarge;
}

MOLAD_EXTERN int MoladGet(int iMonth, int iYear,
                          int* lpiWeekday,
                          int* lpiHour, int* lpiMin, int* lpiParts)
{
  int nMonthElapsed, iNumberOfCycles, iNumberInCycle;
  int iCycleNo, iCurrentMonth;
  int days, hours, min, parts, weekday;

  if ((iMonth < 1) || (iMonth > 13)) return 0;
  if (!Hebrew_Leap_Year(iYear))
  {
    if (iMonth == 13) return 0;
  }
  if (iYear < 1) return 0;
  if (!lpiHour) return 0;
  if (!lpiMin) return 0;
  if (!lpiParts) return 0;

  nMonthElapsed = 0;

  iNumberOfCycles = (iYear-1) / 19;
  nMonthElapsed += (iNumberOfCycles * 235);

  iNumberInCycle = (iYear-1) % 19;
  for (iCycleNo = 0; iCycleNo < iNumberInCycle; iCycleNo++)
  {
    if (Hebrew_Leap_Year((iNumberOfCycles * 19) + 1 + iCycleNo))
      nMonthElapsed += 13;
    else
      nMonthElapsed += 12;
  }

  iCurrentMonth = 7;
  while (iCurrentMonth != iMonth)  
  {
    nMonthElapsed++;
    iCurrentMonth++;
    if (iCurrentMonth == 7)
      return 0;
    if (Hebrew_Leap_Year(iYear))
    {
      if (iCurrentMonth > 13)
        iCurrentMonth = 1;
    }
    else
    {
      if (iCurrentMonth > 12)
        iCurrentMonth = 1;
    }
  }

  MoladMultiplyLarge(nMonthElapsed, 29, 12, 793, &days, &hours, &min, &parts);
  weekday = (days % 7);
  weekday = ((weekday+6) % 7);
  *lpiHour = (hours - 6);
  if (*lpiHour < 0)
  {
    *lpiHour = *lpiHour + 24;
    if (weekday == 0)
      weekday = 6;
    else
      weekday--;
  }
  *lpiMin = min;
  *lpiParts = parts;
  *lpiWeekday = weekday;
  return 1;
}

MOLAD_EXTERN void MoladFormatString(char *dest1, char *dest2, char *dest3,
                                    int iMonth, int iYear,
                                    int iWeekday, 
                                    int iHour, int iMin, int iParts)
{
  int iHour2;
  char dayNight[256], month[256], weekday[256];

  iHour2 = iHour;
  if (iHour2 > 12)
    iHour2 = iHour2-12;
  if (iHour2 == 0) iHour2 = 12;
  if ((iHour < 6) || (iHour >= 18))
    strcpy(dayNight, "night");
  else
    strcpy(dayNight, "day");

  GetHebrewMonthName(iMonth, iYear, month);
  GetWeekdayName(iWeekday, weekday);
  sprintf(dest1, "Molad of %s %d:",
          month, iYear);
  sprintf(dest2, "%s, %s, %d hours,",
          weekday, dayNight, iHour2);
  sprintf(dest3, "%d minutes, %d parts",
          iMin, iParts);
}

MOLAD_EXTERN void MoladRetrieveNextSimple(char *molad, int space,
                                          long absDate)
{
  calDate dateHebrew;
  char str1[256], str2[256], str3[256];
  int i, iMonth, iYear, iWeekday, iHour, iMin, iParts;

  HebrcalForeignFromAbsolute(absDate, &dateHebrew);

  iMonth = dateHebrew.month + 1;
  iYear = dateHebrew.year;
  if (iMonth > HebrcalGetLastMonthOfYear(dateHebrew.year))
    iMonth = 1;
  if (iMonth == 7)
    iYear++;

  if (MoladGet(iMonth, iYear, &iWeekday, &iHour, &iMin, &iParts))
    MoladFormatString(str1, str2, str3, iMonth, iYear,
                      iWeekday, iHour, iMin, iParts);
  else
  {
    strcpy(str1, "");
    strcpy(str2, "");
    strcpy(str3, "");
  }

  strcpy(molad, "");
  for (i = 0; i < space; i++)
    strcat(molad, " ");
  strcat(molad, str1);
  strcat(molad, "\n");
  for (i = 0; i < (space+1); i++)
    strcat(molad, " ");
  strcat(molad, str2);
  strcat(molad, "\n");
  for (i = 0; i < (space+1); i++)
    strcat(molad, " ");
  strcat(molad, str3);
}

