#include "config.h"

#include "date.h"
#include "hebrcal.h"
#include "torahids.h"
#include "names.h"

void GetWeekdayName(int weekday, char *name)
{
  switch (weekday)
  {
    case DAY_SUNDAY: strcpy(name, "Sunday"); break;
    case DAY_MONDAY: strcpy(name, "Monday"); break;
    case DAY_TUESDAY: strcpy(name, "Tuesday"); break;
    case DAY_WEDNESDAY: strcpy(name, "Wednesday"); break;
    case DAY_THURSDAY: strcpy(name, "Thursday"); break;
    case DAY_FRIDAY: strcpy(name, "Friday"); break;
    case DAY_SATURDAY: strcpy(name, "Saturday"); break;
  }
}

void GetWeekdayNameShort(int weekday, char *name)
{
  GetWeekdayName(weekday, name);
  name[3] = 0;
}

void GetGregorianMonthName(int month, char *name)
{
  switch (month)
  {
    case 1: strcpy(name, "January"); break;
    case 2: strcpy(name, "February"); break;
    case 3: strcpy(name, "March"); break;
    case 4: strcpy(name, "April"); break;
    case 5: strcpy(name, "May"); break;
    case 6: strcpy(name, "June"); break;
    case 7: strcpy(name, "July"); break;
    case 8: strcpy(name, "August"); break;
    case 9: strcpy(name, "September"); break;
    case 10: strcpy(name, "October"); break;
    case 11: strcpy(name, "November"); break;
    case 12: strcpy(name, "December"); break;
  }
}

void GetHebrewMonthName(int month, int year, char *name)
{
  switch (month)
  {
    case 1: strcpy(name, "Nisan"); break;
    case 2: strcpy(name, "Iyar"); break;
    case 3: strcpy(name, "Sivan"); break;
    case 4: strcpy(name, "Tammuz"); break;
    case 5: strcpy(name, "Av"); break;
    case 6: strcpy(name, "Elul"); break;
    case 7: strcpy(name, "Tishri"); break;
    case 8: strcpy(name, "Heshvan"); break;
    case 9: strcpy(name, "Kislev"); break;
    case 10: strcpy(name, "Tevet"); break;
    case 11: strcpy(name, "Shevat"); break;
    case 12:
      strcpy(name, "Adar");
      if (Hebrew_Leap_Year(year))
        strcat(name, " I");
      break;
    case 13: strcpy(name, "Adar II"); break;
  }
}

void GetJewishShabbatSectionName(int torahReading, char *name)
{
  strcpy(name, "");
  switch (torahReading)
  {
    case ID_BESHALLAH: strcpy(name, "Shabbat Shirah"); break;
    case ID_DEBARIM: strcpy(name, "Shabbat Khazon"); break;
    case ID_VAETHANAN: strcpy(name, "Shabbat Nakhamu"); break;
    case ID_SHEKALIM: strcpy(name, "Shabbat Shekalim"); break;
    case ID_ZAHOR: strcpy(name, "Shabbat Zakhor"); break;
    case ID_PARAH: strcpy(name, "Shabbat Parah"); break;
    case ID_HAHODESH: strcpy(name, "Shabbat Hakhodesh"); break;
/*
    case ID_HAGGADOL: strcpy(name, "Shabbat Haggadol"); break;
*/
    case ID_SHUVA: strcpy(name, "Shabbat Shuva"); break;
  }
}

void GetJewishTorahSectionName(int torahReading, char *name)
{
  strcpy(name, "");
  switch (torahReading)
  {
    case ID_BERESHITH: strcpy(name, "Bereshith"); break;
    case ID_NOAH: strcpy(name, "Noah"); break;
    case ID_LEHLEHA: strcpy(name, "Le'h Leha"); break;
    case ID_VAYERA: strcpy(name, "Vayera"); break;
    case ID_HAYESARAH: strcpy(name, "Haye Sarah"); break;
    case ID_TOLEDOTH: strcpy(name, "Toledoth"); break;
    case ID_VAYETSE: strcpy(name, "Vayetse"); break;
    case ID_VAYISHLAH: strcpy(name, "Vayishlah"); break;
    case ID_VAYESHEB: strcpy(name, "Vayesheb"); break;
    case ID_MIKKETS: strcpy(name, "Mikkets"); break;
    case ID_VAYIGGASH: strcpy(name, "Vayiggash"); break;
    case ID_VAYHEE: strcpy(name, "Vayhee"); break;
    case ID_SHEMOTH: strcpy(name, "Shemoth"); break;
    case ID_VAERA: strcpy(name, "Vaera"); break;
    case ID_BO: strcpy(name, "Bo"); break;
    case ID_BESHALLAH: strcpy(name, "Beshallah"); break;
    case ID_YITHRO: strcpy(name, "Yithro"); break;
    case ID_MISHPATIM: strcpy(name, "Mishpatim"); break;
    case ID_TERUMAH: strcpy(name, "Terumah"); break;
    case ID_TETSAVVEH: strcpy(name, "Tetsavveh"); break;
    case ID_KITISSA: strcpy(name, "Ki Tissa"); break;
    case ID_VAYAKHEL: strcpy(name, "Vayakhel"); break;
    case ID_PEKUDE: strcpy(name, "Pekude"); break;
    case ID_VAYIKRA: strcpy(name, "Vayikra"); break;
    case ID_TSAV: strcpy(name, "Tsav"); break;
    case ID_SHEMINI: strcpy(name, "Shemini"); break;
    case ID_TAZRIANG: strcpy(name, "Tazria"); break;
    case ID_METSORANG: strcpy(name, "Metsora"); break;
    case ID_AHAREMOTH: strcpy(name, "Aharemoth"); break;
    case ID_KEDOSHIM: strcpy(name, "Kedoshim"); break;
    case ID_EMOR: strcpy(name, "Emor"); break;
    case ID_BEHAR: strcpy(name, "Behar"); break;
    case ID_BEHUKKOTHAI: strcpy(name, "Behukkothai"); break;
    case ID_BEMIDBAR: strcpy(name, "Bemidbar"); break;
    case ID_NASO: strcpy(name, "Naso"); break;
    case ID_BEHAALOTEHA: strcpy(name, "Behaaloteha"); break;
    case ID_SHELAHLEHA: strcpy(name, "Shelah Leha"); break;
    case ID_KORAH: strcpy(name, "Korah"); break;
    case ID_HUKATH: strcpy(name, "Hukath"); break;
    case ID_BALAK: strcpy(name, "Balak"); break;
    case ID_PINHAS: strcpy(name, "Pinhas"); break;
    case ID_MATOTH: strcpy(name, "Matoth"); break;
    case ID_MASEH: strcpy(name, "Maseh"); break;
    case ID_DEBARIM: strcpy(name, "Debarim"); break;
    case ID_VAETHANAN: strcpy(name, "Vaethanan"); break;
    case ID_EKEB: strcpy(name, "Ekeb"); break;
    case ID_REEH: strcpy(name, "Reeh"); break;
    case ID_SHOFETIM: strcpy(name, "Shofetim"); break;
    case ID_KITETSE: strcpy(name, "Ki Tetse"); break;
    case ID_KITABO: strcpy(name, "Ki Tabo"); break;
    case ID_NITSABIM: strcpy(name, "Nitsabim"); break;
    case ID_VAYELEH: strcpy(name, "Vayeleh"); break;
    case ID_HAAZINU: strcpy(name, "Haazinu"); break;
  }
}

