#define PAINT_INTERNAL

#include <gtk/gtk.h>
#include "config.h"
#include "gregor.h"
#include "hebrcal.h"
#include "date.h"
#include "misc.h"
#include "torah.h"
#include "names.h"
#include "holidays.h"
#include "calctime.h"
#include "settings.h"
#include "annifile.h"
#include "paint.h"

PAINT_EXTERN int gCurrentDay;
PAINT_EXTERN int gCurrentMonth;
PAINT_EXTERN int gCurrentYear;

PAINT_EXTERN void generateFontName(char *name, int size, char *style, 
                                   char *dest)
{
  sprintf(dest, "-*-%s-%s-r-normal--%d-*-*-*-*-*-iso8859-1",
          name, style, size); 
}

PAINT_EXTERN void GetMenuButton(GdkFont* font, char *str,
                               int* x, int* y, int* cx, int* cy)
{
  strcpy(str, "Menu");
  *x = 0;
  *y = 0;
  *cx = gdk_string_width(font, str) + 5;
  *cy = font->ascent + font->descent + 5;
} 

PAINT_EXTERN void GetYomButton(GdkFont* font, char *str,
                               int* x, int* y, int* cx, int* cy)
{
  strcpy(str, "Yom");
  *x = gdk_string_width(font, "Menu") + 5;
  *y = 0;
  *cx = gdk_string_width(font, str) + 5;
  *cy = font->ascent + font->descent + 5;
}

PAINT_EXTERN void GetTodayButton(GdkFont* font, char *str,
                                 int* x, int* y, int* cx, int* cy,
                                 int cxWindow)
{
  strcpy(str, "Today");
  *y = 0;
  *cx = gdk_string_width(font, str) + 5;
  *cy = font->ascent + font->descent + 5;
  *x = cxWindow - 1 - *cx;
}

void Draw3DEffect(GdkDrawable* drawable, GdkGC* gc, GdkColormap* colormap,
                  int x, int y, int cx, int cy,
                  int selected)
{
  GdkGCValues gcvalues;
  GdkColor color;
  int i;

  gdk_gc_get_values(gc, &gcvalues);
  
  for (i = 0; i < 2; i++)
  {
    if (!selected)
    {
      memset(&color, 0, sizeof(GdkColor));
      color.red = color.green = color.blue = 32768;
      gdk_colormap_alloc_color(colormap, &color, FALSE, TRUE);
      gdk_gc_set_foreground(gc, &color);
      gdk_draw_line(drawable, gc, x, y + i, x + cx - i, y + i);
      gdk_draw_line(drawable, gc, x + i, y, x + i, y + cy - i);
    }
  
    memset(&color, 0, sizeof(GdkColor));    
    if (selected)
    {
      color.red = color.green = color.blue = 32768;
    }
    else
    {
      color.red = color.green = color.blue = 50000;
    }
    gdk_colormap_alloc_color(colormap, &color, FALSE, TRUE);
    gdk_gc_set_foreground(gc, &color);
    gdk_draw_line(drawable, gc, x + cx - i, y + i, x + cx - i, y + cy);
    gdk_draw_line(drawable, gc, x + i, y + cy - i, x + cx, y + cy - i);
  }

  gdk_gc_set_foreground(gc, &gcvalues.foreground);
}

PAINT_EXTERN void MainWindowPaint(GtkWidget* widget, gpointer data)
{
  GdkEventExpose* e;

  e = (GdkEventExpose*) data;

  gdk_window_clear_area(e->window, e->area.x, e->area.y,
                        e->area.width, e->area.height);

  PaintIt(e->window, TRUE, NULL, NULL);
}

/* 
   On the evenings (Erev) of the following holidays, the algorithm
   for calculating shabbat candle lighting is used:

      Rosh HaShanah
      Yom Kippur
      Sukkot I
      Sukkot II (Diaspora)
      Shemini Azereth
      Simchat Torah (Diaspora)

      Pesach I
      Pesach II (Diaspora)
      Pesach VII
      Pesach VIII (Diaspora)

      Shavuot I
      Shavuot II (Diaspora)

   On Yom Kippur holiday, the algorithm for calculating
   havdalah is used.
*/
   
void GetTimeOfHoliday(long currentDate, char *method,
                      char *candleLighting, char *shabbatEnd)
{
  calDate dateHebrew;

  HebrcalForeignFromAbsolute(currentDate, &dateHebrew);

  if ((dateHebrew.day == 29) && (dateHebrew.month == 6)) 
                                              /* Erev Rosh HaShanah */
    strcpy(method, candleLighting);
  if ((dateHebrew.day == 9) && (dateHebrew.month == 7)) /* Erev Yom Kippur */
    strcpy(method, candleLighting);
  if ((dateHebrew.day == 14) && (dateHebrew.month == 7)) /* Erev Sukkot I */
    strcpy(method, candleLighting);
  if (diasporaGetFlag())
  {
    if ((dateHebrew.day == 15) && (dateHebrew.month == 7))
                                     /* Erev Sukkot II (Diaspora) */
      strcpy(method, candleLighting);
  }
  if ((dateHebrew.day == 21) && (dateHebrew.month == 7)) 
                                           /* Erev Shemini Azereth */
    strcpy(method, candleLighting);
  if (diasporaGetFlag())
  {
    if ((dateHebrew.day == 22) && (dateHebrew.month == 7))
                                     /* Erev Simchat Torah (Diaspora) */
      strcpy(method, candleLighting);
  }
  if ((dateHebrew.day == 14) && (dateHebrew.month == 1)) /* Erev Pesach I */
    strcpy(method, candleLighting);
  if (diasporaGetFlag())
  {
    if ((dateHebrew.day == 15) && (dateHebrew.month == 1)) 
                                       /* Erev Pesach II (Diaspora) */
      strcpy(method, candleLighting);
  }
  if ((dateHebrew.day == 20) && (dateHebrew.month == 1)) /* Erev Pesach VII */
    strcpy(method, candleLighting);
  if (diasporaGetFlag())
  {
    if ((dateHebrew.day == 21) && (dateHebrew.month == 1)) 
                                             /* Erev Pesach VIII (Diaspora) */
      strcpy(method, candleLighting);
  }

  if ((dateHebrew.day == 5) && (dateHebrew.month == 3)) /* Erev Shavuot I */
    strcpy(method, candleLighting);
  if (diasporaGetFlag())
  {
    if ((dateHebrew.day == 6) && (dateHebrew.month == 3))
                                        /* Erev Shavuot II (Diaspora) */
      strcpy(method, candleLighting);
  }

  if ((dateHebrew.day == 10) && (dateHebrew.month == 7)) /* Yom Kippur */
    strcpy(method, shabbatEnd);
}

void GetShabbatTime2(long currentDate, char *text,
                     char *fileName, char *city, dstStruct* dst,
                     char *candleLighting, char *shabbatEnd,
                     int *isCandleLighting, int *isShabbatEnd)
{
  CalDate date;
  char method[256];
  int latitude, longitude, timeZone, time, hour, min, dummy;

  if (isCandleLighting) *isCandleLighting = 0;
  if (isShabbatEnd) *isShabbatEnd = 0;

  GregorForeignFromAbsolute(currentDate, &date);
  strcpy(text, "");

  strcpy(method, "");
  if (GetWeekday(currentDate) == 5)
    strcpy(method, candleLighting);
  if (GetWeekday(currentDate) == 6)
    strcpy(method, shabbatEnd);
  if (strcmp(method, "") == 0)
    GetTimeOfHoliday(currentDate, method,
                     candleLighting, shabbatEnd);
  if (strcmp(method, "") != 0)
  {
    if (strcmp(method, candleLighting) == 0)
    {
      if (isCandleLighting) *isCandleLighting = 1;
    }
    if (strcmp(method, shabbatEnd) == 0)
    {
      if (isShabbatEnd) *isShabbatEnd = 1;
    }
    if (CityGet(fileName, city, &latitude, &longitude, &timeZone))
    {
      time = CalculateZmanim(latitude, longitude, timeZone,
                             date.day, date.month, date.year,
                             method);
      if (time != (-1))
      {
        hour = time / 60;
        min = time % 60;
        if (Daylight_Correction(date.day, date.month, date.year, 
                                &hour, &dummy, dst))
        {
          sprintf(text, "%02d:%02d", hour, min);
        }
      }
    }
  }
}

void GetShabbatTime(long currentDate, char *text,
                    int *isCandleLighting, int *isShabbatEnd)
{
  char cityFileName[256], city[256], havdalahValue[256];
  char candleLighting[256], havdalah[256];
  dstStruct dst;
  int isDST, candleLightingMin, havdalahIsMin;

  getCityFileName(cityFileName);
  getCity(city);
  isDST = getIsDST();
  getDSTstruct(&dst);

  candleLightingMin = getCandleLightingMin();
  sprintf(candleLighting, "sunset - %d", candleLightingMin);
  getHavdalahValue(havdalahValue);
  havdalahIsMin = getHavdalahIsMin();
  if (havdalahIsMin)
    sprintf(havdalah, "sunset + %d", atoi(havdalahValue));
  else
    sprintf(havdalah, "sunset + degrees_below_horizon(%f)", 
            atof(havdalahValue));

  GetShabbatTime2(currentDate, text,
                  cityFileName, city, (isDST ? &dst : NULL),
                  candleLighting, havdalah,
                  isCandleLighting, isShabbatEnd);
}

void draw_line_3(GdkWindow* window, GdkGC* gc,
                 int x1, int y1,
                 int x2, int y2,
                 int x3, int y3)
{
  gdk_draw_line(window, gc, x1, y1, x2, y2);
  gdk_draw_line(window, gc, x2, y2, x3, y3);
  gdk_draw_line(window, gc, x3, y3, x1, y1);
}

#define holidaycolor_none        0
#define holidaycolor_blue        1
#define holidaycolor_pink        2
#define holidaycolor_green       3
#define holidaycolor_light_green 4

void GetHolidayColor(char *holidayName, GdkColor *color)
{
  int holidaycolor;

  holidaycolor = holidaycolor_none;

  if (strstr(holidayName, "Erev") != NULL)
    return;

  if ((strstr(holidayName, "Rosh HaShana") != NULL) ||
      (strstr(holidayName, "Yom Kippur") != NULL))
    holidaycolor = holidaycolor_blue; 

  if ((strstr(holidayName, "Tsom Gedalya") != NULL) ||
      (strstr(holidayName, "Tsom Tevet") != NULL) ||
      (strstr(holidayName, "Taanit Ester") != NULL) ||
      (strstr(holidayName, "Yom HaShoa") != NULL) ||
      (strstr(holidayName, "Yom HaZikaron") != NULL) ||
      (strstr(holidayName, "Tsom Tammuz") != NULL) ||
      (strstr(holidayName, "Tisha BeAv") != NULL))
    holidaycolor = holidaycolor_pink;

  if ((strstr(holidayName, "Sukkot") != NULL) ||
      (strstr(holidayName, "Hoshana Rabba") != NULL) ||
      (strstr(holidayName, "Shemini Atseret") != NULL) ||
      (strstr(holidayName, "Simkhat Tora") != NULL) ||
      (strstr(holidayName, "Pesakh") != NULL) ||
      (strstr(holidayName, "Shavuot") != NULL))
    holidaycolor = holidaycolor_green;
  
  if ((strstr(holidayName, "Khanukka") != NULL) ||
      (strstr(holidayName, "Tu beShvat") != NULL) ||
      (strstr(holidayName, "Purim") != NULL) ||
      (strstr(holidayName, "Yom HaAtsmaut") != NULL) ||
      (strstr(holidayName, "Yom Yerushalayim") != NULL) ||
      (strstr(holidayName, "Lag BeOmer") != NULL) ||
      (strstr(holidayName, "Khol HaMoed") != NULL) ||
      (strstr(holidayName, "Isru Khag") != NULL))
    holidaycolor = holidaycolor_light_green;

  switch (holidaycolor)
  {
    case holidaycolor_blue:
      {
        color->red = 0; color->green = color->blue = 65535;
      }
      break;

    case holidaycolor_pink:
      {
        color->red = color->blue = 65535; color->green = 40960;
      }
      break;

    case holidaycolor_green:
      {
        color->red = color->blue = 0; color->green = 53248;
      }
      break;

    case holidaycolor_light_green:
      {
        color->red = 40960; color->green = 65535; color->blue = 0;
      }
      break;
  }
}

PAINT_EXTERN void PaintIt(GdkWindow* window, int doPaint,
                          PaintItTest* pit, PaintItPart *pip)
{
  GdkGC* gc;
  GdkColormap* colormap;
  int cxDrawingArea, cyDrawingArea, xMenu, yMenu, cxMenu, cyMenu;
  int xYom, yYom, cxYom, cyYom, xToday, yToday, cxToday, cyToday; 
  int y, xBegin, xEnd, cxCellWidth, cyCellHeight, i, j;
  int arrowBorder, arrowSize, arrowSizeX;  
  int iDay, iWeekday, iCurrentLine, iLastDay, omerCount;
  int xSpecialRect, ySpecialRect, cxSpecialRect, cySpecialRect;
  int aidTorah[3], countHolidays, holidayCounter, firstYear;
  int permitHoliday, countTextLines, holiday, shabbatY;
  int isCandleLighting, isShabbatEnd;
  char fontName[256], text[256], month[256], monthStart[256], monthEnd[256];
  char month1[256], month2[256], dateString[256], omerString[256];
  char roshHodeshString[256], holidayString[256], torahSectionString[256];
  char specialShabbatString[256], aText[3][256];
  char menuString[256], yomString[256], todayString[256];
  GdkFont *font, *fontCaption1, *fontCaption2, *fontWeekdays;
  GdkFont *fontDayNumbers, *fontMenu, *fontYom, *fontToday, *fontShabbatTime;
  calDate dateStart, dateEnd, dateCurrent, dateCurrentHebrew, dateHoliday;
  long startDate, endDate, currentDate;
  GdkGCValues gcvalues;
  GdkColor newColor;

  if (pit)
  {
    pit->returnIsMonth = 0;
    pit->returnIsYear = 0;
  
    pit->returnDay = 0;

    pit->returnIsPrev = 0;
    pit->returnIsNext = 0;
  }

  gc = gdk_gc_new(window);

  colormap = gdk_window_get_colormap(window);

  gdk_window_get_geometry(window, NULL, NULL,
                          &cxDrawingArea, &cyDrawingArea,
                          NULL);

  generateFontName("helvetica", 14, "bold", fontName);
  fontCaption1 = gdk_font_load(fontName);
  generateFontName("helvetica", 12, "bold", fontName);
  fontCaption2 = gdk_font_load(fontName);
  generateFontName("helvetica", 12, "bold", fontName);
  fontWeekdays = gdk_font_load(fontName);
  generateFontName("helvetica", 10, "medium", fontName);
  fontDayNumbers = gdk_font_load(fontName);
  generateFontName("helvetica", 10, "medium", fontName);
  fontMenu = gdk_font_load(fontName);
  generateFontName("helvetica", 10, "medium", fontName);
  fontYom = gdk_font_load(fontName);
  generateFontName("helvetica", 10, "medium", fontName);
  fontToday = gdk_font_load(fontName);
  generateFontName("helvetica", 10, "medium", fontName);
  fontShabbatTime = gdk_font_load(fontName);

  GetMenuButton(fontMenu, menuString, &xMenu, &yMenu, &cxMenu, &cyMenu);
  gdk_draw_rectangle(window, gc, FALSE,
                     xMenu, yMenu, cxMenu, cyMenu);
  gdk_draw_string(window, fontMenu, gc,
                  xMenu + cxMenu/2 - gdk_string_width(fontMenu, menuString)/2,
                  yMenu + cyMenu/2 - 
                      (fontMenu->ascent + fontMenu->descent)/2 +
                      fontMenu->ascent + fontMenu->descent,
                  menuString);
  GetYomButton(fontYom, yomString, &xYom, &yYom, &cxYom, &cyYom);
  gdk_draw_rectangle(window, gc, FALSE,
                     xYom, yYom, cxYom, cyYom);
  gdk_draw_string(window, fontYom, gc,
                  xYom + cxYom/2 - gdk_string_width(fontYom, yomString)/2,
                  yYom + cyYom/2 -
                     (fontYom->ascent + fontYom->descent)/2 +
                     fontYom->ascent + fontYom->descent,
                  yomString);

  GetTodayButton(fontToday, todayString, &xToday, &yToday, &cxToday, &cyToday,
                 cxDrawingArea);
  gdk_draw_rectangle(window, gc, FALSE,
                     xToday, yToday, cxToday, cyToday);
  gdk_draw_string(window, fontToday, gc,
                  xToday + cxToday/2 - 
                      gdk_string_width(fontToday, todayString)/2,
                  yToday + cyToday/2 -
                      (fontToday->ascent + fontToday->descent)/2 +
                      fontToday->ascent + fontToday->descent,
                  todayString);
 
  y = 0;

  font = fontCaption1;

  GetGregorianMonthName(gCurrentMonth, month);
  sprintf(text, "%d", gCurrentYear);
  xBegin = cxDrawingArea/2 -
       (gdk_string_width(font, month) +
        gdk_string_width(font, " ") +
        gdk_string_width(font, text))/2;
  if (pit)
  {
    if ((pit->y >= y) && (pit->y <= y + font->ascent + font->descent))
    {
      if ((pit->x >= xBegin) && 
          (pit->x <= xBegin + gdk_string_width(font, month))) 
      {
        pit->returnIsMonth = 1;
        return;
      }
      if ((pit->x >= xBegin + gdk_string_width(font, month) + 
                 gdk_string_width(font, " ")) &&
          (pit->x <= xBegin + gdk_string_width(font, month) + 
                 gdk_string_width(font, " ") + gdk_string_width(font, text)))
      {
        pit->returnIsYear = 1;
        return;
      }
    }
  }

  if (doPaint)
  {
    gdk_draw_string(window, font, gc,
                    xBegin,
                    y + font->ascent + font->descent,
                    month);
    gdk_draw_string(window, font, gc,
                    xBegin +
                      gdk_string_width(font, month) +
                      gdk_string_width(font, " "),
                    y + font->ascent + font->descent,
                    text);
  }

  y += (font->ascent + font->descent);

  font = fontCaption2;

  arrowBorder = 3;
  arrowSize = font->ascent + font->descent;
  arrowSizeX = arrowSize/2;

  xBegin = arrowBorder + arrowSizeX;
  xEnd = cxDrawingArea - arrowBorder - arrowSizeX;

  if (doPaint)
  {
    draw_line_3(window, gc,
                xBegin - arrowBorder, y + 5,
                xBegin - arrowBorder - arrowSizeX, y + 5 + arrowSize/2,
                xBegin - arrowBorder, y + 5 + arrowSize);
    draw_line_3(window, gc,
                xEnd + arrowBorder, y + 5,
                xEnd + arrowBorder + arrowSizeX, y + 5 + arrowSize/2,
                xEnd + arrowBorder, y + 5 + arrowSize);
  }

  if (pit)
  {
    if ((pit->y >= y + 5) && (pit->y <= y + 5 + arrowSize))
    {
      if ((pit->x >= xBegin - arrowBorder - arrowSizeX) &&
          (pit->x <= xBegin - arrowBorder))
      {
        pit->returnIsPrev = 1;
        return;
      }
      if ((pit->x >= xEnd + arrowBorder) &&
          (pit->x <= xEnd + arrowBorder + arrowSizeX))
      {
        pit->returnIsNext = 1;
        return;
      }
    }
  }

  dateStart.day = 1;
  dateStart.month = gCurrentMonth;
  dateStart.year = gCurrentYear;
  startDate = GregorAbsoluteFromForeign(&dateStart);
  HebrcalForeignFromAbsolute(startDate, &dateStart);
  dateEnd.day = GregorGetLastDayOfMonth(gCurrentMonth, gCurrentYear);
  dateEnd.month = gCurrentMonth;
  dateEnd.year = gCurrentYear;
  endDate = GregorAbsoluteFromForeign(&dateEnd);
  HebrcalForeignFromAbsolute(endDate, &dateEnd);
  GetHebrewMonthName(dateStart.month, dateStart.year, monthStart);
  GetHebrewMonthName(dateEnd.month, dateEnd.year, monthEnd);
  if (dateStart.year == dateEnd.year)
  {
    if (dateStart.month == dateEnd.month)
      sprintf(text, "%d - %d %s %d",
              dateStart.day, dateEnd.day, monthEnd, dateEnd.year);
    else
      sprintf(text, "%d %s - %d %s %d",
              dateStart.day, monthStart, dateEnd.day, monthEnd, dateEnd.year);
  }
  else
    sprintf(text, "%d %s %d - %d %s %d",
            dateStart.day, monthStart, dateStart.year,
            dateEnd.day, monthEnd, dateEnd.year);
  if (doPaint)
    gdk_draw_string(window, font, gc,
                    cxDrawingArea/2 - gdk_string_width(font, text)/2,
                    y + font->ascent + font->descent,
                    text);
  y += (font->ascent + font->descent);

  y += (font->ascent + font->descent)/3;

  font = fontWeekdays;

  cxCellWidth = (cxDrawingArea-2)/7;
  for (i = 0; i < 7; i++)
  {
    GetWeekdayNameShort(i, text);
    if (doPaint)
      gdk_draw_string(window, font, gc,
                      i * cxCellWidth + cxCellWidth/2 - 
                             gdk_string_width(font, text)/2,
                      y + font->ascent + font->descent,
                      text);
  }
  y += (font->ascent + font->descent);
  y += 2;
 
  cyCellHeight = (cyDrawingArea - y - 2) / 6;
  dateCurrent.day = 1;
  dateCurrent.month = gCurrentMonth;
  dateCurrent.year = gCurrentYear;
  currentDate = AbsoluteFromGregorian(&dateCurrent);
  iWeekday = GetWeekday(currentDate);
  iCurrentLine = 0;
  iLastDay = GregorGetLastDayOfMonth(gCurrentMonth, gCurrentYear);
  for (iDay = 1; iDay <= iLastDay; iDay++)
  {
    if (pip)
    {
      if ((iDay != pip->iDay1) && (iDay != pip->iDay2))
      {
        currentDate++;
        iWeekday++;
        if (iWeekday == 7)
        {
          iWeekday = 0;
          iCurrentLine++;
        }

        continue;
      }
    }

    if (pit)
    {
      if ((pit->x >= (iWeekday * cxCellWidth)) &&
          (pit->x <= ((iWeekday+1) * cxCellWidth)))
      {
        if ((pit->y >= (y + iCurrentLine * cyCellHeight)) &&
            (pit->y <= (y + (iCurrentLine+1) * cyCellHeight)))
        {
          pit->returnDay = iDay;
        }
      }
    }

    GregorForeignFromAbsolute(currentDate, &dateCurrent);
    HebrcalForeignFromAbsolute(currentDate, &dateCurrentHebrew);

    memset(&newColor, 0, sizeof(GdkColor));
    newColor.red = newColor.green = newColor.blue = 53248;

    holiday = 0;
    countHolidays = GetJewishHolidayCount();
    for (holidayCounter = 0; holidayCounter < countHolidays; holidayCounter++)
    {
      firstYear = GetJewishHolidayFirstYear(holidayCounter);
      if (firstYear == 0)
        permitHoliday = 1;
      else
      {
        if (dateCurrent.year >= firstYear)
          permitHoliday = 1;
        else
          permitHoliday = 0;
      }
      if (permitHoliday)
      {
        GetJewishHolidayDate(holidayCounter, dateCurrentHebrew.year,
                             &dateHoliday);
        if (dateHoliday.day != 0)
        {
          if ((dateHoliday.day == dateCurrentHebrew.day) &&
              (dateHoliday.month == dateCurrentHebrew.month) &&
              (dateHoliday.year == dateCurrentHebrew.year))
          {
            GetJewishHolidayName(holidayCounter, holidayString);
            GetHolidayColor(holidayString, &newColor);
            holiday = 1;
          }
        }
      }
    }

    gdk_gc_get_values(gc, &gcvalues);
    if (iDay == gCurrentDay)
    {
      newColor.red = newColor.green = newColor.blue = 40960;
    }
    gdk_colormap_alloc_color(colormap, &newColor, FALSE, TRUE);
    gdk_gc_set_foreground(gc, &newColor);
      
    if (doPaint)
      gdk_draw_rectangle(window, gc, TRUE,
                           iWeekday * cxCellWidth + 1,
                           y + iCurrentLine * cyCellHeight + 1,
                           cxCellWidth - 1, cyCellHeight - 1);
    
    gdk_gc_set_foreground(gc, &gcvalues.foreground);

    if (doPaint)
      Draw3DEffect(window, gc, colormap, 
                   iWeekday * cxCellWidth + 1,
                   y + iCurrentLine * cyCellHeight + 1,
                   cxCellWidth - 2, cyCellHeight - 2,
                   (iDay == gCurrentDay) ? 1 : 0);

    font = fontDayNumbers;

    sprintf(text, "%d", dateCurrent.day);
    if (doPaint)
      gdk_draw_string(window, font, gc,
                      iWeekday * cxCellWidth + 4,
                      y + iCurrentLine * cyCellHeight +
                           font->ascent + font->descent,
                      text);
    
    sprintf(text, "%d", dateCurrentHebrew.day);

    if (IsDateRoshHodesh(currentDate))
    {
      if (doPaint)
        gdk_draw_arc(window, gc, FALSE,
                     iWeekday * cxCellWidth + cxCellWidth -
                         gdk_string_width(font, text) - 4,
                     y + iCurrentLine * cyCellHeight + 2,
                     gdk_string_width(font, text) + 2,
                     font->ascent + font->descent,
                     0, 64 * 360);
    }

    gdk_gc_get_values(gc, &gcvalues);
    memset(&newColor, 0, sizeof(GdkColor));
    if (holiday)
      newColor.red = 65535;
    else
      newColor.blue = 65535;
    gdk_colormap_alloc_color(colormap, &newColor, FALSE, TRUE);
    gdk_gc_set_foreground(gc, &newColor);
    if (doPaint)
      gdk_draw_string(window, font, gc,
                      iWeekday * cxCellWidth + cxCellWidth -
                          gdk_string_width(font, text) - 2,
                      y + iCurrentLine * cyCellHeight +
                            font->ascent + font->descent,
                      text);
    gdk_gc_set_foreground(gc, &gcvalues.foreground);

    font = fontShabbatTime;

    GetShabbatTime(currentDate, text, &isCandleLighting, &isShabbatEnd);
    if (strcmp(text, "") != 0)
    {
      gdk_gc_get_values(gc, &gcvalues);
      memset(&newColor, 0, sizeof(GdkColor));
      if (isCandleLighting)
        newColor.red = 65535;
      if (isShabbatEnd)
        newColor.blue = 65535;
      gdk_colormap_alloc_color(colormap, &newColor, FALSE, TRUE);
      gdk_gc_set_foreground(gc, &newColor);
      if (doPaint)
        gdk_draw_string(window, font, gc,
                        iWeekday * cxCellWidth + cxCellWidth/2 -
                            gdk_string_width(font, text)/2,
                        y + iCurrentLine * cyCellHeight +
                            font->ascent + font->descent +
                            font->ascent + font->descent,
                        text);
      gdk_gc_set_foreground(gc, &gcvalues.foreground);
      shabbatY = font->ascent + font->descent;
    }
    else
      shabbatY = 0;

    if (pit)
    {
      if (pit->returnDay)
        pit->returnDayAnni = 0;
    }

    if (AnnifileGetForDay(currentDate, text, NULL))
    {
      if (doPaint)
        gdk_draw_string(window, font, gc,
                        iWeekday * cxCellWidth + cxCellWidth/2 -
                            gdk_string_width(font, text)/2,
                        y + iCurrentLine * cyCellHeight +
                            font->ascent + font->descent +
                            font->ascent + font->descent +
                            shabbatY,
                       text);
      if (pit)
      {
        if (pit->returnDay)
        {
          if ((pit->y >= y + iCurrentLine * cyCellHeight +
                             font->ascent + font->descent +
                             shabbatY) &&
              (pit->y <= y + iCurrentLine * cyCellHeight +
                             font->ascent + font->descent +
                             font->ascent + font->descent +
                             shabbatY))
            pit->returnDayAnni = 1;
        }
      }
    }

    if (pit)
    {
      if (pit->returnDay)
        return;
    }

    currentDate++;
    iWeekday++;
    if (iWeekday == 7)
    {
      iWeekday = 0;
      iCurrentLine++;
    }
  }

  xSpecialRect = 2 * cxCellWidth;
  ySpecialRect = y + 5 * cyCellHeight;
  cxSpecialRect = cxDrawingArea - xSpecialRect - 1;
  cySpecialRect = cyDrawingArea - ySpecialRect;

  gdk_gc_get_values(gc, &gcvalues);
  memset(&newColor, 0, sizeof(GdkColor));
  newColor.red = newColor.green = newColor.blue = 53248;
  gdk_colormap_alloc_color(colormap, &newColor, FALSE, TRUE);
  gdk_gc_set_foreground(gc, &newColor);

  if (doPaint)
    gdk_draw_rectangle(window, gc, TRUE,
                       xSpecialRect + 1, ySpecialRect + 1,
                       cxSpecialRect - 1, cySpecialRect - 1);

  gdk_gc_set_foreground(gc, &gcvalues.foreground);
  
  dateCurrent.day = gCurrentDay;
  dateCurrent.month = gCurrentMonth;
  dateCurrent.year = gCurrentYear;
  currentDate = GregorAbsoluteFromForeign(&dateCurrent);
  HebrcalForeignFromAbsolute(currentDate, &dateCurrentHebrew);
  GetGregorianMonthName(dateCurrent.month, month1);
  month1[3] = 0;
  GetHebrewMonthName(dateCurrentHebrew.month, dateCurrentHebrew.year, 
                     month2);
  sprintf(dateString,
          "%d %s %d-%d %s %d",
          dateCurrent.day, month1, dateCurrent.year,
          dateCurrentHebrew.day, month2, dateCurrentHebrew.year);

  omerCount = GetOmerCount(currentDate);
  if (omerCount > 0)
    sprintf(omerString, "%d L'Omer", omerCount);
  else
    strcpy(omerString, "");

  if (IsDateRoshHodesh(currentDate))
    strcpy(roshHodeshString, "Rosh Khodesh");
  else
    strcpy(roshHodeshString, "");

  strcpy(holidayString, "");
  countHolidays = GetJewishHolidayCount();
  for (holidayCounter = 0; holidayCounter < countHolidays; holidayCounter++)
  {
    firstYear = GetJewishHolidayFirstYear(holidayCounter);
    if (firstYear == 0)
      permitHoliday = 1;
    else
    {
      if (dateCurrent.year >= firstYear)
        permitHoliday = 1;
      else
        permitHoliday = 0;
    }
    if (permitHoliday)
    {
      GetJewishHolidayDate(holidayCounter, dateCurrentHebrew.year, 
                           &dateHoliday);
      if (dateHoliday.day != 0)
      {
        if ((dateHoliday.day == dateCurrentHebrew.day) &&
            (dateHoliday.month == dateCurrentHebrew.month) &&
            (dateHoliday.year == dateCurrentHebrew.year))
        {
          GetJewishHolidayName(holidayCounter, holidayString);
          break;
        }
      }
    }
  }

  strcpy(torahSectionString, "");
  strcpy(specialShabbatString, "");
  if (GetTorahSectionShabbat(currentDate,
                             &aidTorah[0], &aidTorah[1], &aidTorah[2]))
  {
    for (i = 0; i < 3; i++)
    {
      GetJewishTorahSectionName(aidTorah[i], text);
      if (strcmp(text, "") != 0)
      {
        if (strcmp(torahSectionString, "") == 0)
          strcpy(torahSectionString, text);
        else
        {
          strcat(torahSectionString, ", ");
          strcat(torahSectionString, text);
        }
      }

      GetJewishShabbatSectionName(aidTorah[i], text);
      if (strcmp(text, "") != 0)
        strcpy(specialShabbatString, text);
    }
  }

  if (doPaint)
  {
    strcpy(aText[0], dateString);

    strcpy(aText[1], "");
    if (strcmp(omerString, "") != 0)
      strcpy(aText[1], omerString);
    if (strcmp(roshHodeshString, "") != 0)
    {
      if (strcmp(aText[1], "") != 0)
        strcat(aText[1], " - ");
      strcat(aText[1], roshHodeshString);
    }
    if (strcmp(holidayString, "") != 0)
    {
      if (strcmp(aText[1], "") != 0)
        strcat(aText[1], " - ");
      strcat(aText[1], holidayString);
    }

    strcpy(aText[2], "");
    if (strcmp(torahSectionString, "") != 0)
      strcpy(aText[2], torahSectionString);
    
    if (strcmp(specialShabbatString, "") != 0)
    {
      if (strcmp(aText[2], "") != 0)
        strcat(aText[2], " - ");
      strcat(aText[2], specialShabbatString);
    }

    countTextLines = 0;
    for (i = 0; i < 3; i++)
    {
      if (strcmp(aText[i], "") == 0)
        continue;
      countTextLines++;
    }
    j = 0;
    for (i = 0; i < 3; i++)
    {
      if (strcmp(aText[i], "") == 0) 
        continue;
      if (doPaint)
        gdk_draw_string(window, font, gc,
                        xSpecialRect + cxSpecialRect/2 -
                           gdk_string_width(font, aText[i])/2,
                        ySpecialRect + cySpecialRect/2 -
                           (countTextLines * (font->ascent + font->descent))/2 +
                           j * (font->ascent + font->descent) +
                           font->ascent + font->descent,
                        aText[i]);
      j++;
    }    
  }

  if (doPaint)
    Draw3DEffect(window, gc, colormap,
                 xSpecialRect + 1, ySpecialRect + 1,
                 cxSpecialRect - 2, cySpecialRect - 2,
                 FALSE);
}

