#include "config.h"
#include "ini.h"
#include "dst.h"
#include "halacha.h"
#include "settings.h"

int diasporaFlag;
int postponeShushanPurimFlag;
int currentStyle;

char currentCityFileName[256], currentCity[256];
int candleLightingMin;

char havdalahValue[256];
int havdalahIsMin;
int isDST;
dstStruct dstSetting;

void getDefaultCityFilename(char *dest, char *name)
{
  char *homePath;

  homePath = getenv("HOME");
  if (homePath)
  {
    strcpy(dest, homePath);
    if (strlen(dest) > 0)
    {
      if (dest[strlen(dest)-1] != '/')
        strcat(dest, "/");
    }
    strcat(dest, name);
  }
  else
    strcpy(dest, name);
}

void settingsInit(void)
{
  char text[256];

  diasporaFlag = 1;
  postponeShushanPurimFlag = 1;
  currentStyle = ashkenazi;
  getDefaultCityFilename(currentCityFileName, CITIES_FILENAME);
  strcpy(currentCity, "Pforzheim");

  candleLightingMin = 18;
  strcpy(havdalahValue, "45");
  havdalahIsMin = 1;

  isDST = 1;
  dstSetting.startWeekNo = DST_WEEK_LAST;
  dstSetting.startWeekday = 0;
  dstSetting.startMonth = 3;
  dstSetting.endWeekNo = DST_WEEK_LAST;
  dstSetting.endWeekday = 0;
  dstSetting.endMonth = 10;  

  diasporaFlag = SettingsGetInt("Diaspora", diasporaFlag);
  postponeShushanPurimFlag = SettingsGetInt("PostponeShushanPurim",
                                            postponeShushanPurimFlag);

  currentStyle = SettingsGetInt("Style", currentStyle);
  if ((currentStyle != sephardic) && (currentStyle != ashkenazi))
    currentStyle = ashkenazi;

  SettingsGet("CurrentCityFileName", text, currentCityFileName);
    strcpy(currentCityFileName, text);
  SettingsGet("CurrentCity", text, currentCity);
    strcpy(currentCity, text);

  candleLightingMin = SettingsGetInt("CandleLightingMin",
                                     candleLightingMin);
  SettingsGet("HavdalahValue", text, havdalahValue);
    strcpy(havdalahValue, text);
  havdalahIsMin = SettingsGetInt("HavdalahIsMin",
                                 havdalahIsMin);

  isDST = SettingsGetInt("IsDST", isDST);
  dstSetting.startWeekNo = SettingsGetInt("DSTStartWeekNo", 
                                          dstSetting.startWeekNo);
  dstSetting.startWeekday = SettingsGetInt("DSTStartWeekday",
                                           dstSetting.startWeekday);
  dstSetting.startMonth = SettingsGetInt("DSTStartMonth",
                                         dstSetting.startMonth);
  dstSetting.endWeekNo = SettingsGetInt("DSTEndWeekNo",
                                        dstSetting.endWeekNo);
  dstSetting.endWeekday = SettingsGetInt("DSTEndWeekday",
                                         dstSetting.endWeekday);
  dstSetting.endMonth = SettingsGetInt("DSTEndMonth",
                                       dstSetting.endMonth);
}

void settingsDone(void)
{
  SettingsSetInt("Diaspora", diasporaFlag);
  SettingsSetInt("PostponeShushanPurim", postponeShushanPurimFlag);
  SettingsSetInt("Style", currentStyle);

  SettingsSet("CurrentCityFileName", currentCityFileName);
  SettingsSet("CurrentCity", currentCity);

  SettingsSetInt("CandleLightingMin", candleLightingMin);
  SettingsSet("HavdalahValue", havdalahValue);
  SettingsSetInt("HavdalahIsMin", havdalahIsMin);

  SettingsSetInt("IsDST", isDST);
  SettingsSetInt("DSTStartWeekNo", dstSetting.startWeekNo);
  SettingsSetInt("DSTStartWeekday", dstSetting.startWeekday);
  SettingsSetInt("DSTStartMonth", dstSetting.startMonth);
  SettingsSetInt("DSTEndWeekNo", dstSetting.endWeekNo);
  SettingsSetInt("DSTEndWeekday", dstSetting.endWeekday);
  SettingsSetInt("DSTEndMonth", dstSetting.endMonth);
}

int diasporaGetFlag(void)
{
  return diasporaFlag;
}

void diasporaSetFlag(int isDiaspora)
{
  diasporaFlag = isDiaspora;
}

int postponeShushanPurimGetFlag(void)
{
  return postponeShushanPurimFlag;
}

void postponeShushanPurimSetFlag(int postponeShushanPurim)
{
  postponeShushanPurimFlag = postponeShushanPurim;
}

int getStyle(void)
{
  return currentStyle;
}

void setStyle(int style)
{
  currentStyle = style;
}

void getCityFileName(char *cityFileName)
{
  strcpy(cityFileName, currentCityFileName);
}

void setCityFileName(char *cityFileName)
{
  strcpy(currentCityFileName, cityFileName);
}

void getCity(char *city)
{
  strcpy(city, currentCity);
}

void setCity(char *city)
{
  strcpy(currentCity, city);
}

int getCandleLightingMin(void)
{
  return candleLightingMin;
}

void setCandleLightingMin(int value)
{
  candleLightingMin = value;
}

int getHavdalahIsMin(void)
{
  return havdalahIsMin;
}

void setHavdalahIsMin(int value)
{
  havdalahIsMin = value;
}

void getHavdalahValue(char *value)
{
  strcpy(value, havdalahValue);
}

void setHavdalahValue(char *value)
{
  strcpy(havdalahValue, value);
}

int getIsDST(void)
{
  return (isDST);
}

void setIsDST(int value)
{
  isDST = value;
}

void getDSTstruct(dstStruct *dst)
{
  memcpy(dst, &dstSetting, sizeof(dstStruct));
}

void setDSTstruct(dstStruct *dst)
{
  memcpy(&dstSetting, dst, sizeof(dstStruct));
}
  
