#include <gtk/gtk.h>
#include "config.h"
#include "dialogs.h"
#include "misc.h"
#include "names.h"
#include "settings.h"
#include "shabconf.h"

#define DIALOG_TITLE       "Configure Shabbat Times"

#define LABEL_LOCATION     "Location:"

#define LABEL_CANDLELIGHTING      "Candle Lighting:"
#define LABEL_CANDLELIGHTING_MIN  "min"

#define LABEL_HAVDALAH            "End of Shabbat:"
#define LABEL_HAVDALAH_MIN        "min"
#define LABEL_HAVDALAH_DEG        "degrees below horizon"

extern int lstrcmpi(char *str1, char *str2);
extern int CityProcessLine(char *lpLine, char *lpCity,
                           int *lpiLatitude, int *lpiLongitude,
                           int *lpiTimeZone, char *lpErrorMessage);

gint ShabbatCompare(gconstpointer a, gconstpointer b)
{
  char *str1, *str2;

  if (a) str1 = (char *) a; else str1 = "";
  if (b) str2 = (char *) b; else str2 = "";
  return strcmp(str1, str2);
}

GList* CityGetStringList(char *fileName)
{
  GList* list;
  FILE *stream;
  long fileSize;
  char *lpBuffer, *lpBufferLauf;
  char szLine[256], *szLineLauf, *szComment, szCity[256];
  int iLatitude, iLongitude, iTimeZone, iLineNo;
  char errorMessage[256], errorMessage2[256];

  list = g_list_alloc();
  stream = fopen(fileName, "rb");
  if (stream)
  {
    fseek(stream, 0, SEEK_END);
    fileSize = ftell(stream);
    fseek(stream, 0, SEEK_SET);
    lpBuffer = malloc(fileSize+1);
    fread(lpBuffer, fileSize, 1, stream);
    *(lpBuffer + fileSize) = 0;
    fclose(stream);

    iLineNo = 1;
    lpBufferLauf = lpBuffer;
    while (*lpBufferLauf)
    {
      szLineLauf = szLine;
      while ((*lpBufferLauf != 0) &&
             (*lpBufferLauf != 13) &&
             (*lpBufferLauf != 10))
      {
        *szLineLauf++ = *lpBufferLauf;
        lpBufferLauf++;
      }
      *szLineLauf = 0;
      while ((*lpBufferLauf == 13) ||
             (*lpBufferLauf == 10))
      {
        if (*lpBufferLauf == 10)
          iLineNo++;
        lpBufferLauf++;
      }

      szComment = strstr(szLine, "#");
      if (szComment) *szComment = 0;
      if (!IsBlank(szLine))
      {
        if (CityProcessLine(szLine, szCity, &iLatitude, &iLongitude,
                            &iTimeZone, errorMessage))
        {
          list = g_list_insert_sorted(list, strdup(szCity), ShabbatCompare);
        }
        else
        {
          sprintf(errorMessage2, "File %s, line %d: %s",
                   (char *) fileName, iLineNo, errorMessage);
          printf("%s\n", errorMessage2);
        }                       
      }
    }

    free(lpBuffer);
  }
  else
  {
    sprintf(errorMessage, "File %s could not be opened", fileName);
    printf("%s\n", errorMessage);
  }
  return (list);
}

int NumberIsInteger(char *str)
{
  char *lauf;

  lauf = str;
  while (*lauf == ' ') lauf++;
  if (*lauf == 0)
    return 0;
  while ((*lauf >= '0') && (*lauf <= '9')) lauf++;
  while (*lauf == ' ') lauf++;
  if (*lauf == 0)
    return 1;
  else
    return 0;
}

int NumberIsFloat(char *str)
{
  char *lauf;

  lauf = str;
  while (*lauf == ' ') lauf++;
  while ((*lauf >= '0') && (*lauf <= '9')) lauf++;
  if (*lauf == '.')
  {
    lauf++;
    while ((*lauf >= '0') && (*lauf <= '9')) lauf++;
  }
  while (*lauf == ' ') lauf++;
  if (*lauf == 0)
    return 1;
  else
    return 0;
}

int idShabbatConfigClicked;

void ShabbatConfigClick(GtkWidget *widget, gpointer data)
{
  idShabbatConfigClicked = (int) data;
}

int DoConfigureShabbatTimes(void)
{
  GtkDialog *dialog;
  GtkHBox *hbox, *hbox2;
  GtkVBox *vbox;
  GtkLabel *label;
  GtkButton *button;
  GtkCombo *comboLoc;
  GtkEntry *candleLighting, *havdalah;
  GtkRadioButton *havdalahMin, *havdalahDeg;
  char text[256], cityFilename[256], currentCity[256];
  char *location, *candleLightingValue, *havdalahValue;
  int value, candleLightingInt, havdalahIsMin, result;

  dialog = GTK_DIALOG(gtk_dialog_new());
  gtk_window_set_title(GTK_WINDOW(dialog), DIALOG_TITLE);

  label = GTK_LABEL(gtk_label_new(LABEL_LOCATION));
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));

  comboLoc = GTK_COMBO(gtk_combo_new());
  getCityFileName(cityFilename);
  gtk_combo_set_popdown_strings(comboLoc, CityGetStringList(cityFilename));
  gtk_combo_set_value_in_list(comboLoc, TRUE, FALSE);
  getCity(currentCity);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(comboLoc)->entry), currentCity);
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(comboLoc));
  gtk_widget_show(GTK_WIDGET(comboLoc));

  label = GTK_LABEL(gtk_label_new(LABEL_CANDLELIGHTING));
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));

  hbox = GTK_HBOX(gtk_hbox_new(FALSE, 1));

  candleLighting = GTK_ENTRY(gtk_entry_new());
  value = getCandleLightingMin();
  sprintf(text, "%d", value);
  gtk_entry_set_text(GTK_ENTRY(candleLighting), text);
  gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(candleLighting));
  gtk_widget_show(GTK_WIDGET(candleLighting));

  label = GTK_LABEL(gtk_label_new(LABEL_CANDLELIGHTING_MIN));
  gtk_container_add(GTK_CONTAINER(hbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));

  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(hbox));
  gtk_widget_show(GTK_WIDGET(hbox));

  label = GTK_LABEL(gtk_label_new(LABEL_HAVDALAH));
  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(label));
  gtk_widget_show(GTK_WIDGET(label));

  vbox = GTK_VBOX(gtk_vbox_new(FALSE, 1));

  havdalah = GTK_ENTRY(gtk_entry_new());
  getHavdalahValue(text);
  gtk_entry_set_text(GTK_ENTRY(havdalah), text);
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(havdalah));
  gtk_widget_show(GTK_WIDGET(havdalah));

  hbox2 = GTK_HBOX(gtk_hbox_new(FALSE, 1));
    havdalahMin = GTK_RADIO_BUTTON(gtk_radio_button_new_with_label
                        (NULL, LABEL_HAVDALAH_MIN));
    gtk_container_add(GTK_CONTAINER(hbox2), GTK_WIDGET(havdalahMin));
    gtk_widget_show(GTK_WIDGET(havdalahMin));

    havdalahDeg = GTK_RADIO_BUTTON(gtk_radio_button_new_with_label_from_widget
                        (havdalahMin, LABEL_HAVDALAH_DEG));
    gtk_container_add(GTK_CONTAINER(hbox2), GTK_WIDGET(havdalahDeg));
    gtk_widget_show(GTK_WIDGET(havdalahDeg));

    if (getHavdalahIsMin())
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(havdalahMin), TRUE);
    else
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(havdalahDeg), TRUE);
  gtk_container_add(GTK_CONTAINER(vbox), GTK_WIDGET(hbox2));
  gtk_widget_show(GTK_WIDGET(hbox2));

  gtk_container_add(GTK_CONTAINER(dialog->vbox), GTK_WIDGET(vbox));
  gtk_widget_show(GTK_WIDGET(vbox));

  button = GTK_BUTTON(gtk_button_new_with_label("OK"));
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(ShabbatConfigClick), (gpointer) 1);
  gtk_container_add(GTK_CONTAINER(dialog->action_area), GTK_WIDGET(button));
  gtk_widget_show(GTK_WIDGET(button));

  button = GTK_BUTTON(gtk_button_new_with_label("Cancel"));
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(ShabbatConfigClick), (gpointer) 2);
  gtk_container_add(GTK_CONTAINER(dialog->action_area), GTK_WIDGET(button));
  gtk_widget_show(GTK_WIDGET(button));

  gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
                     GTK_SIGNAL_FUNC(ShabbatConfigClick), (gpointer) 2);

  gtk_widget_show(GTK_WIDGET(dialog));
 
  for (;;)
  {
    idShabbatConfigClicked = (-1);
    while (idShabbatConfigClicked == (-1))
    {
/*    if (gtk_events_pending())*/
        gtk_main_iteration();
    }
    if (idShabbatConfigClicked == 1)
    {
      location = (char *) gtk_entry_get_text
                  (GTK_ENTRY(GTK_COMBO(comboLoc)->entry));
      if (strcmp(location, "") != 0)
      {
        candleLightingValue = (char *) gtk_entry_get_text
                        (GTK_ENTRY(candleLighting));
        if (NumberIsInteger(candleLightingValue))
        {
          candleLightingInt = atoi(candleLightingValue);
          if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(havdalahMin)))
            havdalahIsMin = 1;
          else
            havdalahIsMin = 0; 
          havdalahValue = (char *) gtk_entry_get_text(GTK_ENTRY(havdalah));
          if (havdalahIsMin)
            result = NumberIsInteger(havdalahValue);
          else
            result = NumberIsFloat(havdalahValue);
          if (result)
          {
            {
              setCity(location);
              setCandleLightingMin(candleLightingInt);
              setHavdalahIsMin(havdalahIsMin);
              setHavdalahValue(havdalahValue);

              break;
            }
          }
          else
            DialogsDoMessage("Havdalah must be a number");
        }
        else
          DialogsDoMessage("Candle Lighting must be an integer number");
      }
      else
        DialogsDoMessage("A location must be selected");
    }
    else
      break;
  }

  if (GTK_IS_DIALOG(dialog)) gtk_widget_destroy(GTK_WIDGET(dialog));

  if (idShabbatConfigClicked == 1)
    return 1;
  else
    return 0;
}

