#include "config.h"
#include "gregor.h"
#include "suntime.h"

int leap(int y)
{
  if (y % 400 == 0)
    return 1;
  if (y % 100 != 0)
  {
    if (y % 4 == 0)
      return 1;
  }
  return 0;
}

int monCount[14] = {0, 1, 32, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366};

int doy(int d, int m, int y)
{
  if ((m > 2) && (leap(y)))
    return monCount[m] + d + 1;
  else  
    return monCount[m] + d;
}

SUNTIME_FLOAT todec(SUNTIME_FLOAT deg, SUNTIME_FLOAT min)
{
  return (deg + min / 60.0);
}

SUNTIME_FLOAT M(SUNTIME_FLOAT x) 
{
  return (0.9856 * x - 3.251);
}

SUNTIME_FLOAT L(SUNTIME_FLOAT x)
{
  return (x + 1.916 * sin(0.01745 * x) + 0.02 * sin(2 * 0.01745 * x) + 282.565);
}

SUNTIME_FLOAT adj(SUNTIME_FLOAT x)
{
  return (-0.06571 * x - 6.620);
}

SUNTIME_FLOAT float_abs(SUNTIME_FLOAT x)
{
	if (x < 0.0)
		return (-x);
	else
		return (x);
}

int suntime(int d, int m, int y,
            int zendeg, int zenmin,
            int londeg, int lonmin, int ew,
            int latdeg, int latmin, int ns,
            int tz,
            SUNTIME_FLOAT* lpSunrise,
            SUNTIME_FLOAT* lpSunset)
{
  SUNTIME_FLOAT lonhr;
  SUNTIME_FLOAT longitude, latitude;
  SUNTIME_FLOAT coslat, sinlat, cosz;
  SUNTIME_FLOAT sindec, cosdec;
  SUNTIME_FLOAT xm_rise,  xm_set;
  SUNTIME_FLOAT xl_rise,  xl_set;
  SUNTIME_FLOAT a_rise,   a_set;
  SUNTIME_FLOAT ahr_rise, ahr_set;
  SUNTIME_FLOAT h_rise,   h_set;
  SUNTIME_FLOAT t_rise,   t_set;
  SUNTIME_FLOAT ut_rise,  ut_set;

  int retval;     // NORMAL
  int day;
  
  retval = 1;
  
  day = doy(d, m, y);

  cosz = cos(0.01745 * todec(zendeg, zenmin));

  longitude = todec(londeg, lonmin) * ((ew == 0) ? 1 : -1);
  lonhr     = longitude / 15.0;
  latitude  = todec(latdeg, latmin) * ((ns == 0) ? 1 : -1);
  coslat    = cos(0.01745 * latitude);
  sinlat    = sin(0.01745 * latitude);

  t_rise = day + (6.0 + lonhr) / 24.0;
  t_set  = day + (18.0 + lonhr) / 24.0;

  xm_rise = M(t_rise);
  xl_rise = L(xm_rise);
  xm_set  = M(t_set);
  xl_set  = L(xm_set);
  
  a_rise = 57.29578 * atan( 0.91746 * tan(0.01745 * xl_rise) );
  a_set  = 57.29578 * atan( 0.91746 * tan(0.01745 * xl_set) );

  if (float_abs(a_rise + 360.0 - xl_rise) > 90.0)
    a_rise += 180.0;
  if (a_rise > 360.0)
    a_rise -= 360.0;

  if (float_abs(a_set + 360.0 - xl_set) > 90.0)
    a_set += 180.0;
  if (a_set > 360.0)
    a_set -= 360.0;

  ahr_rise = a_rise / 15.0;
  sindec = 0.39782 * sin(0.01745 * xl_rise);
  cosdec = sqrt(1.0 - sindec * sindec);
  h_rise = (cosz - sindec * sinlat) / (cosdec * coslat);

  ahr_set = a_set / 15.0;
  sindec = 0.39782 * sin(0.01745 * xl_set);
  cosdec = sqrt(1.0 - sindec * sindec);
  h_set = (cosz - sindec * sinlat) / (cosdec * coslat);

  if (float_abs(h_rise) <= 1.0)
    h_rise = 57.29578 * acos(h_rise);
  else
    retval = 0; //NO_SUNRISE;

  if (float_abs(h_set) <= 1.0)
    h_set = 57.29578 * acos(h_set);
  else
    retval = 0; //NO_SUNSET;

  ut_rise  = ((360.0 - h_rise) / 15.0) + ahr_rise + adj(t_rise) + lonhr;
  ut_set  = (h_rise / 15.0) + ahr_set + adj(t_set) + lonhr;

  *lpSunrise = ut_rise + tz;  // sunrise
  *lpSunset = ut_set  + tz;  // sunset
  return retval;
}

void timeadj(SUNTIME_FLOAT t, int* lpiHour, int* lpiMin)
{
  int hour, min;
  SUNTIME_FLOAT time;
        
  time = t;

  if (time < 0)
    time += 24.0;

  hour = (int) floor(time);
  min  = (int) floor((time - hour) * 60.0 + 0.5);

  if(min >= 60) 
  {
    hour += 1;
    min  -= 60;
  }

  if (hour > 24)
    hour -= 24;

  *lpiHour = hour;
  *lpiMin = min;
}

int GetSunsetSunrise(int day, int month, int year,
                     int occurrence,
                     int latitude, int longitude, int timeZone,
                     int *hour, int *min)
{ 
  SUNTIME_FLOAT sunrise, sunset;                                         
  int iSunriseHour, iSunriseMin, iSunsetHour, iSunsetMin;
  
  if (suntime(day, month, year,
              90, 50, 
              abs(longitude / 100), abs(longitude % 100), (longitude < 0) ? 0 : 1,
              abs(latitude / 100), abs(latitude % 100), (latitude < 0) ? 1 : 0,
              timeZone, &sunrise, &sunset))
  {              
    timeadj(sunrise, &iSunriseHour, &iSunriseMin);
    timeadj(sunset, &iSunsetHour, &iSunsetMin);
    
    while (iSunriseHour > 12)
      iSunriseHour -= 12;
    while (iSunsetHour < 12)
      iSunsetHour += 12;
    
    if (occurrence == OCCURRENCE_SUNRISE)
    {
      *hour = iSunriseHour;
      *min = iSunriseMin;
    }  
    if (occurrence == OCCURRENCE_SUNSET)
    {
      *hour = iSunsetHour;
      *min = iSunsetMin;
    }  

    return 1;
  }
  else
    return 0;
}
  
int GetDegreesBelowHorizon(int day, int month, int year,
                           SUNTIME_FLOAT degreesBelowHorizon,
                           int latitude, int longitude, int timeZone,
                           int *hour, int *min)
{ 
  SUNTIME_FLOAT sunrise, sunset, sunset2, db;                                         
  int iSunsetHour, iSunsetMin, iSunset2Hour, iSunset2Min, deghour, degmin;
  int iS, iS2, iDiff;
  
  if (suntime(day, month, year,
              90, 50, 
              abs(longitude / 100), abs(longitude % 100), (longitude < 0) ? 0 : 1,
              abs(latitude / 100), abs(latitude % 100), (latitude < 0) ? 1 : 0,
              timeZone, &sunrise, &sunset))
  {              
    db = degreesBelowHorizon + 90.0;
    deghour = (int) db;
    db = db - (SUNTIME_FLOAT) deghour;
    db *= 60.0;
    degmin = (int) db;
    if (suntime(day, month, year,
                deghour, degmin, 
                abs(longitude / 100), abs(longitude % 100), (longitude < 0) ? 0 : 1,
                abs(latitude / 100), abs(latitude % 100), (latitude < 0) ? 1 : 0,
                timeZone, &sunrise, &sunset2))
    {              
      timeadj(sunset, &iSunsetHour, &iSunsetMin);
      timeadj(sunset2, &iSunset2Hour, &iSunset2Min);
    
      while (iSunsetHour < 12)
        iSunsetHour += 12;
      while (iSunset2Hour < 12)
        iSunset2Hour += 12;
      
      iS = iSunsetHour * 60 + iSunsetMin;  
      iS2 = iSunset2Hour * 60 + iSunset2Min;  
      iDiff = iS2-iS;
    
      *hour = iDiff / 60;
      *min = iDiff % 60;

      return 1;
    }
    else  
      return 0;
  }
  else
    return 0;
}
                        
