#include "config.h"
#include "date.h"
#include "hebrcal.h"
#include "holidays.h"
#include "settings.h"
#include "torahids.h"
#include "torahids2.h"
#include "torahnames.h"
#include "torahfull.h"

#define HebrcalGetHolidayCount    GetJewishHolidayCount
#define HebrcalGetHolidayDate     GetJewishHolidayDate

#define lstrcpy      strcpy
#define lstrcat      strcat
#define lstrcmp      strcmp

#define IDMONTH_ELUL        6
#define IDMONTH_TISHRI      7
#define IDMONTH_KISLEV      9

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

int TorahGetYearType(int uHebrewYear); /* from torah.c */
long DetermineBereshith(int uHebrewYear); /* from torah.c */

int GetTorahBase(int id);
int GetHaphtarahBase(int id);
void MyLoadString(int reserved, int id, char *lpStr, int cb);

#define PARASH_NEW_MOON_PROCLAMATION   0x0001
#define PARASH_HAPHTAROTH              0x0002
#define PARASH_1                       0x0004
#define PARASH_2                       0x0008
#define PARASH_3                       0x0010
#define PARASH_4                       0x0020
#define PARASH_5                       0x0040
#define PARASH_6                       0x0080
#define PARASH_7                       0x0100
#define PARASH_8                       0x0200
#define PARASH_8A                      0x0400
#define PARASH_9                       0x0800
#define PARASH_10                      0x1000
#define PARASH_11                      0x2000
#define PARASH_12                      0x4000

void ParashGetParamString(int uParam, char* string)
{
  int fFirst, fDone;

  lstrcpy(string, "");
  if (uParam & PARASH_NEW_MOON_PROCLAMATION)
    lstrcat(string, "*");

  fFirst = TRUE;
  fDone = FALSE;

  if (uParam & PARASH_HAPHTAROTH)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "0");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_1)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "1");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_2)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "2");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_3)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "3");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_4)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "4");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_5)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "5");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_6)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "6");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_7)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "7");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_8)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "8");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_8A)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "8a");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_9)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "9");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_10)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "10");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_11)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "11");
    fDone = TRUE;
    fFirst = FALSE;
  }
  if (uParam & PARASH_12)
  {
    if (fFirst)
      lstrcat(string, " (");
    else
      lstrcat(string, ", ");
    lstrcat(string, "12");
    fDone = TRUE;
    fFirst = FALSE;
  }

  if (fDone)
    lstrcat(string, ")");
}

void AddParsha(LPPARSHA lpParsha,
               char* lpText,
               int uORValue)
{
  if (lstrcmp(lpParsha->achParasha1, "") == 0)
  {
    lstrcpy(lpParsha->achParasha1, lpText);
    lpParsha->uParam1 |= uORValue;
  }
  else
  {
    if (lstrcmp(lpParsha->achParasha2, "") == 0)
    {
      lstrcpy(lpParsha->achParasha2, lpText);
      lpParsha->uParam2 |= uORValue;
    }
    else
    {
      lstrcpy(lpParsha->achParasha3, lpText);
      lpParsha->uParam3 |= uORValue;
    }
  }
}

#define ID_1        1
#define ID_2        2
#define ID_3        3
#define ID_4        4
#define ID_13       5 /* 1 in leap years, 3 in common years */
#define ID_14       6 
#define ID_34       7
#define ID_1234C    8 /* 1,2 in common years, 3,4 in leap years */
#define ID_1234L    9 /* 1,2 in leap years, 3,4 in common years */
#define ID_43      10 /* 3 in I, otherwise 4 */

#define MAXROSHHODESH 19

typedef struct tagROSHHODESH
{
  int uTorahReading;
  int uRoshHodeshID;
} ROSHHODESH;
 
ROSHHODESH RoshHodesh[MAXROSHHODESH] =
  {{ID_BERESHITH, ID_2},     /* 2 in B,G,J,N */
   {ID_NOAH,      ID_1},     /* 1 in D,F,I,M */
   {ID_TOLEDOTH,  ID_2},     /* 2 in D,F,I,M */
   {ID_MIKKETS,   ID_34},    /* 3,4 in C,E,K,L */
   {ID_VAERA,     ID_1},     /* 1 in A,H,N */
   {ID_MISHPATIM, ID_1234L}, /* 3 in B, 4 in G, 2 in H,N, 1 in J */
   {ID_TERUMAH,   ID_13},    /* 3 in F, 1 in M */
   {ID_VAYAKHEL,  ID_4},     /* 4 in B (V-P), 4 in J (V) */
   {ID_PEKUDE,    ID_34},    /* 4 in F,M, 3 in I */
   {ID_VAYIKRA,   ID_3},     /* 3 in D */
   {ID_SHEMINI,   ID_2},     /* 2 in D */
   {ID_TAZRIANG,  ID_1234C}, /* 1 in C,E (T-M), 4 in I (T), 3 in K,L (T) */
   {ID_AHAREMOTH, ID_2},     /* 2 in K,L */
   {ID_KEDOSHIM,  ID_1},     /* 1 in N */
   {ID_BEMIDBAR,  ID_2},     /* 2 in C,E,H,N */
   {ID_KORAH,     ID_14},    /* 1,4 in A,G,J */
   {ID_HUKATH,    ID_14},    /* 1,4 in M */
   {ID_MASEH,     ID_3},     /* 3 in B,F,I (Maseh) */
   {ID_REEH,      ID_34}};   /* 4 in B,I, 3,4 in D,K,L */

int LeapYear(int uHebrewYear)
{
  if ((((uHebrewYear*7)+1) % 19) < 7)
    return TRUE;
  else
    return FALSE;
}

void GetRoshHodesh(int uHebrewDay,
                   int uHebrewMonth,
                   int uHebrewYear,
                   LPPARSHA lpParsha,
                   int uID1,
                   int uID2,
                   int uID3)
{
  int f1or3, f2or4;
  int uRHReading;
  int i;

  /* Shabbat & Rosh Hodesh */
  f1or3 = FALSE;
  if ((uHebrewDay == 1) || (uHebrewDay == 30))
    f1or3 = TRUE;

  /* Shabbat preceding Rosh Hodesh */
  f2or4 = FALSE;
  if ((uHebrewDay == 29) || (uHebrewDay == 30))
    f2or4 = TRUE;

  if (!f1or3 && !f2or4)
    return;

  for (i = 0; i < MAXROSHHODESH; i++)
  {
    if ((uID1 == RoshHodesh[i].uTorahReading) ||
        (uID2 == RoshHodesh[i].uTorahReading))
    {
      uRHReading = 0;

      if (f1or3)
      {
        switch (RoshHodesh[i].uRoshHodeshID)
        {
          case ID_1: uRHReading |= PARASH_1; break;
          case ID_3: uRHReading |= PARASH_3; break;
          case ID_14: uRHReading |= PARASH_1; break;
          case ID_34: uRHReading |= PARASH_3; break;
          case ID_1234L:
          case ID_13:
            if (LeapYear(uHebrewYear))
              uRHReading |= PARASH_1;
            else
              uRHReading |= PARASH_3;
            break;
          case ID_1234C:
            if (LeapYear(uHebrewYear))
              uRHReading |= PARASH_3;
            else
              uRHReading |= PARASH_1;
            break;
        }
      }
      if (f2or4)
      {
        switch (RoshHodesh[i].uRoshHodeshID)
        {
          case ID_2: uRHReading |= PARASH_2; break;
          case ID_4: uRHReading |= PARASH_4; break;
          case ID_14: uRHReading |= PARASH_4; break;
          case ID_34: uRHReading |= PARASH_4; break;
          case ID_1234L:
            if (LeapYear(uHebrewYear))
              uRHReading |= PARASH_2;
            else
              uRHReading |= PARASH_4;
            break;
          case ID_1234C:
            if (LeapYear(uHebrewYear))
              uRHReading |= PARASH_4;
            else
              uRHReading |= PARASH_2;
            break;
        }
      }

      if (lstrcmp(lpParsha->achParasha3, "") != 0)
        lpParsha->uParam3 |= uRHReading;
      else
      {
        if (lstrcmp(lpParsha->achParasha2, "") != 0)
          lpParsha->uParam2 |= uRHReading;
        else
          lpParsha->uParam1 |= uRHReading;
      }
    } 
  }
}

#define Z_EXCEPT_1234        0x0001
#define Z_WITH_NITSABIM      0x0002

typedef struct tagZEROREC
{
  int uTorahReading;
  int uFlags;
} ZEROREC;

#define MAXZEROREC       9

ZEROREC ZeroRecs[MAXZEROREC] =
  {{ID_BALAK,       0},
   {ID_BEHUKKOTHAI, 0},
   {ID_HAGGADOL,    0},
   {ID_HAHODESH,    0},
   {ID_METSORANG,   Z_EXCEPT_1234},
   {ID_VAYELEH,     Z_WITH_NITSABIM},
   {ID_PARAH,       0},
   {ID_SHEKALIM,    0},
   {ID_ZAHOR,       0}};

typedef struct tagHOLIDAYREC
{
  int uHolidayID;
  int uCalfltID;
} HOLIDAYREC;

#define MAXHOLIDAYREC      37

HOLIDAYREC HolidayRecs[MAXHOLIDAYREC] =
  {{ID_ROSH_HASHANAH_I,      HOLIDAY_ROSH_HA_SHANA},   /*  1 */
   {ID_ROSH_HASHANAH_II,     HOLIDAY_ROSH_HA_SHANA_2}, /*  2 */
   {ID_FAST_OF_GEDALIAH,     HOLIDAY_TZOM_GEDALIAH},   /*  3 */
   {ID_YOM_KIPPUR,           HOLIDAY_YOM_KIPPUR},      /*  4 */
   {ID_SUCCOTH_I,            HOLIDAY_SUKKOT_1},        /*  5 */
   {ID_SUCCOTH_II,           HOLIDAY_SUKKOT_2},        /*  6 */
  {ID_SUCCOTH_III_NO_SHABBAT,HOLIDAY_SUKKOT_3},        /*  7 */
   {ID_SUCCOTH_IV,           HOLIDAY_SUKKOT_4},        /*  8 */
   {ID_SUCCOTH_V_NO_SHABBAT, HOLIDAY_SUKKOT_5},        /*  9 */
   {ID_SUCCOTH_VI_NO_SHABBAT,HOLIDAY_SUKKOT_6},        /* 10 */
   {ID_HOSHANAH_RABBAH,      HOLIDAY_SUKKOT_7},        /* 11 */
   {ID_SHEMINI_AZERETH,      HOLIDAY_SHEMINI_AZERETH}, /* 12 */
   {ID_SIMHATHTORAH,         HOLIDAY_SIMCHAT_TORAH},   /* 13 */
   {ID_FAST_OF_ESTHER,       HOLIDAY_TAANITESTHER},    /* 14 */
   {ID_PURIM,                HOLIDAY_PURIM},           /* 15 */
   {ID_FAST_OF_TEVET_10,     HOLIDAY_TZOM_TEVETH},     /* 16 */
   {ID_YOM_HAATZMAUT,        HOLIDAY_YOM_HAATZMAUT},   /* 17 */
   {ID_SHAVUOTH_I,           HOLIDAY_SHAVUOT},         /* 18 */
  {ID_SHAVUOTH_II_NO_SHABBAT,HOLIDAY_SHAVUOT_2},       /* 19 */
   {ID_FAST_OF_TAMMUZ_17,    HOLIDAY_TZOM_TAMMUZ},     /* 20 */
   {ID_FAST_OF_TISHA_BAV,    HOLIDAY_TISHA_BAV},       /* 21 */

   {ID_CHANUKKAH_I,          HOLIDAY_CHANUKKAH_1},     /* 22 */
   {ID_CHANUKKAH_II,         HOLIDAY_CHANUKKAH_2},     /* 23 */
   {ID_CHANUKKAH_III,        HOLIDAY_CHANUKKAH_3},     /* 24 */
   {ID_CHANUKKAH_IV,         HOLIDAY_CHANUKKAH_4},     /* 25 */
   {ID_CHANUKKAH_V,          HOLIDAY_CHANUKKAH_5},     /* 26 */
   {ID_CHANUKKAH_VI,         HOLIDAY_CHANUKKAH_6},     /* 27 */
   {ID_CHANUKKAH_VII,        HOLIDAY_CHANUKKAH_7},     /* 28 */
   {ID_CHANUKKAH_VIII,       HOLIDAY_CHANUKKAH_8},     /* 29 */
   {ID_PESAH_I,              HOLIDAY_PASSOVER_1},      /* 30 */
   {ID_PESAH_II,             HOLIDAY_PASSOVER_2},      /* 31 */
   {ID_PESAH_III,            HOLIDAY_PASSOVER_3},      /* 32 */
   {ID_PESAH_IV,             HOLIDAY_PASSOVER_4},      /* 33 */
   {ID_PESAH_V,              HOLIDAY_PASSOVER_5},      /* 34 */
   {ID_PESAH_VI,             HOLIDAY_PASSOVER_6},      /* 35 */
   {ID_PESAH_VII,            HOLIDAY_PASSOVER_7},      /* 36 */
   {ID_PESAH_VIII_NO_SHABBAT,HOLIDAY_PASSOVER_8}};     /* 37 */

void RetrieveHolidays(int uHebrewDay,
                      int uHebrewMonth,
                      int uHebrewYear,
                      LPPARSHA lpParsha,
                      long lHebrewDate,
                      long lChanukkahDate,
                      long lChanukkahDateEnd)
{
  calDate dateHoliday;
  long lHolidayDate;
  int uHolidayID;
  char achRoshHodeshString[256];
  int i;

  for (i = 0; i < MAXHOLIDAYREC; i++)
  {
    HebrcalGetHolidayDate(HolidayRecs[i].uCalfltID,
                          uHebrewYear,
                          &dateHoliday);
    if ((dateHoliday.day == uHebrewDay) &&
        (dateHoliday.month == uHebrewMonth) &&
        (dateHoliday.year == uHebrewYear))
    {
      uHolidayID = HolidayRecs[i].uHolidayID;
      switch (uHolidayID)
      {
        case ID_PESAH_IV:
          {
            lHolidayDate = HebrcalAbsoluteFromForeign(&dateHoliday);
            if (GetWeekday(lHolidayDate) == DAY_SUNDAY)
              uHolidayID = ID_PESAH_IV_SUNDAY;
            else
              uHolidayID = ID_PESAH_IV_NOT_SUNDAY;
          }
          break;

        case ID_PESAH_V:
          {
            lHolidayDate = HebrcalAbsoluteFromForeign(&dateHoliday);
            if (GetWeekday(lHolidayDate) == DAY_MONDAY)
              uHolidayID = ID_PESAH_V_MONDAY;
            else
              uHolidayID = ID_PESAH_V_NOT_MONDAY;
          }
          break;

        case ID_CHANUKKAH_VI:
          if ((uHebrewDay == 30) || (uHebrewDay == 1))
            uHolidayID = ID_CHANUKKAH_VI_ROSH_HODESH;
          break;

        case ID_CHANUKKAH_VII:
          if ((uHebrewDay == 30) || (uHebrewDay == 1))
            uHolidayID = ID_CHANUKKAH_VII_ROSH_HODESH;
          break;
      }
      MyLoadString(0,
                 GetTorahBase(uHolidayID)+uHolidayID,
                 lpParsha->achParasha1, 256);
      if ((lHebrewDate >= lChanukkahDate) &&
          (lHebrewDate <= lChanukkahDateEnd))
        lstrcpy(lpParsha->achHaphtarah1, "");
      else
        MyLoadString(0,
                   GetHaphtarahBase(uHolidayID)+uHolidayID,
                   lpParsha->achHaphtarah1, 256);

      /* On Simchat Torah three readings are read */
      if ((uHolidayID == ID_SIMHATHTORAH) && (diasporaGetFlag()))
      {
        MyLoadString(0,
                   GetTorahBase(ID_SIMHATHTORAH_2)+ID_SIMHATHTORAH_2,
                   lpParsha->achParasha2, 256);
        MyLoadString(0,
                   GetTorahBase(ID_SIMHATHTORAH_3)+ID_SIMHATHTORAH_3,
                   lpParsha->achParasha3, 256);
      }

      /* On Shemini Azereth three readings are read */
      if (uHolidayID == ID_SHEMINI_AZERETH)
      {
        MyLoadString(0,
                   GetTorahBase(ID_SHEMINI_AZERETH_1)+ID_SHEMINI_AZERETH_1,
                   lpParsha->achParasha1, 256);
        MyLoadString(0,
                   GetTorahBase(ID_SHEMINI_AZERETH_2)+ID_SHEMINI_AZERETH_2,
                   lpParsha->achParasha2, 256);
        MyLoadString(0,
                   GetTorahBase(ID_SHEMINI_AZERETH_3)+ID_SHEMINI_AZERETH_3,
                   lpParsha->achParasha3, 256);
      }
    }
  }

  if ((uHebrewDay == 30) || ((uHebrewDay == 1) &&
       (uHebrewMonth != IDMONTH_TISHRI)))
  {
    MyLoadString(0,
               GetTorahBase(ID_ROSH_HODESH)+ID_ROSH_HODESH,
               achRoshHodeshString, 256);

    if (lstrcmp(lpParsha->achParasha1, "") == 0)
      lstrcpy(lpParsha->achParasha1,
              achRoshHodeshString);
    else
      lstrcpy(lpParsha->achParasha2,
              achRoshHodeshString);
  }
}

int haphtarah_table[13] =
  {PARASH_11, PARASH_10, PARASH_9, PARASH_8, PARASH_8A,
   PARASH_7, PARASH_6, PARASH_5, PARASH_3, PARASH_4,
   PARASH_1, PARASH_2, PARASH_12};

#define IDS_TORAHBASE_FIRST        1
#define IDS_TORAHBASE_LAST         1000

#define IDS_HAPHTARAHBASE_FIRST    2000
#define IDS_HAPHTARAHBASE_LAST     3000

void MyLoadString(int reserved, int id, char* lpText, int cb)
{
  int diaspora;

  diaspora = diasporaGetFlag();
  if ((id >= IDS_TORAHBASE_FIRST) && (id <= IDS_TORAHBASE_LAST))
  {
    id -= IDS_TORAHBASE_FIRST;
    GetTorahReading(diaspora, id, lpText);
  }
  else
  {
    if ((id >= IDS_HAPHTARAHBASE_FIRST) && (id <= IDS_HAPHTARAHBASE_LAST))
    {
      id -= IDS_HAPHTARAHBASE_FIRST;
      GetHaphtarahReading(diaspora, id, lpText);
    }
  }

  if (strcmp(lpText, "(none)") == 0)
    strcpy(lpText, "");
}

int GetTorahBase(int id)
{
  return (IDS_TORAHBASE_FIRST);
}

int GetHaphtarahBase(int id)
{
  return (IDS_HAPHTARAHBASE_FIRST);
}   

int ParashGetReadings(int uHebrewDay,
                       int uHebrewMonth,
                       int uHebrewYear,
                       LPPARSHA lpParsha)
{
  return ParashGetReadingsWithIDs
               (uHebrewDay, uHebrewMonth, uHebrewYear,
                lpParsha, NULL);
}
                              
int ParashGetReadingsWithIDs(int uHebrewDay,
                              int uHebrewMonth,
                              int uHebrewYear,
                              LPPARSHA lpParsha,
                              LPPARSHAIDS lpParshaIDs)
{
  int uYearType, uReferenceYear, uID1, uID2, uID3;
  int uSuccothDayNo;
  calDate dateHebrew, dateChanukkah, dateRoshHaShanah;
  long lHebrewDate, lBereshithDate, lWeekNo;
  long lRoshHaShanahDate, lChanukkahDate;
  long lChanukkahShabbat, lChanukkahShabbat2;
  long lChanukkahDateEnd, lShuvaDate;
  int iMatoth, iMatoth2_ID, i;
  int fZero;
  char szText[256];

  int fHaphtarahFirst;
  char achHaphtarahString[256];
  int uHaphtarahID;
  
  if (lpParshaIDs)
    lpParshaIDs->fValid = FALSE;

  lstrcpy(lpParsha->achParasha1, "");
  lpParsha->uParam1 = 0;
  lstrcpy(lpParsha->achParasha2, "");
  lpParsha->uParam2 = 0;
  lstrcpy(lpParsha->achParasha3, "");
  lpParsha->uParam3 = 0;

  lstrcpy(lpParsha->achHaphtarah1, "");
  lstrcpy(lpParsha->achHaphtarah2, "");
  lstrcpy(lpParsha->achHaphtarah3, "");
  
  dateHebrew.day    = uHebrewDay;
  dateHebrew.month  = uHebrewMonth;
  dateHebrew.year   = uHebrewYear;
  lHebrewDate = HebrcalAbsoluteFromForeign(&dateHebrew);

  dateChanukkah.day    = 25;
  dateChanukkah.month  = IDMONTH_KISLEV;
  dateChanukkah.year   = uHebrewYear;
  lChanukkahDate = HebrcalAbsoluteFromForeign(&dateChanukkah);
  lChanukkahShabbat = lChanukkahDate;
  while (GetWeekday(lChanukkahShabbat) != DAY_SATURDAY)
    lChanukkahShabbat++;
  lChanukkahShabbat2 = lChanukkahShabbat+1;
  while (GetWeekday(lChanukkahShabbat2) != DAY_SATURDAY)
    lChanukkahShabbat2++;
  lChanukkahDateEnd = lChanukkahDate+7;

  if (GetWeekday(lHebrewDate) != DAY_SATURDAY)
  {
    RetrieveHolidays(uHebrewDay,
                     uHebrewMonth,
                     uHebrewYear,
                     lpParsha,
                     lHebrewDate,
                     lChanukkahDate,
                     lChanukkahDateEnd);
    return FALSE;
  }

  dateRoshHaShanah.day    = 1;
  dateRoshHaShanah.month  = IDMONTH_TISHRI;
  dateRoshHaShanah.year   = uHebrewYear;
  lRoshHaShanahDate = HebrcalAbsoluteFromForeign(&dateRoshHaShanah);    

  lBereshithDate = DetermineBereshith(uHebrewYear);
  if (lHebrewDate < lBereshithDate)
    uReferenceYear = uHebrewYear-1;
  else
    uReferenceYear = uHebrewYear;

  uYearType = TorahGetYearType(uReferenceYear);
  if (uYearType == YEARTYPE_INVALID)
    return FALSE;

  lBereshithDate = DetermineBereshith(uReferenceYear);
  lWeekNo = (lHebrewDate-lBereshithDate)/7;

  switch (uYearType)
  {
    case YEARTYPE_A:
      uID1 = uTorahReadingsA[lWeekNo].uID_1;
      uID2 = uTorahReadingsA[lWeekNo].uID_2;
      uID3 = uTorahReadingsA[lWeekNo].uID_3;
      iMatoth = 0;
      while (uTorahReadingsA[iMatoth].uID_1 != ID_MATOTH)
        iMatoth++;
      iMatoth2_ID = uTorahReadingsA[iMatoth].uID_2;
      break;

    case YEARTYPE_B:
      uID1 = uTorahReadingsB[lWeekNo].uID_1;
      uID2 = uTorahReadingsB[lWeekNo].uID_2;
      uID3 = uTorahReadingsB[lWeekNo].uID_3;
      iMatoth = 0;
      while (uTorahReadingsB[iMatoth].uID_1 != ID_MATOTH)
        iMatoth++;
      iMatoth2_ID = uTorahReadingsB[iMatoth].uID_2;
      break;

    case YEARTYPE_C:
      if (diasporaGetFlag())
      {
        uID1 = uTorahReadingsCDiaspora[lWeekNo].uID_1;
        uID2 = uTorahReadingsCDiaspora[lWeekNo].uID_2;
        uID3 = uTorahReadingsCDiaspora[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsCDiaspora[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsCDiaspora[iMatoth].uID_2;
      }
      else
      {
        uID1 = uTorahReadingsCIsrael[lWeekNo].uID_1;
        uID2 = uTorahReadingsCIsrael[lWeekNo].uID_2;
        uID3 = uTorahReadingsCIsrael[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsCIsrael[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsCIsrael[iMatoth].uID_2;
      }  
      break;

    case YEARTYPE_D:
      if (diasporaGetFlag())
      {
        uID1 = uTorahReadingsDDiaspora[lWeekNo].uID_1;
        uID2 = uTorahReadingsDDiaspora[lWeekNo].uID_2;
        uID3 = uTorahReadingsDDiaspora[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsDDiaspora[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsDDiaspora[iMatoth].uID_2;
      }
      else
      {
        uID1 = uTorahReadingsDIsrael[lWeekNo].uID_1;
        uID2 = uTorahReadingsDIsrael[lWeekNo].uID_2;
        uID3 = uTorahReadingsDIsrael[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsDIsrael[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsDIsrael[iMatoth].uID_2;
      }  
      break;

    case YEARTYPE_E:
      if (diasporaGetFlag())
      {
        uID1 = uTorahReadingsEDiaspora[lWeekNo].uID_1;
        uID2 = uTorahReadingsEDiaspora[lWeekNo].uID_2;
        uID3 = uTorahReadingsEDiaspora[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsEDiaspora[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsEDiaspora[iMatoth].uID_2;
      }
      else
      {
        uID1 = uTorahReadingsEIsrael[lWeekNo].uID_1;
        uID2 = uTorahReadingsEIsrael[lWeekNo].uID_2;
        uID3 = uTorahReadingsEIsrael[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsEIsrael[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsEIsrael[iMatoth].uID_2;
      }  
      break;

    case YEARTYPE_F:
      uID1 = uTorahReadingsF[lWeekNo].uID_1;
      uID2 = uTorahReadingsF[lWeekNo].uID_2;
      uID3 = uTorahReadingsF[lWeekNo].uID_3;
      iMatoth = 0;
      while (uTorahReadingsF[iMatoth].uID_1 != ID_MATOTH)
        iMatoth++;
      iMatoth2_ID = uTorahReadingsF[iMatoth].uID_2;
      break;

    case YEARTYPE_G:
      uID1 = uTorahReadingsG[lWeekNo].uID_1;
      uID2 = uTorahReadingsG[lWeekNo].uID_2;
      uID3 = uTorahReadingsG[lWeekNo].uID_3;
      iMatoth = 0;
      while (uTorahReadingsG[iMatoth].uID_1 != ID_MATOTH)
        iMatoth++;
      iMatoth2_ID = uTorahReadingsG[iMatoth].uID_2;
      break;

    case YEARTYPE_H:
      if (diasporaGetFlag())
      {
        uID1 = uTorahReadingsHDiaspora[lWeekNo].uID_1;
        uID2 = uTorahReadingsHDiaspora[lWeekNo].uID_2;
        uID3 = uTorahReadingsHDiaspora[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsHDiaspora[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsHDiaspora[iMatoth].uID_2;
      }
      else
      {
        uID1 = uTorahReadingsHIsrael[lWeekNo].uID_1;
        uID2 = uTorahReadingsHIsrael[lWeekNo].uID_2;
        uID3 = uTorahReadingsHIsrael[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsHIsrael[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsHIsrael[iMatoth].uID_2;
      }  
      break;

    case YEARTYPE_I:
      uID1 = uTorahReadingsI[lWeekNo].uID_1;
      uID2 = uTorahReadingsI[lWeekNo].uID_2;
      uID3 = uTorahReadingsI[lWeekNo].uID_3;
      iMatoth = 0;
      while (uTorahReadingsI[iMatoth].uID_1 != ID_MATOTH)
        iMatoth++;
      iMatoth2_ID = uTorahReadingsI[iMatoth].uID_2;
      break;

    case YEARTYPE_J:
      uID1 = uTorahReadingsJ[lWeekNo].uID_1;
      uID2 = uTorahReadingsJ[lWeekNo].uID_2;
      uID3 = uTorahReadingsJ[lWeekNo].uID_3;
      iMatoth = 0;
      while (uTorahReadingsJ[iMatoth].uID_1 != ID_MATOTH)
        iMatoth++;
      iMatoth2_ID = uTorahReadingsJ[iMatoth].uID_2;
      break;

    case YEARTYPE_K:
      if (diasporaGetFlag())
      {
        uID1 = uTorahReadingsKDiaspora[lWeekNo].uID_1;
        uID2 = uTorahReadingsKDiaspora[lWeekNo].uID_2;
        uID3 = uTorahReadingsKDiaspora[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsKDiaspora[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsKDiaspora[iMatoth].uID_2;
      }
      else
      {
        uID1 = uTorahReadingsKIsrael[lWeekNo].uID_1;
        uID2 = uTorahReadingsKIsrael[lWeekNo].uID_2;
        uID3 = uTorahReadingsKIsrael[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsKIsrael[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsKIsrael[iMatoth].uID_2;
      }  
      break;

    case YEARTYPE_L:
      if (diasporaGetFlag())
      {
        uID1 = uTorahReadingsLDiaspora[lWeekNo].uID_1;
        uID2 = uTorahReadingsLDiaspora[lWeekNo].uID_2;
        uID3 = uTorahReadingsLDiaspora[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsLDiaspora[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsLDiaspora[iMatoth].uID_2;
      }
      else
      {
        uID1 = uTorahReadingsLIsrael[lWeekNo].uID_1;
        uID2 = uTorahReadingsLIsrael[lWeekNo].uID_2;
        uID3 = uTorahReadingsLIsrael[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsLIsrael[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsLIsrael[iMatoth].uID_2;
      }  
      break;

    case YEARTYPE_M:
      uID1 = uTorahReadingsM[lWeekNo].uID_1;
      uID2 = uTorahReadingsM[lWeekNo].uID_2;
      uID3 = uTorahReadingsM[lWeekNo].uID_3;
      iMatoth = 0;
      while (uTorahReadingsM[iMatoth].uID_1 != ID_MATOTH)
        iMatoth++;
      iMatoth2_ID = uTorahReadingsM[iMatoth].uID_2;
      break;

    case YEARTYPE_N:
      if (diasporaGetFlag())
      {
        uID1 = uTorahReadingsNDiaspora[lWeekNo].uID_1;
        uID2 = uTorahReadingsNDiaspora[lWeekNo].uID_2;
        uID3 = uTorahReadingsNDiaspora[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsNDiaspora[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsNDiaspora[iMatoth].uID_2;
      }
      else
      {
        uID1 = uTorahReadingsNIsrael[lWeekNo].uID_1;
        uID2 = uTorahReadingsNIsrael[lWeekNo].uID_2;
        uID3 = uTorahReadingsNIsrael[lWeekNo].uID_3;
        iMatoth = 0;
        while (uTorahReadingsNIsrael[iMatoth].uID_1 != ID_MATOTH)
          iMatoth++;
        iMatoth2_ID = uTorahReadingsNIsrael[iMatoth].uID_2;
      }  
      break;
  }
  
  /* Shabbat readings for succoth */
  if (uID1 == ID_HOL_HAMOED_SUCCOTH)
  {
    uSuccothDayNo = uHebrewDay-15+1;
    switch (uSuccothDayNo)
    {
      case 3: uID1 = ID_SUCCOTH_III;
              break;
      case 5: uID1 = ID_SUCCOTH_V;
              break;
      case 6: uID1 = ID_SUCCOTH_VI;
              break;
    }
    MyLoadString(0,
               GetTorahBase(uID1)+uID1,
               lpParsha->achParasha1, 256);
    MyLoadString(0,
               GetHaphtarahBase(uID1)+uID1,
               lpParsha->achHaphtarah1, 256);
  }
  else
  {
    MyLoadString(0,
               GetTorahBase(uID1)+uID1,
               lpParsha->achParasha1, 256);
    MyLoadString(0,
               GetHaphtarahBase(uID1)+uID1,
               lpParsha->achHaphtarah1, 256);
  }
  
  /* Shabbat readings for Shemini Azereth */
  if (uID1 == ID_SHEMINI_AZERETH)
  {
    MyLoadString(0,
               GetTorahBase(ID_SHEMINI_AZERETH_1)+ID_SHEMINI_AZERETH_1,
               lpParsha->achParasha1, 256);
    MyLoadString(0,
               GetTorahBase(ID_SHEMINI_AZERETH_2)+ID_SHEMINI_AZERETH_2,
               lpParsha->achParasha2, 256);
    MyLoadString(0,
               GetTorahBase(ID_SHEMINI_AZERETH_3)+ID_SHEMINI_AZERETH_3,
               lpParsha->achParasha3, 256);
  }

  /* Shabbat readings for Chanukkah */
  if ((lHebrewDate >= lChanukkahDate) &&
      (lHebrewDate <= lChanukkahDateEnd))
  {
    switch (lHebrewDate-lChanukkahDate+1)
    {
      case 1: uID2 = ID_CHANUKKAH_I;    break;
      case 2: uID2 = ID_CHANUKKAH_II;   break;
      case 3: uID2 = ID_CHANUKKAH_III;  break;
      case 4: uID2 = ID_CHANUKKAH_IV;   break;
      case 5: uID2 = ID_CHANUKKAH_V;    break;
      case 6: uID2 = ID_CHANUKKAH_VI;   break;
      case 7: uID2 = ID_CHANUKKAH_VII;  break;
      case 8: uID2 = ID_CHANUKKAH_VIII; break;
    }
  }

  if (uID2 != ID_NULL)
  {
    MyLoadString(0,
               GetTorahBase(uID2)+uID2,
               lpParsha->achParasha2, 256);

    if ((lHebrewDate >= lChanukkahDate) &&
        (lHebrewDate <= lChanukkahDateEnd))
    {
      if (lHebrewDate != lChanukkahShabbat2)
        MyLoadString(0,
                   GetHaphtarahBase(uID2)+uID2,
                   lpParsha->achHaphtarah2, 256);

    }
    else
      MyLoadString(0,
                 GetHaphtarahBase(uID2)+uID2,
                 lpParsha->achHaphtarah2, 256);
  }

  /* Include reading for Rosh Hodesh falling on a shabbat */
  if ((uHebrewDay == 30) || ((uHebrewDay == 1) &&
       (uHebrewMonth != IDMONTH_TISHRI)))
  {
    if (uID1 == ID_NULL)
    {
      uID1 = ID_ROSH_HODESH_SHABBAT;
      MyLoadString(0,
                   GetTorahBase(uID1)+uID1,
                   lpParsha->achParasha1, 256);
    }  
    else
    {
      if (uID2 == ID_NULL)
      {
        uID2 = ID_ROSH_HODESH_SHABBAT;
        MyLoadString(0,
                     GetTorahBase(uID2)+uID2,
                     lpParsha->achParasha2, 256);
      }
      else
      {
        uID3 = ID_ROSH_HODESH_SHABBAT;
        MyLoadString(0,
                     GetTorahBase(uID3)+uID3,
                     lpParsha->achParasha3, 256);
      }  
    }
  }

  if (uID3 != ID_NULL)
  {
    MyLoadString(0,
               GetTorahBase(uID3)+uID3,
               lpParsha->achParasha3, 256);
    MyLoadString(0,
               GetHaphtarahBase(uID3)+uID3,
               lpParsha->achHaphtarah3, 256);
  }

  /* Determine haphtaroth readings */

  /* Proclamation of the New Moon (not of the */
  /* New Moon of Tishri) */
  if (uHebrewDay >= 23)
  {
    if (uHebrewMonth != IDMONTH_ELUL)
    {
      if ((uID1 == ID_VAYAKHEL) &&
          (uID2 == ID_PEKUDE))
        lpParsha->uParam2 |= PARASH_NEW_MOON_PROCLAMATION;
      else
        lpParsha->uParam1 |= PARASH_NEW_MOON_PROCLAMATION;
    }
  }

  /* Get 1,2,3,4 (Rosh Hodesh) */
  GetRoshHodesh(uHebrewDay,
                uHebrewMonth,
                uHebrewYear,
                lpParsha,
                uID1, uID2, uID3);

  /* Get 0 of second or third reading if it is available */
  if (uID2 != ID_NULL)
  {
    for (i = 0; i < MAXZEROREC; i++)
    {
      if (ZeroRecs[i].uTorahReading == uID2)
      {
        fZero = TRUE;

        if (ZeroRecs[i].uFlags & Z_EXCEPT_1234)
        {
          if ((lpParsha->uParam2 & PARASH_1) ||
              (lpParsha->uParam2 & PARASH_2) ||
              (lpParsha->uParam2 & PARASH_3) ||
              (lpParsha->uParam2 & PARASH_4))
            fZero = FALSE;
        }

        if (ZeroRecs[i].uFlags & Z_WITH_NITSABIM)
          if (uID1 != ID_NITSABIM)
            fZero = FALSE;
              
        if (fZero)
        {
          if (uID1 == ID_NITSABIM)
            lpParsha->uParam1 |= PARASH_HAPHTAROTH;
          else
            lpParsha->uParam2 |= PARASH_HAPHTAROTH;
        }
      }

      if (ZeroRecs[i].uTorahReading == uID3)
      {
        fZero = TRUE;

        if (ZeroRecs[i].uFlags & Z_EXCEPT_1234)
        {
          if ((lpParsha->uParam3 & PARASH_1) ||
              (lpParsha->uParam3 & PARASH_2) ||
              (lpParsha->uParam3 & PARASH_3) ||
              (lpParsha->uParam3 & PARASH_4))
            fZero = FALSE;
        }

        if (ZeroRecs[i].uFlags & Z_WITH_NITSABIM)
          if (uID1 != ID_NITSABIM)
            fZero = FALSE;
              
        if (fZero)
        {
          if (uID1 == ID_NITSABIM)
            lpParsha->uParam1 |= PARASH_HAPHTAROTH;
          else
            lpParsha->uParam3 |= PARASH_HAPHTAROTH;
        }
      }
    }
  }

  /* 5 and 6 for Chanukkah - the first shabbat */
  /* at Chanukkah is 5, the second shabbat (if */
  /* present) is 6 */
  if (lHebrewDate == lChanukkahShabbat)
    lpParsha->uParam1 |= PARASH_5;
  if ((lChanukkahShabbat2 <= lChanukkahDateEnd) &&
      (lHebrewDate == lChanukkahShabbat2))
    lpParsha->uParam1 |= PARASH_6;

  /* 6 is read if on a shabbat only Vayakhel or */
  /* Pekude is read */
  if ((uID1 == ID_VAYAKHEL) || (uID1 == ID_PEKUDE))
  {
    if (uID2 == ID_NULL)
      lpParsha->uParam1 |= PARASH_6;
  }

  /* If no 1,2,3,4 reading and no additional reading */
  /* is on Ahare, then 7 is read */
  if (uID1 == ID_AHAREMOTH)
  {
    if (uID2 == ID_NULL)
    {
      if ((!(lpParsha->uParam1 & PARASH_1)) &&
          (!(lpParsha->uParam1 & PARASH_2)) &&
          (!(lpParsha->uParam1 & PARASH_3)) &&
          (!(lpParsha->uParam1 & PARASH_4)))
        lpParsha->uParam1 |= PARASH_7;
    }
  }

  /* 8 is read on Kedoshim unless an 1,2,3,4 reading */
  /* is read */
  if (uID1 == ID_KEDOSHIM)
  {
    if ((!(lpParsha->uParam1 & PARASH_1)) &&
        (!(lpParsha->uParam1 & PARASH_2)) &&
        (!(lpParsha->uParam1 & PARASH_3)) &&
        (!(lpParsha->uParam1 & PARASH_4)))
      lpParsha->uParam1 |= PARASH_8;
  }
  else
  {
    if (uID2 == ID_KEDOSHIM)
    {
      if ((!(lpParsha->uParam2 & PARASH_1)) &&
          (!(lpParsha->uParam2 & PARASH_2)) &&
          (!(lpParsha->uParam2 & PARASH_3)) &&
          (!(lpParsha->uParam2 & PARASH_4)))
        lpParsha->uParam2 |= PARASH_8;
    }
  }

  /* 8a is read on Kedoshim in year type J */
  if (uYearType == YEARTYPE_J)
  {
    if (uID1 == ID_KEDOSHIM)
    {
      lpParsha->uParam1 &= (~PARASH_8);
      lpParsha->uParam1 |= PARASH_8A;
    }
  }

  /* If Matoth and Maseh are read on the same shabbat, */
  /* then on Pin'has 10 is read, otherwise on Matoth */
  /* 10 is read and on Pin'has 9 is read */
  if (iMatoth2_ID == ID_MASEH)
  {
    if (uID1 == ID_PINHAS)
      lpParsha->uParam1 |= PARASH_10;
  }
  else
  {
    if (uID1 == ID_MATOTH)
      lpParsha->uParam1 |= PARASH_10;
    if (uID1 == ID_PINHAS)
      lpParsha->uParam1 |= PARASH_9;
  }

  /* 11 is read at Mase (in year type A a 0 is added) */
  if (uID1 == ID_MASEH)
  {
    if (uYearType == YEARTYPE_A)
      lpParsha->uParam1 |= PARASH_HAPHTAROTH;
    lpParsha->uParam1 |= PARASH_11;
  }
  else
  {
    if (uID2 == ID_MASEH)
    {
      if (uYearType == YEARTYPE_A)
        lpParsha->uParam2 |= PARASH_HAPHTAROTH;
      lpParsha->uParam2 |= PARASH_11;
    }
    else
    {
      if (uID3 == ID_MASEH)
      {
        if (uYearType == YEARTYPE_A)
          lpParsha->uParam3 |= PARASH_HAPHTAROTH;
        lpParsha->uParam3 |= PARASH_11;
      }
    }
  }

  /* 12 (Shuva) - first shabbat of Tishri; */
  /* if Rosh Hashana is on a shabbat, it is */
  /* postponed by a week */
  if (GetWeekday(lRoshHaShanahDate) == DAY_SATURDAY)
    lShuvaDate = lRoshHaShanahDate+7;
  else
  {
    lShuvaDate = lRoshHaShanahDate;
    while (GetWeekday(lShuvaDate) != DAY_SATURDAY)
      lShuvaDate++;
  }
  if (lHebrewDate == lShuvaDate)
  {
    MyLoadString(0,
               GetTorahBase(ID_SHUVA)+ID_SHUVA,
               szText, 256);
    AddParsha(lpParsha, szText, PARASH_12);
  }

  /* 0, ... */
  fHaphtarahFirst = TRUE;
  if (lpParsha->uParam1 & PARASH_HAPHTAROTH)
  {
    if (fHaphtarahFirst)
    {
      lstrcpy(lpParsha->achHaphtarah1, "");
      lstrcpy(lpParsha->achHaphtarah2, "");
      lstrcpy(lpParsha->achHaphtarah3, "");
    }
    MyLoadString(0,
               GetHaphtarahBase(uID1)+uID1,
               achHaphtarahString, 256);
    if (lstrcmp(lpParsha->achHaphtarah1, "") == 0)
      lstrcpy(lpParsha->achHaphtarah1, achHaphtarahString);
    else
    {
      if (lstrcmp(lpParsha->achHaphtarah2, "") == 0)
        lstrcpy(lpParsha->achHaphtarah2, achHaphtarahString);
      else
        lstrcpy(lpParsha->achHaphtarah3, achHaphtarahString);
    }
    fHaphtarahFirst = FALSE;
  }
  if (lpParsha->uParam2 & PARASH_HAPHTAROTH)
  {
    if (fHaphtarahFirst)
    {
      lstrcpy(lpParsha->achHaphtarah1, "");
      lstrcpy(lpParsha->achHaphtarah2, "");
      lstrcpy(lpParsha->achHaphtarah3, "");
    }
    MyLoadString(0,
               GetHaphtarahBase(uID2)+uID2,
               achHaphtarahString, 256);
    if (lstrcmp(lpParsha->achHaphtarah1, "") == 0)
      lstrcpy(lpParsha->achHaphtarah1, achHaphtarahString);
    else
    {
      if (lstrcmp(lpParsha->achHaphtarah2, "") == 0)
        lstrcpy(lpParsha->achHaphtarah2, achHaphtarahString);
      else
        lstrcpy(lpParsha->achHaphtarah3, achHaphtarahString);
    }
    fHaphtarahFirst = FALSE;
  }
  if (lpParsha->uParam3 & PARASH_HAPHTAROTH)
  {
    if (fHaphtarahFirst)
    {
      lstrcpy(lpParsha->achHaphtarah1, "");
      lstrcpy(lpParsha->achHaphtarah2, "");
      lstrcpy(lpParsha->achHaphtarah3, "");
    }
    MyLoadString(0,
               GetHaphtarahBase(uID3)+uID3,
               achHaphtarahString, 256);
    if (lstrcmp(lpParsha->achHaphtarah1, "") == 0)
      lstrcpy(lpParsha->achHaphtarah1, achHaphtarahString);
    else
    {
      if (lstrcmp(lpParsha->achHaphtarah2, "") == 0)
        lstrcpy(lpParsha->achHaphtarah2, achHaphtarahString);
      else
        lstrcpy(lpParsha->achHaphtarah3, achHaphtarahString);
    }
    fHaphtarahFirst = FALSE;
  }

  /* ... 11, 10, 9, 8, 8a, 7, 6, 5, 3, 4, 1, 2, 12 */
  for (i = 0; i < 13; i++)
  {
    if ((lpParsha->uParam1 & haphtarah_table[i]) ||
        (lpParsha->uParam2 & haphtarah_table[i]) ||
        (lpParsha->uParam3 & haphtarah_table[i]))
    {
      if (fHaphtarahFirst)
      {
        lstrcpy(lpParsha->achHaphtarah1, "");
        lstrcpy(lpParsha->achHaphtarah2, "");
        lstrcpy(lpParsha->achHaphtarah3, "");
      }
      switch (haphtarah_table[i])
      {
        case PARASH_1:  uHaphtarahID = ID_SPECIAL_1;  break;
        case PARASH_2:  uHaphtarahID = ID_SPECIAL_2;  break;
        case PARASH_3:  uHaphtarahID = ID_SPECIAL_3;  break;
        case PARASH_4:  uHaphtarahID = ID_SPECIAL_4;  break;
        case PARASH_5:  uHaphtarahID = ID_SPECIAL_5;  break;
        case PARASH_6:  uHaphtarahID = ID_SPECIAL_6;  break;
        case PARASH_7:  uHaphtarahID = ID_SPECIAL_7;  break;
        case PARASH_8:  uHaphtarahID = ID_SPECIAL_8;  break;
        case PARASH_8A: uHaphtarahID = ID_SPECIAL_8A; break;
        case PARASH_9:  uHaphtarahID = ID_SPECIAL_9;  break;
        case PARASH_10: uHaphtarahID = ID_SPECIAL_10; break;
        case PARASH_11: uHaphtarahID = ID_SPECIAL_11; break;
        case PARASH_12: uHaphtarahID = ID_SPECIAL_12; break;
      }
      MyLoadString(0,
                 GetHaphtarahBase(uHaphtarahID)+uHaphtarahID,
                 achHaphtarahString, 256);
      if (lstrcmp(lpParsha->achHaphtarah1, "") == 0)
        lstrcpy(lpParsha->achHaphtarah1, achHaphtarahString);
      else
      {
        if (lstrcmp(lpParsha->achHaphtarah2, "") == 0)
          lstrcpy(lpParsha->achHaphtarah2, achHaphtarahString);
        else
          lstrcpy(lpParsha->achHaphtarah3, achHaphtarahString);
      }
      fHaphtarahFirst = FALSE;
    }
  }

  if (lpParshaIDs)
  {
    lpParshaIDs->fValid = TRUE;
    lpParshaIDs->uID1 = uID1;
    lpParshaIDs->uID2 = uID2;
    lpParshaIDs->uID3 = uID3;
  }  

  return TRUE;
}

void ParashGetQuickly(int day, int month, int year, /* Hebrew date */
                      char *torahReadings, char *haphtarahReadings)
{
  PARSHA parsha;

  ParashGetReadings(day, month, year, &parsha);

  strcpy(torahReadings, "");
  if (strcmp(parsha.achParasha1, "") != 0)
  {
    if (strcmp(torahReadings, "") != 0)
      strcat(torahReadings, "; ");
    strcat(torahReadings, parsha.achParasha1);
  }
  if (strcmp(parsha.achParasha2, "") != 0)
  {
    if (strcmp(torahReadings, "") != 0)
      strcat(torahReadings, "; ");
    strcat(torahReadings, parsha.achParasha2);
  }
  if (strcmp(parsha.achParasha3, "") != 0)
  {
    if (strcmp(torahReadings, "") != 0)
      strcat(torahReadings, "; ");
    strcat(torahReadings, parsha.achParasha3);
  }

  strcpy(haphtarahReadings, "");
  if (strcmp(parsha.achHaphtarah1, "") != 0)
  {
    if (strcmp(haphtarahReadings, "") != 0)
      strcat(haphtarahReadings, "; ");
    strcat(haphtarahReadings, parsha.achHaphtarah1);
  }
  if (strcmp(parsha.achHaphtarah2, "") != 0)
  {
    if (strcmp(haphtarahReadings, "") != 0)
      strcat(haphtarahReadings, "; ");
    strcat(haphtarahReadings, parsha.achHaphtarah2);
  }
  if (strcmp(parsha.achHaphtarah3, "") != 0)
  {
    if (strcmp(haphtarahReadings, "") != 0)
      strcat(haphtarahReadings, "; ");
    strcat(haphtarahReadings, parsha.achHaphtarah3);
  }
}

