#include "config.h"
#include "torahids.h"
#include "torahids2.h"
#include "torahnames.h"

#define ID_BOGUS        12345

typedef struct tagReadingRec
{
  int id;
  char *name;
} ReadingRec;

ReadingRec ReadingTorahIsrael[];
ReadingRec ReadingHaphtarahIsrael[];
ReadingRec ReadingTorahDiaspora[];
ReadingRec ReadingHaphtarahDiaspora[];

void LoadTorahString(ReadingRec *pReadingRec, int id, char *returnName)
{
  ReadingRec *lpLauf;

  strcpy(returnName, "");
  lpLauf = pReadingRec;
  while (lpLauf->id != ID_BOGUS)
  {
    if (lpLauf->id == id)
    {
      strcpy(returnName, lpLauf->name);
      break;
    }
    lpLauf++;
  }
}

void GetTorahReading(int diaspora, int id, char *returnString)
{
  if (diaspora)
    LoadTorahString(&ReadingTorahDiaspora[0], id, returnString);
  else
  {
    LoadTorahString(&ReadingTorahIsrael[0], id, returnString);
    if (strcmp(returnString, "") == 0)
      LoadTorahString(&ReadingTorahDiaspora[0], id, returnString);
  }

  if (strcmp(returnString, "(none)") == 0)
    strcpy(returnString, "");
}

void GetHaphtarahReading(int diaspora, int id, char *returnString)
{
  if (diaspora)
    LoadTorahString(&ReadingHaphtarahDiaspora[0], id, returnString);
  else
  {
    LoadTorahString(&ReadingHaphtarahIsrael[0], id, returnString);
    if (strcmp(returnString, "") == 0)
      LoadTorahString(&ReadingHaphtarahDiaspora[0], id, returnString);
  }

  if (strcmp(returnString, "(none)") == 0)
    strcpy(returnString, "");
}

/* Torah readings, Diaspora */

ReadingRec ReadingTorahDiaspora[] = {
  {ID_BERESHITH, "Bereshith, Genesis 1:1 - 6:8"},
  {ID_NOAH, "Noah, Genesis 6:9 - 11:32"},
  {ID_LEHLEHA, "Le'h Leha, Genesis 12:1 - 17:27"},
  {ID_VAYERA, "Vayera, Genesis 18:1 - 22:24"},
  {ID_HAYESARAH, "Haye Sarah, Genesis 23:1 - 25:18"},
  {ID_TOLEDOTH, "Toledoth, Genesis 25:19 - 28:9"},
  {ID_VAYETSE, "Vayetse, Genesis 28:10 - 32:3"},
  {ID_VAYISHLAH, "Vayishlah, Genesis 32:4 - 36:43"},
  {ID_VAYESHEB, "Vayesheb, Genesis 37:1 - 40:23"},
  {ID_MIKKETS, "Mikkets, Genesis 41:1 - 44:17"},
  {ID_VAYIGGASH, "Vayiggash, Genesis 44:18 - 47:27"},
  {ID_VAYHEE, "Vayhee, Genesis 47:28-end"},
  {ID_SHEMOTH, "Shemoth, Exodus 1:1 - 6:1"},
  {ID_VAERA, "Vaera, Exodus 6:2 - 9:35"},
  {ID_BO, "Bo, Exodus 10:1 - 13:16"},
  {ID_BESHALLAH, "Beshallah, Shabbat Shirah, Exodus 13:17 - 17:16"},
  {ID_YITHRO, "Yithro, Exodus 18:1 - 20:23"},
  {ID_MISHPATIM, "Mishpatim, Exodus 21:1 - 24:18"},
  {ID_TERUMAH, "Terumah, Exodus 25:1 - 27:19"},
  {ID_TETSAVVEH, "Tetsavveh, Exodus 27:20 - 30:10"},
  {ID_KITISSA, "Ki Tissa, Exodus 30:11 - 34:35"},
  {ID_VAYAKHEL, "Vayakhel, Exodus 35:1 - 38:20"},
  {ID_PEKUDE, "Pekude, Exodus 38:21-end"},
  {ID_VAYIKRA, "Vayikra, Leviticus 1:1 - 5:26"},
  {ID_TSAV, "Tsav, Leviticus, 6:1 - 8:36"},
  {ID_SHEMINI, "Shemini, Leviticus 9:1 - 11:47"},
  {ID_TAZRIANG, "Tazria, Leviticus 12:1 - 13:59"},
  {ID_METSORANG, "Metsora, Leviticus 14:1 - 15:33"},
  {ID_AHAREMOTH, "Aharemoth, Leviticus 16:1 - 18:30"},
  {ID_KEDOSHIM, "Kedoshim, Leviticus 19:1 - 20:27"},
  {ID_EMOR, "Emor, Leviticus 21:1 - 24:23"},
  {ID_BEHAR, "Behar, Leviticus 25:1 - 26:2"},
  {ID_BEHUKKOTHAI, "Behukkothai, Leviticus 26:3-end"},
  {ID_BEMIDBAR, "Bemidbar, Numbers 1:1 - 4:20"},
  {ID_NASO, "Naso, Numbers 4:21 - 7:89"},
  {ID_BEHAALOTEHA, "Behaaloteha, Numbers 8:1 - 12:16"},
  {ID_SHELAHLEHA, "Shelah Leha, Numbers 13:1 - 15:41"},
  {ID_KORAH, "Korah, Numbers 16:1 - 18:32"},
  {ID_HUKATH, "Hukath, Numbers 19:1 - 22:1"},
  {ID_BALAK, "Balak, Numbers 22:2 - 25:9"},
  {ID_PINHAS, "Pinhas, Numbers 25:10 - 30:1"},
  {ID_MATOTH, "Matoth, Numbers 30:2 - 32:42"},
  {ID_MASEH, "Maseh, Numbers 33:1-end"},
  {ID_DEBARIM, "Debarim, Shabbat Hazon, Deuteronomy 1:1 - 3:22"},
  {ID_VAETHANAN, "Vaethanan, Shabbat Nahamu, Deuteronomy 3:23 - 7:11"},
  {ID_EKEB, "Ekeb, Deuteronomy 7:12 - 11:25"},
  {ID_REEH, "Reeh, Deuteronomy 11:26 - 16:17"},
  {ID_SHOFETIM, "Shofetim, Deuteronomy 16:18 - 21:9"},
  {ID_KITETSE, "Ki Tetse, Deuteronomy 21:10 - 25:19"},
  {ID_KITABO, "Ki Tabo, Deuteronomy 26:1 - 29:8"},
  {ID_NITSABIM, "Nitsabim, Deuteronomy 29:9 - 30:20"},
  {ID_VAYELEH, "Vayeleh, Deuteronomy 31:1-30"},
  {ID_HAAZINU, "Haazinu, Deuteronomy 32:1-52"},
  {ID_SIMHATHTORAH, "Simkhat Tora, Deuteronomy 33:1-end"},
  {ID_SIMHATHTORAH_2, "Genesis 1:1 - 2:3"},
  {ID_SIMHATHTORAH_3, "Numbers 29:35 - 30:1"},

  {ID_ROSH_HODESH_SHABBAT, "Rosh Khodesh, Numbers 28:9-15"},
  {ID_SHEKALIM, "Shekalim, Exodus 30:11-16"},
  {ID_ZAHOR, "Za'hor, Deuteronomy 25:17-19"},
  {ID_PARAH, "Parah, Numbers 19:1-22"},
  {ID_HAHODESH, "Hahodesh, Exodus 12:1-20"},
  {ID_HAGGADOL, "Shabbat Haggadol"},
  {ID_SHUVA, "Shabbat Shuva"},

  {ID_ROSH_HASHANAH_I, "Rosh HaShanah I, Genesis 21:1-34; Numbers 29:1-6"},
  {ID_ROSH_HASHANAH_II, "Rosh HaShanah II, Genesis 22:1-24; Numbers 29:1-6"},
  {ID_FAST_OF_GEDALIAH, "Tsom Gedalya, Morning: Exodus 32:11-14 and Exodus 34:1-10; Afternoon: Exodus 32:11-14 and Exodus 34:1-10"},
  {ID_YOM_KIPPUR, "Yom Kippur, Morning: Leviticus 16:1-34 and Numbers 29:7-11; Afternoon: Leviticus, 18:1-30"},
  {ID_SUCCOTH_I, "Sukkot I, Leviticus 22:26 - 23:44; Numbers 29:12-16"},
  {ID_SUCCOTH_II, "Sukkot II, Leviticus 22:26 - 23:44; Numbers 29:12-16"},
  {ID_SUCCOTH_III_NO_SHABBAT, "Sukkot III, Numbers 29:17-25"},
  {ID_SUCCOTH_III, "Sukkot III, Exodus 33:12 - 34:26; Numbers 29:17-25"},
  {ID_SUCCOTH_IV, "Sukkot IV, Numbers 29:20-28"},
  {ID_SUCCOTH_V_NO_SHABBAT, "Sukkot V, Numbers 29:23-31"},
  {ID_SUCCOTH_V, "Sukkot V, Exodus 33:12 - 34:26; Numbers 29:23-31"},
  {ID_SUCCOTH_VI_NO_SHABBAT, "Sukkot VI, Numbers 29:26-34"},
  {ID_SUCCOTH_VI, "Sukkot VI, Exodus 33:12 - 34:26; Numbers 29:23-31"},
  {ID_HOSHANAH_RABBAH, "Hoshana Rabba, Numbers 29:26-34"},
  {ID_HOL_HAMOED_SUCCOTH, "Khol HaMoed Sukkot"},
  {ID_SHEMINI_AZERETH_1, "Shemini Atseret, Deuteronomy 14:22 - 16:17; Numbers 29:35 - 30:1"},
  {ID_SHEMINI_AZERETH_2, ""},
  {ID_SHEMINI_AZERETH_3, ""},
  {ID_FAST_OF_ESTHER, "Taanit Ester, Morning: Exodus 32:11-14 and Exodus 34:1-10; Afternoon: Exodus 32:11-14 and Exodus 34:1-10"},
  {ID_PURIM, "Purim, Exodus 17:8-16"},
  {ID_FAST_OF_TEVET_10, "Tsom Tevet, Morning: Exodus 32:11-14 and 34:1-10; Afternoon: Exodus 32:11-14 and Exodus 34:1-10"},
  {ID_PESAH_I, "Pesakh I, Exodus 12:21-51; Numbers 28:16-25"},
  {ID_HOL_HAMOED_PESAH, "Khol HaMoed Pesakh, Exodus 33:12 - 34:26; Numbers 28:19-25"}, /* on a shabbat */
  {ID_PESAH_VII, "Pesakh VII, Exodus 13:17 - 15:26; Numbers 28:19-25"},
  {ID_PESAH_VIII, "Pesakh VIII, Deuteronomy 14:22 - 16:17; Numbers 28:19-25"},
  {ID_PESAH_VIII_NO_SHABBAT, "Pesakh VIII, Deuteronomy 15:19 - 16:17; Numbers 28:19-25"},
  {ID_SHAVUOTH_I, "Shavuot I, Exodus 19:1 - 20:23; Numbers 28-26, 31"},
  {ID_SHAVUOTH_II_NO_SHABBAT, "Shavuot II, Deuteronomy 15:19 - 16:17, Numbers 28:26-31"},
  {ID_SHAVUOTH_II, "Shavuot II, Deuteronomy 14:22 - 16:17; Numbers 28:26-31"},
  {ID_YOM_HAATZMAUT, "Yom HaAtsmaut, Deuteronomy 7:12 - 8:18"},
  {ID_FAST_OF_TAMMUZ_17, "Tsom Tammuz, Morning: Exodus, 32:11-14 and Exodus 34:1-10; Afternoon: Exodus 32:11-14 and Exodus 34:1-10"},
  {ID_FAST_OF_TISHA_BAV, "Tisha BeAv, Morning: Deuteronomy 4:25-40; Afternoon: Exodus 32:11-14 and Exodus 34:1-10"},
  {ID_CHANUKKAH_I, "Khanukka I, Numbers 7:1-17"},
  {ID_CHANUKKAH_II, "Khanukka II, Numbers 7:18-29"},
  {ID_CHANUKKAH_III, "Khanukka III, Numbers 7:24-35"},
  {ID_CHANUKKAH_IV, "Khanukka IV, Numbers 7:30-41"},
  {ID_CHANUKKAH_V, "Khanukka V, Numbers 7:36-47"},
  {ID_CHANUKKAH_VI, "Khanukka VI, Numbers 7:42-47"},
  {ID_CHANUKKAH_VI_ROSH_HODESH, "Khanukka VI, Numbers 7:42-47"},
  {ID_CHANUKKAH_VII, "Khanukka VII, Numbers 7:48-59"},
  {ID_CHANUKKAH_VII_ROSH_HODESH, "Khanukka VII, Numbers 7:48-53"},
  {ID_CHANUKKAH_VIII, "Khanukka VIII, Numbers 7:54 - 8:4"},
  {ID_SECOND_SHABBAT_CHANUKKAH, "Numbers 7:54 - 8:4"},
  {ID_ROSH_HODESH, "Rosh Khodesh, Numbers 28:1-15"},
  {ID_PESAH_II, "Pesakh II, Leviticus 22:26 - 23:44; Numbers 28:16-25"},
  {ID_PESAH_III, "Pesakh III, Exodus 13:1-16; Numbers 28:19-25"},
  {ID_PESAH_IV_NOT_SUNDAY, "Pesakh IV, Exodus 22:24 - 23:19; Numbers 28:19-25"},
  {ID_PESAH_IV_SUNDAY, "Pesakh IV, Exodus 13:1-16; Numbers 28:19-25"},
  {ID_PESAH_V_NOT_MONDAY, "Pesakh V, Exodus 34:1-26; Numbers 28:19-25"},
  {ID_PESAH_V_MONDAY, "Pesakh V, Exodus 22:24 - 23:19; Numbers 28:19-25"},
  {ID_PESAH_VI, "Pesakh VI, Numbers 9:1-14; 28:19-25"},

  {ID_BOGUS, ""}};

/* Haphtarah readings, Diaspora */

ReadingRec ReadingHaphtarahDiaspora[] = {
  {ID_BERESHITH, "Isaiah 42:5 - 43:10 (Sephardi ritual: Isaiah 42:5-21)"},
  {ID_NOAH, "Isaiah 54:1 - 55:5 (Sephardi ritual: Isaiah 54:1-10)"},
  {ID_LEHLEHA, "Isaiah 40:27 - 41:16"},
  {ID_VAYERA, "II Kings 4:1-37 (Sephardi ritual: II Kings 4:1-23)"},
  {ID_HAYESARAH, "I Kings 1:1-31"},
  {ID_TOLEDOTH, "Malachi 1:1 - 2:7"},
  {ID_VAYETSE, "Hosea 12:13 - 14:10 (Optional addition: Micah 7, 18) (Sephardi ritual: Hosea 11:7 - 12:12)"},
  {ID_VAYISHLAH, "Obadiah 1:1-21 (Optional substitution: Hosea 11:7 - 12:12)"},
  {ID_VAYESHEB, "Amos 2:6 - 3:8"},
  {ID_MIKKETS, "I Kings 3:15 - 4:1"},
  {ID_VAYIGGASH, "Ezekiel 37:15-28"},
  {ID_VAYHEE, "I Kings 2:1-12"},
  {ID_SHEMOTH, "Isaiah 27:6 - 28:13; 29:22-23 (Sephardi ritual: Jeremiah 1:1 - 2:3)"},
  {ID_VAERA, "Ezekiel 28:25 - 29:21"},
  {ID_BO, "Jeremiah 46:13-28"},
  {ID_BESHALLAH, "Judges 4:4 - 5:31 (Sephardi ritual: Judges 5:1-31)"},
  {ID_YITHRO, "Isaiah 6:1 - 7:6; 9:5-6 (Sephardi ritual: 6:1-13)"},
  {ID_MISHPATIM, "Jeremiah 34:8-22; 33:25-26"},
  {ID_TERUMAH, "I Kings 5:26 - 6:13"},
  {ID_TETSAVVEH, "Ezekiel 43:10-27"},
  {ID_KITISSA, "I Kings 18:1-39 (Sephardi ritual: I Kings 18:20-39)"},
  {ID_VAYAKHEL, "I Kings 7:40-50 (Sephardi ritual: I Kings 7:13-26)"},
  {ID_PEKUDE, "I Kings 7:51 - 8:21 (Sephardi: substitute I Kings 7:40-50"},
  {ID_VAYIKRA, "Isaiah 43:21 - 44:23"},
  {ID_TSAV, "Jeremiah 7:21 - 8:3; 9:22-23"},
  {ID_SHEMINI, "II Samuel 6:1 - 7:17 (Sephardi ritual: II Samuel 6:1-19)"},
  {ID_TAZRIANG, "II Kings 4:42 - 5:19"},
  {ID_METSORANG, "II Kings 7:3-20"},
  {ID_AHAREMOTH, "Ezekiel 22:1-19 (Sephardi ritual: Ezekiel 22:1-16)"},
  {ID_KEDOSHIM, "Amos 9:7-15 (Sephardi ritual: Ezekiel 20:2-20)"},
  {ID_EMOR, "Ezekiel 44:15-31"},
  {ID_BEHAR, "Jeremiah 32:6-27"},
  {ID_BEHUKKOTHAI, "Jeremiah 16:19 - 17:14"},
  {ID_BEMIDBAR, "Hosea 2:1-22"},
  {ID_NASO, "Judges 13:2-25"},
  {ID_BEHAALOTEHA, "Zechariah 2:14 - 4:7"},
  {ID_SHELAHLEHA, "Joshua 2:1-24"},
  {ID_KORAH, "I Samuel 11:14 - 12:22"},
  {ID_HUKATH, "Judges 11:1-33"},
  {ID_BALAK, "Micah 5:6 - 6:8"},
  {ID_PINHAS, "I Kings 18:46 - 19:21"},
  {ID_MATOTH, "Jeremiah 1:1- 2,3"},
  {ID_MASEH, "Jeremiah 2:4-28; 3, 4; 4:1-2 (Sephardi ritual: Jeremiah 2:4-28; 4:1-2)"},
  {ID_DEBARIM, "Isaiah 1:1-27"},
  {ID_VAETHANAN, "Isaiah 40:1-26"},
  {ID_EKEB, "Isaiah 49:14 - 51:3"},
  {ID_REEH, "Isaiah 54:11 - 55:5"},
  {ID_SHOFETIM, "Isaiah 51:12 - 52:12"},
  {ID_KITETSE, "Isaiah 54:1-10"},
  {ID_KITABO, "Isaiah 60:1-22"},
  {ID_NITSABIM, "Isaiah 61:10 - 63:9"},
  {ID_VAYELEH, "Isaiah 55:6 - 56:8"},
  {ID_HAAZINU, "II Samuel 22:1-51"},
  {ID_SIMHATHTORAH, "Joshua 1:1-18 (Sephardi ritual: Joshua 1:1-9)"},
  {ID_SIMHATHTORAH_2, ""}, /* (none) */
  {ID_SIMHATHTORAH_3, ""}, /* (none) */

  {ID_ROSH_HODESH_SHABBAT, ""}, /* (none) */
  {ID_SHEKALIM, "II Kings 12:1-17 (Sephardi ritual: II Kings 11:17 - 12:17)"},
  {ID_ZAHOR, "I Samuel 15:2-34 (Sephardi ritual: I Samuel 15:1-34)"},
  {ID_PARAH, "Ezekiel 36:16-38 (Sephardi ritual: Ezekiel 36:16-36)"},
  {ID_HAHODESH, "Ezekiel 45:16 - 46:18 (Sephardi ritual: Ezekiel 45:18 - 46:15)"},
  {ID_HAGGADOL, "Malachi 3:4-24"},
  {ID_SHUVA, "Hosea 14:2-10; Micah 7:18-20 or Hosea 14:2-10; Joel 2:15-27"},

  {ID_SPECIAL_1, "Isaiah 66:1-24"},
  {ID_SPECIAL_2, "I Samuel 20:18-42"},
  {ID_SPECIAL_3, "Isaiah 66, 1 & 23"}, /* add */
  {ID_SPECIAL_4, "I Samuel 20, 18 & 42"}, /* add */
  {ID_SPECIAL_5, "Zechariah 2:14 - 4:7"},
  {ID_SPECIAL_6, "I Kings 7:40-50 (Sephardi add I Kings 7:13-26)"},
  {ID_SPECIAL_7, "Amos 9:7-15 (Sephardi ritual: Ezekiel 22:1-16)"},
  {ID_SPECIAL_8, "Amos 9:7-15 (Sephardi ritual: Ezekiel 20:2-20)"},
  {ID_SPECIAL_8A, "Ezekiel 22:1-19 (Sephardi ritual: Ezekiel 20:2-20)"},
  {ID_SPECIAL_9, "I Kings 18:46 - 19:21"},
  {ID_SPECIAL_10, "Jeremiah 1:1 - 2:3"},
  {ID_SPECIAL_11, "Jeremiah 2:4 - 2:28; 3, 4 (Optional: 4:1-2 may replace or supplement 3, 4)"},
  {ID_SPECIAL_12, "Hosea 14:2-10; Micah 7:18-20 or Hosea 14:2-10; Joel 2:15-27"},

  {ID_ROSH_HASHANAH_I, "I Samuel 1:1 - 2:10"},
  {ID_ROSH_HASHANAH_II, "Jeremiah 31:1-19"},
  {ID_FAST_OF_GEDALIAH, "Afternoon: Isaiah 55:6 - 56:8 (Sephardi ritual: Hosea 14:2-10; Micah 7:18-20)"},
  {ID_YOM_KIPPUR, "Morning: Isaiah 57:14 - 58:14; Afternoon: Jonah 1:1 - 4:11; Micah 7:18-20"},
  {ID_SUCCOTH_I, "Zechariah 14:1-21"},
  {ID_SUCCOTH_II, "I Kings 8:2-21"},
  {ID_SUCCOTH_III_NO_SHABBAT, ""}, /* (none) */
  {ID_SUCCOTH_III, "Ezekiel 38:18 - 39:16"},
  {ID_SUCCOTH_IV, ""}, /* (none) */
  {ID_SUCCOTH_V_NO_SHABBAT, ""}, /* (none) */
  {ID_SUCCOTH_V, "Ezekiel 38:18 - 39:16"},
  {ID_SUCCOTH_VI_NO_SHABBAT, ""}, /* (none) */
  {ID_SUCCOTH_VI, "Ezekiel 38:18 - 39:16"},
  {ID_HOSHANAH_RABBAH, ""}, /* (none) */
  {ID_HOL_HAMOED_SUCCOTH, ""},
  {ID_SHEMINI_AZERETH, "I Kings 8:54-66"},
  {ID_FAST_OF_ESTHER, "Afternoon: Isaiah 55:6 - 56:8 (Sephardi ritual: Hosea 14:2-10; Micah 7:18-20)"},
  {ID_PURIM, ""}, /* (none) */
  {ID_FAST_OF_TEVET_10, "Afternoon: Isaiah 55:6 - 56:8 (Sephardi ritual: Hosea 14:2-10; Micah 7:18-20)"},
  {ID_PESAH_I, "Joshua 3:5-7; 5:2 - 6:1; 6, 27 (Sephardi ritual: Joshua 5:2 - 6:1)"},
  {ID_HOL_HAMOED_PESAH, "Ezekiel 36:37 - 37:14 (Sephardi ritual: Ezekiel 37:1-14)"}, /* on a shabbat */
  {ID_PESAH_VII, "II Samuel 22:1-51"},
  {ID_PESAH_VIII, "Isaiah 10:32 - 12:6"},
  {ID_PESAH_VIII_NO_SHABBAT, "Isaiah 10:32 - 12:6"},
  {ID_SHAVUOTH_I, "Ezekiel 1:1-28; 3, 12"},
  {ID_SHAVUOTH_II_NO_SHABBAT, "Habakkuk 3:1-19 (Sephardi ritual: Habakkuk 2:20 - 3:19)"},
  {ID_SHAVUOTH_II, "Habakkuk 3:1-19 (Sephardi ritual: Habakkuk 2:20 - 3:19)"},
  {ID_YOM_HAATZMAUT, "Isaiah 10:32 - 12:6"},
  {ID_FAST_OF_TAMMUZ_17, "Afternoon: Isaiah 55:6 - 56:8 (Sephardi ritual: Hosea 14:2-10; Micah 7:18-20)"},
  {ID_FAST_OF_TISHA_BAV, "Morning: Jeremiah 8:13 - 9:23; Afternoon: Isaiah 55:6 - 56:8 (Sephardi ritual: Hosea 14:2-10; Micah 7:18-20)"},
  {ID_CHANUKKAH_I, "Zechariah 2:14 - 4:7"},
  {ID_CHANUKKAH_II, "Zechariah 2:14 - 4:7"},
  {ID_CHANUKKAH_III, "Zechariah 2:14 - 4:7"},
  {ID_CHANUKKAH_IV, "Zechariah 2:14 - 4:7"},
  {ID_CHANUKKAH_V, "Zechariah 2:14 - 4:7"},
  {ID_CHANUKKAH_VI, "Zechariah 2:14 - 4:7"},
  {ID_CHANUKKAH_VI_ROSH_HODESH, "Zechariah 2:14 - 4:7"},
  {ID_CHANUKKAH_VII, "Zechariah 2:14 - 4:7"},
  {ID_CHANUKKAH_VII_ROSH_HODESH, "Zechariah 2:14 - 4:7"},
  {ID_CHANUKKAH_VIII, ""}, /* (none) */
  {ID_SECOND_SHABBAT_CHANUKKAH, "I Kings 7:40-50"},
  {ID_ROSH_HODESH, ""}, /* (none) */
  {ID_PESAH_II, "II Kings 23:1-9; 21-25"},
  {ID_PESAH_III, ""}, /* (none) */
  {ID_PESAH_IV_NOT_SUNDAY, ""}, /* (none) */
  {ID_PESAH_IV_SUNDAY, ""}, /* (none) */
  {ID_PESAH_V_NOT_MONDAY, ""}, /* (none) */
  {ID_PESAH_V_MONDAY, ""}, /* (none) */
  {ID_PESAH_VI, ""}, /* (none) */

  {ID_BOGUS, ""}};

/* Torah readings, Israel */

ReadingRec ReadingTorahIsrael[] = {
  {ID_PESAH_I, "Pesakh I, Exodus 12:21-51; Numbers 28:16-25"},
  {ID_PESAH_II, "Pesakh II, Leviticus 22:26 - 23:44; Numbers 28:19-25"},
  {ID_PESAH_III, "Pesakh III, Exodus 13:1-16; Numbers 28:19-25"},
  {ID_PESAH_IV_NOT_SUNDAY, "Pesakh IV, Exodus 22:24 - 23:19; Numbers 28:19-25"},
  {ID_PESAH_IV_SUNDAY, "Pesakh IV, Exodus 13:1-16; Numbers 28:19-25"},
  {ID_PESAH_V_NOT_MONDAY, "Pesakh V, Exodus 34:1-26; Numbers 28:19-25"},
  {ID_PESAH_V_MONDAY, "Pesakh V, Exodus 22:24 - 23:19; Numbers 28:19-25"},
  {ID_PESAH_VI, "Pesakh VI, Numbers 9:1-14; 28:19-25"},
  {ID_PESAH_VII, "Pesakh VII, Exodus 13:17 - 15:26; Numbers 28:19-25"},
  {ID_HOL_HAMOED_PESAH, "Khol HaMoed Pesakh, Exodus 33:12 - 34:26; Numbers 28:19-25"}, /* on a shabbat */
  {ID_PESAH_VIII, "(none)"},
  {ID_PESAH_VIII_NO_SHABBAT, "(none)"},
  {ID_SUCCOTH_I, "Sukkot I, Exodus 12:31-51; Numbers 29:12-16"},
  {ID_SUCCOTH_II, "Sukkot II, Numbers 29:17-19"},
  {ID_SUCCOTH_III_NO_SHABBAT, "Sukkot III, Numbers 29:20-22"},
  {ID_SUCCOTH_III, "Sukkot III, Exodus 12:31 - 51:26; Numbers 29:20-22"},
  {ID_SUCCOTH_IV, "Sukkot IV, Numbers 29:23-25"},
  {ID_SUCCOTH_V_NO_SHABBAT, "Sukkot V, Numbers 29:26-28"},
  {ID_SUCCOTH_V, "Sukkot V, Exodus 12:31-51; Numbers 29:26-28"},
  {ID_SUCCOTH_VI_NO_SHABBAT, "Sukkot VI, Numbers 29:29-31"},
  {ID_SUCCOTH_VI, "Sukkot VI, Exodus 12:31-51; Numbers 29:29-31"},
  {ID_HOSHANAH_RABBAH, "Hoshana Rabba, Numbers 29:32-34"},
  {ID_HOL_HAMOED_SUCCOTH, "Khol HaMoed Sukkot"},
  {ID_SHEMINI_AZERETH_1, "Shemini Atseret/Simkhat Tora"},
  {ID_SHEMINI_AZERETH_2, "Deuteronomy 33:1 - 34:12"},
  {ID_SHEMINI_AZERETH_3, "Genesis 1:1 - 2:3; Numbers 29:35 - 30:1"},
  {ID_SIMHATHTORAH, "(none)"},
  {ID_SIMHATHTORAH_2, "(none)"},
  {ID_SIMHATHTORAH_3, "(none)"},
  {ID_SHAVUOTH_I, "Shavuot, Exodus 19:1 - 20:23; Numbers 28-26, 31"},
  {ID_SHAVUOTH_II_NO_SHABBAT, "(none)"},
  {ID_SHAVUOTH_II, "(none)"},
  {ID_BOGUS, ""}};

/* Haphtarah readings, Israel */

ReadingRec ReadingHaphtarahIsrael[] = {
  {ID_PESAH_I, "Joshua 3:5-7; 5:2 - 6:1; 6, 27 (Sephardi ritual: Joshua 5:2 - 6:1)"},
  {ID_PESAH_II, "(none)"},
  {ID_PESAH_III, "(none)"}, /* (none) */
  {ID_PESAH_IV_NOT_SUNDAY, "(none)"}, /* (none) */
  {ID_PESAH_IV_SUNDAY, "(none)"}, /* (none) */
  {ID_PESAH_V_NOT_MONDAY, "(none)"}, /* (none) */
  {ID_PESAH_V_MONDAY, "(none)"}, /* (none) */
  {ID_PESAH_VI, "(none)"}, /* (none) */
  {ID_PESAH_VII, "II Samuel 22:1-51"},
  {ID_HOL_HAMOED_PESAH, "Ezekiel 36:37 - 37:14 (Sephardi ritual: Ezekiel 37:1-14)"}, /* on a shabbat */
  {ID_PESAH_VIII, "(none)"},
  {ID_PESAH_VIII_NO_SHABBAT, "(none)"},
  {ID_SHAVUOTH_II, "(none)"},
  {ID_SHAVUOTH_II_NO_SHABBAT, "(none)"},
  {ID_SUCCOTH_I, "Zechariah 14:1-21"},
  {ID_SUCCOTH_II, "(none)"},
  {ID_SUCCOTH_III_NO_SHABBAT, "(none)"}, /* (none) */
  {ID_SUCCOTH_III, "Ezekiel 38:18 - 39:16"},
  {ID_SUCCOTH_IV, "(none)"}, /* (none) */
  {ID_SUCCOTH_V_NO_SHABBAT, "(none)"}, /* (none) */
  {ID_SUCCOTH_V, "Ezekiel 38:18 - 39:16"},
  {ID_SUCCOTH_VI_NO_SHABBAT, "(none)"}, /* (none) */
  {ID_SUCCOTH_VI, "Ezekiel 38:18 - 39:16"},
  {ID_HOSHANAH_RABBAH, "(none)"}, /* (none) */
  {ID_HOL_HAMOED_SUCCOTH, ""},
  {ID_SHEMINI_AZERETH, "Joshua 1:1-18 (Sephardi ritual: Joshua 1:1-9)"},
  {ID_BOGUS, ""}};
