#define ZMANIM_INTERNAL
#include "config.h"
#include "calctime.h"
#include "gregor.h"
#include "settings.h"
#include "ini.h"
#include "zmanim.h"
#include "zmanimnames.h"

#define ZMANIM_NOTHING          "----"

void ZmanimFormatTime(int hour, int min, char *time)
{
  sprintf(time, "%02d:%02d", hour, min);
}

char ZmanimAlotHashachar[9][ZMANIMMAX] = {
  "AlotHashachar72", "72 min before sunr.",
            "sunrise - 72",
  "AlotHashachar16", "16.1 deg bel. horizon",
            "sunrise - degrees_below_horizon(16.1)",
  "", "", ""};

char ZmanimEarliestTallit[9][ZMANIMMAX] = {
  "EarliestTallit11", "11 deg bel. horizon",
           "sunrise - degrees_below_horizon(11)",
  "EarliestTallit10", "10.2 deg bel. horizon",
           "sunrise - degrees_below_horizon(10.2)",
  "", "", ""};

char ZmanimKriatShema[9][ZMANIMMAX] = {
  "KriatShemaGRO", "GR\"O", "sunrise + proportional_hour(3)",
  "KriatShemaMA", "M\"A", "sunrise + proportional_hour(3) - 36",
  "", "", ""};

char ZmanimTefila[9][ZMANIMMAX] = {
  "TefilaGRO", "GR\"O", "sunrise + proportional_hour(4)",
  "TefilaMA", "M\"A", "sunrise + proportional_hour(4) - 24",
  "", "", ""};

char ZmanimChatzot[9][ZMANIMMAX] = {
  "ChatzotGRO", "GR\"O", "sunrise + proportional_hour(6)",
  "ChatzotMA", "M\"A", "sunrise + proportional_hour(6)",
  "", "", ""};

char ZmanimMinchaGedola[9][ZMANIMMAX] = {
  "MinchaGedolaGRO", "GR\"O", "sunrise + proportional_hour(6.5)",
  "MinchaGedolaMA", "M\"A", "sunrise + proportional_hour(6.5) + 6",
  "", "", ""};

char ZmanimMinchaKtana[9][ZMANIMMAX] = {
  "MinchaKtanaGRO", "GR\"O", "sunrise + proportional_hour(9.5)",
  "MinchaKtanaMA", "M\"A", "sunrise + proportional_hour(9.5 + 42",
  "", "", ""};

char ZmanimPlagHaMincha[9][ZMANIMMAX] = {
  "PlagHaMinchaGRO", "GR\"O", "sunrise + proportional_hour(10.75)",
  "PlagHaMinchaMA", "M\"A", "sunrise + proportional_hour(10.75) + 57",
  "", "", ""};

char ZmanimTzaitHaChochavim[18][ZMANIMMAX] = {
  "Sunset72", "72 min after sunset",
            "sunset + 72",
  "Deg595", "5.95 deg bel. horizon",
            "sunset + degrees_below_horizon(5.95)",
  "Deg708", "7.08 deg bel. horizon",
            "sunset + degrees_below_horizon(7.08)",
  "Deg75", "7.5 deg bel. horizon",
           "sunset + degrees_below_horizon(7.5)",
  "Deg875", "8.75 deg bel. horizon",
            "sunset + degrees_below_horizon(8.75)",
  "", "", ""};

ZMANIMREC ZmanimRecs[] = {
  {ZMANIM_ALOTHASHACHAR, "",
   "AlotHaShachar", ZMANIM_GROUP,
                       (void *) &ZmanimAlotHashachar[0]},
  {ZMANIM_EARLIEST_TALLIT, "",
   "EarliestTallit", ZMANIM_GROUP,
                       (void *) &ZmanimEarliestTallit[0]},
  {ZMANIM_HANETZ_HACHAMA, "",
   "HaNetzHaChama", ZMANIM_CONST,
                       (void *) "sunrise"},
  {ZMANIM_KRIAT_SHEMA_1, ZMANIM_KRIAT_SHEMA_2,
   "KriatShema", ZMANIM_GROUP,
                       (void *) &ZmanimKriatShema[0]},
  {ZMANIM_TEFILA_1, ZMANIM_TEFILA_2,
   "Tefila", ZMANIM_GROUP,
                       (void *) &ZmanimTefila[0]},
  {ZMANIM_CHATZOT, "",
   "Chatzot", ZMANIM_GROUP,
                       (void *) &ZmanimChatzot[0]},
  {ZMANIM_MINCHA_GEDOLA, "",
   "MinchaGedola", ZMANIM_GROUP,
                       (void *) &ZmanimMinchaGedola[0]},
  {ZMANIM_MINCHA_KTANA, "",
   "MinchaKtana", ZMANIM_GROUP,
                       (void *) &ZmanimMinchaKtana[0]},
  {ZMANIM_PLAG_HAMINCHA, "",
   "PlagHaMincha", ZMANIM_GROUP,
                       (void *) &ZmanimPlagHaMincha[0]},
  {ZMANIM_SHKIA_HACHAMA, "",
   "ShkiaHaChama", ZMANIM_CONST,
                       (void *) "sunset"},
  {ZMANIM_TZAIT_HAKOCHAVIM, "",
   "TzaitHakochavim", ZMANIM_GROUP,
                       (void *) &ZmanimTzaitHaChochavim[0]},
  {"", "", "", ZMANIM_NULL, (void *) NULL}};

ZMANIM_EXTERN void GetZmanimForDay(long currentDate, char *dest)
{
  char cityFileName[256], city[256], method[256], key[256], groupName[256];
  char timeString[256], *currentGroup;
  int isDST, latitude, longitude, timeZone, iZmanim, timeValue;
  int hour, min, dummy;
  dstStruct dst;
  calDate dateCurrent;

  getCityFileName(cityFileName);
  getCity(city);
  isDST = getIsDST();
  getDSTstruct(&dst);

  if (CityGet(cityFileName, city, &latitude, &longitude, &timeZone))
  {
    GregorForeignFromAbsolute(currentDate, &dateCurrent);
    sprintf(dest, "Zmanim for %d.%d.%d (%s):\n",
            dateCurrent.day, dateCurrent.month, dateCurrent.year,
            city);
    iZmanim = 0;
    for (;;)
    {
      if (ZmanimRecs[iZmanim].iType == ZMANIM_NULL)
        break;
      strcat(dest, ZmanimRecs[iZmanim].achName1);
      if (strcmp(ZmanimRecs[iZmanim].achName2, "") != 0)
      {
        strcat(dest, " ");
        strcat(dest, ZmanimRecs[iZmanim].achName2);
      }
      strcat(dest, ": ");
      switch (ZmanimRecs[iZmanim].iType)
      {
        case ZMANIM_CONST:
          strcpy(method, (char *) ZmanimRecs[iZmanim].lpvData);
          break;

        case ZMANIM_GROUP:
          {
            strcpy(key, "Zmanim");
            strcat(key, ZmanimRecs[iZmanim].achInternalName);
            SettingsGet(key, groupName, "");
            currentGroup = (char *) ZmanimRecs[iZmanim].lpvData;
            strcpy(method, currentGroup + 2*ZMANIMMAX);
            while (strcmp(currentGroup, "") != 0)
            {
              if (strcmp(currentGroup, groupName) == 0)
              {
                currentGroup += 2*ZMANIMMAX;
                strcpy(method, currentGroup);
                break;
              }
              else
                currentGroup += 3*ZMANIMMAX;
            }
          }
          break;
      }
      timeValue = CalculateZmanim(latitude, longitude, timeZone,
                                  dateCurrent.day, dateCurrent.month,
                                  dateCurrent.year,
                                  method);
      if (timeValue != (-1))
      {
        hour = timeValue / 60;
        min = timeValue % 60;
        if (Daylight_Correction(dateCurrent.day, dateCurrent.month,
                                dateCurrent.year,
                                &hour, &dummy, isDST ? &dst : NULL))
        {
          ZmanimFormatTime(hour, min, timeString);
          strcat(dest, timeString);
        }
        else
          strcat(dest, ZMANIM_NOTHING);
      }
      else
        strcat(dest, ZMANIM_NOTHING);
      strcat(dest, "\n");
      iZmanim++;
    }
  }
}

